@@ -242,7 +242,32 @@ impl DataSourceArgsBuilder {
         self
     }
 
-    /// Set incremental state clear flag.
+    /// Set whether this data source wants to receive incremental state clear notifications.
+    ///
+    /// This controls the **policy** of *whether* the tracing service should send clear
+    /// notifications to this data source. This is separate from the [`Clear`] trait,
+    /// which defines *how* to clear when notifications are received.
+    ///
+    /// # Arguments
+    ///
+    /// * `true` - The service will send IPC notifications when incremental state should
+    ///   be cleared, and your [`Clear::clear()`] implementation will be called.
+    /// * `false` (default) - The service will not send clear notifications, avoiding
+    ///   unnecessary IPC overhead. Your [`Clear::clear()`] method will never be called.
+    ///
+    /// # When to set this to `true`
+    ///
+    /// Set this to `true` if your incremental state benefits from periodic clearing:
+    /// - You intern data (strings, events, etc.) that should be reset periodically
+    /// - You maintain caches or temporary state that grows over time
+    /// - You want to ensure consistent data across trace buffer wraps
+    ///
+    /// # When to leave this as `false`
+    ///
+    /// Leave as `false` (default) if:
+    /// - You don't use incremental state (just use the default `IncrementalState`)
+    /// - Your incremental state is small or doesn't accumulate data
+    /// - You want to avoid the IPC overhead of clear notifications
     #[must_use = "Builder methods return an updated builder; use the returned value or keep chaining."]
     pub fn handles_incremental_state_clear(
         mut self,
@@ -409,6 +434,56 @@ impl TraceContextBase {
     }
 }
 
+/// Trait for clearing incremental state without destroying and recreating it.
+///
+/// This trait defines the **mechanism** for how to clear incremental state when
+/// the tracing service requests it. All incremental state types must implement
+/// this trait.
+///
+/// **Note**: This trait defines *how* to clear, not *whether* to receive clear
+/// notifications. Use [`DataSourceArgsBuilder::handles_incremental_state_clear`]
+/// to control whether the service should send clear notifications at all.
+///
+/// # Default Implementation
+///
+/// The default implementation simply replaces the current instance with a new
+/// default instance (`*self = Self::default()`), which is equivalent to
+/// destroying and recreating the state.
+///
+/// # Custom Implementation
+///
+/// Override `clear()` to reuse allocated memory instead of reallocating, which
+/// can significantly improve performance for data sources with large incremental
+/// state (e.g., hash maps for interned data).
+///
+/// # Example
+///
+/// ```ignore
+/// struct MyIncrementalState {
+///     interned_strings: HashMap<String, u32>,
+/// }
+///
+/// impl Clear for MyIncrementalState {
+///     fn clear(&mut self) {
+///         // Reuses the HashMap's allocation instead of dropping and reallocating
+///         self.interned_strings.clear();
+///     }
+/// }
+/// ```
+pub trait Clear: Default {
+    /// Clears the incremental state.
+    ///
+    /// This method is called when the tracing service sends a clear notification
+    /// (only if you've set `handles_incremental_state_clear(true)`).
+    ///
+    /// The default implementation replaces the current instance with a new
+    /// default instance. Override this to reuse allocations for better performance.
+    fn clear(&mut self) {
+        // Replaces the current instance with a new default instance.
+        *self = Self::default();
+    }
+}
+
 /// Default incremental state struct used if not specified.
 pub struct IncrementalState {
     /// Set to true when incremental state has been cleared and not yet acknowledged by
@@ -422,14 +497,16 @@ impl Default for IncrementalState {
     }
 }
 
+impl Clear for IncrementalState {}
+
 /// Trace context struct passed to data source trace callbacks.
-pub struct TraceContext<'a, IncrT: Default = IncrementalState> {
+pub struct TraceContext<'a, IncrT: Default + Clear = IncrementalState> {
     base: TraceContextBase,
     pub(crate) impl_: *mut PerfettoDsImpl,
     pub(crate) _marker: PhantomData<&'a IncrT>,
 }
 
-impl<IncrT: Default> TraceContext<'_, IncrT> {
+impl<IncrT: Default + Clear> TraceContext<'_, IncrT> {
     /// Calls `cb` with the incremental state for the instance.
     pub fn with_incremental_state<F>(&mut self, mut cb: F)
     where
@@ -462,21 +539,21 @@ impl<IncrT: Default> TraceContext<'_, IncrT> {
     }
 }
 
-impl<IncrT: Default> std::ops::Deref for TraceContext<'_, IncrT> {
+impl<IncrT: Default + Clear> std::ops::Deref for TraceContext<'_, IncrT> {
     type Target = TraceContextBase;
     fn deref(&self) -> &Self::Target {
         &self.base
     }
 }
 
-impl<IncrT: Default> std::ops::DerefMut for TraceContext<'_, IncrT> {
+impl<IncrT: Default + Clear> std::ops::DerefMut for TraceContext<'_, IncrT> {
     fn deref_mut(&mut self) -> &mut Self::Target {
         &mut self.base
     }
 }
 
 /// Data source struct.
-pub struct DataSource<'a: 'static, IncrT: Default = IncrementalState> {
+pub struct DataSource<'a: 'static, IncrT: Default + Clear = IncrementalState> {
     enabled: *mut bool,
     impl_: *mut PerfettoDsImpl,
     callbacks: Mutex<Option<Box<DsCallbacks>>>,
@@ -576,7 +653,7 @@ unsafe extern "C" fn on_flush_callback_trampoline(
     }
 }
 
-unsafe extern "C" fn on_create_incr_trampoline<IncrT: Default>(
+unsafe extern "C" fn on_create_incr_trampoline<IncrT: Default + Clear>(
     _ds: *mut PerfettoDsImpl,
     _inst_id: PerfettoDsInstanceIndex,
     _tracer: *mut PerfettoDsTracerImpl,
@@ -586,14 +663,33 @@ unsafe extern "C" fn on_create_incr_trampoline<IncrT: Default>(
     Box::into_raw(boxed) as *mut c_void
 }
 
-unsafe extern "C" fn on_delete_incr_trampoline<IncrT: Default>(data: *mut c_void) {
+unsafe extern "C" fn on_delete_incr_trampoline<IncrT: Default + Clear>(data: *mut c_void) {
     // Reclaims the Box and calls drop.
     //
     // SAFETY: `data` must be a pointer to a boxed IncrT struct.
     unsafe { drop(Box::from_raw(data as *mut IncrT)) };
 }
 
-impl<'a: 'static, IncrT: Default> DataSource<'a, IncrT> {
+unsafe extern "C" fn on_clear_incr_trampoline<IncrT: Default + Clear>(
+    incremental_state: *mut c_void,
+    _user_arg: *mut c_void,
+) -> bool {
+    let result = std::panic::catch_unwind(|| {
+        // SAFETY: `incremental_state` must be a pointer to a valid IncrT instance.
+        let state: &mut IncrT = unsafe { &mut *(incremental_state as *mut IncrT) };
+        state.clear();
+        true
+    });
+    match result {
+        Ok(success) => success,
+        Err(err) => {
+            eprintln!("Fatal panic: {:?}", err);
+            std::process::abort();
+        }
+    }
+}
+
+impl<'a: 'static, IncrT: Default + Clear> DataSource<'a, IncrT> {
     /// Create new data source type with a non-default `IncrT` type.
     pub fn new_with_incremental_state_type() -> Self {
         Self::default()
@@ -634,6 +730,7 @@ impl<'a: 'static, IncrT: Default> DataSource<'a, IncrT> {
             PerfettoDsSetOnFlushCallback(ds_impl, Some(on_flush_callback_trampoline));
             PerfettoDsSetOnCreateIncr(ds_impl, Some(on_create_incr_trampoline::<IncrT>));
             PerfettoDsSetOnDeleteIncr(ds_impl, Some(on_delete_incr_trampoline::<IncrT>));
+            PerfettoDsSetOnClearIncr(ds_impl, Some(on_clear_incr_trampoline::<IncrT>));
             PerfettoDsSetCbUserArg(ds_impl, user_arg);
             PerfettoDsSetBufferExhaustedPolicy(
                 ds_impl,
@@ -713,7 +810,7 @@ impl<'a: 'static> DataSource<'a, IncrementalState> {
     }
 }
 
-impl<'a: 'static, IncrT: Default> Default for DataSource<'a, IncrT> {
+impl<'a: 'static, IncrT: Default + Clear> Default for DataSource<'a, IncrT> {
     fn default() -> Self {
         Self {
             // `perfetto_atomic_false` is a pointer to a primitive with layout that
@@ -727,10 +824,10 @@ impl<'a: 'static, IncrT: Default> Default for DataSource<'a, IncrT> {
 }
 
 /// SAFETY: Internal handle must be thread-safe.
-unsafe impl<'a: 'static, IncrT: Default> Send for DataSource<'a, IncrT> {}
+unsafe impl<'a: 'static, IncrT: Default + Clear> Send for DataSource<'a, IncrT> {}
 
 /// SAFETY: Internal handle must be thread-safe.
-unsafe impl<'a: 'static, IncrT: Default> Sync for DataSource<'a, IncrT> {}
+unsafe impl<'a: 'static, IncrT: Default + Clear> Sync for DataSource<'a, IncrT> {}
 
 #[cfg(test)]
 mod tests {
