@@ -22,12 +22,12 @@ from typing import Dict, List, Optional, NamedTuple
 from python.generators.sql_processing.docs_extractor import DocsExtractor
 from python.generators.sql_processing.utils import ObjKind
 from python.generators.sql_processing.utils import COLUMN_TYPES, MACRO_ARG_TYPES
-
 from python.generators.sql_processing.utils import ALLOWED_PREFIXES
 from python.generators.sql_processing.utils import OBJECT_NAME_ALLOWLIST
-
 from python.generators.sql_processing.utils import ANY_PATTERN
 from python.generators.sql_processing.utils import ARG_DEFINITION_PATTERN
+from python.generators.sql_processing.utils import CREATE_TABLE_ALLOWLIST
+from python.generators.sql_processing.utils import is_internal
 
 
 @dataclass
@@ -35,10 +35,8 @@ class DocParseOptions:
   # TODO(lalitm): set this to true and/or remove this check once Google3 modules
   # adhere to this check.
   enforce_every_column_set_is_documented: bool = False
-
-
-def _is_internal(name: str) -> bool:
-  return re.match(r'^_.*', name, re.IGNORECASE) is not None
+  # Include internal artifacts (those starting with _) in the output
+  include_internal: bool = False
 
 
 def _is_snake_case(s: str) -> bool:
@@ -55,6 +53,9 @@ def parse_comment(comment: str) -> str:
 
 def get_module_prefix_error(name: str, path: str, module: str) -> Optional[str]:
   """Returns error message if the name is not correct, None otherwise."""
+  # Internal artifacts (starting with _) don't need to follow naming conventions
+  if is_internal(name):
+    return None
   if module in ["common", "prelude", "deprecated"]:
     if name.startswith(module):
       return (f'Names of tables/views/functions in the "{module}" module '
@@ -81,6 +82,7 @@ class Arg(NamedTuple):
   type: str
   long_type: str
   description: str
+  is_variadic: bool = False
 
 
 class AbstractDocParser(ABC):
@@ -111,9 +113,10 @@ class AbstractDocParser(ABC):
     return self.name.strip()
 
   def _parse_desc_not_empty(self, desc: str):
-    if not desc:
+    # Internal artifacts don't need descriptions
+    if not desc and not is_internal(self.name):
       self._error('Description of the table/view/function/macro is missing')
-    return desc.strip()
+    return desc.strip() if desc else ""
 
   def _parse_columns(
       self,
@@ -121,13 +124,17 @@ class AbstractDocParser(ABC):
       kind: ObjKind,
   ) -> Dict[str, Arg]:
     columns = self._parse_args_definition(schema) if schema else {}
-    if not schema and self.options.enforce_every_column_set_is_documented:
+
+    # Internal artifacts don't need column documentation
+    is_internal_artifact = is_internal(self.name)
+
+    if not schema and self.options.enforce_every_column_set_is_documented and not is_internal_artifact:
       self._error(
           'Description of the columns of table/view/function is missing')
       return columns
 
     for column_name, properties in columns.items():
-      if not properties.description:
+      if not properties.description and not is_internal_artifact:
         self._error(
             f'Column "{column_name}" is missing a description. Please add a '
             'comment in front of the column definition')
@@ -152,8 +159,11 @@ class AbstractDocParser(ABC):
   def _parse_args(self, sql_args_str: str, kind: ObjKind) -> Dict[str, Arg]:
     args = self._parse_args_definition(sql_args_str)
 
+    # Internal artifacts don't need arg documentation
+    is_internal_artifact = is_internal(self.name)
+
     for arg in args:
-      if not args[arg].description:
+      if not args[arg].description and not is_internal_artifact:
         self._error(f'Arg "{arg}" is missing a description. '
                     'Please add a comment in front of the arg definition.')
 
@@ -173,9 +183,11 @@ class AbstractDocParser(ABC):
 
   # Parse function argument definition list or a table schema, e.g.
   # arg1 INT, arg2 STRING, including their comments.
+  # Supports variadic arguments with TYPE... syntax.
   def _parse_args_definition(self, args_str: str) -> Dict[str, Arg]:
     result = {}
     remaining_args = args_str.strip()
+    found_variadic = False
     while remaining_args:
       m = re.match(fr'^{ARG_DEFINITION_PATTERN}({ANY_PATTERN})', remaining_args)
       if not m:
@@ -188,19 +200,34 @@ class AbstractDocParser(ABC):
       name = groups[-3]
       type = groups[-2]
 
+      # Check for variadic syntax (TYPE...)
+      is_variadic = type.endswith('...')
+      if is_variadic:
+        if found_variadic:
+          self._error(
+              'Only one variadic argument is allowed and it must be last')
+          return result
+        found_variadic = True
+        type = type[:-3]  # Strip the trailing ...
+
+      # Validate that non-variadic args don't come after variadic
+      if found_variadic and not is_variadic:
+        self._error('Variadic argument must be the last argument')
+        return result
+
       m = re.match(r'JOINID\(([_A-Za-z\.]*)\)', type)
       if m:
-        result[name] = Arg('JOINID', type, comment)
+        result[name] = Arg('JOINID', type, comment, is_variadic)
         remaining_args = groups[-1].lstrip().lstrip(',').lstrip()
         continue
 
       m = re.match(r'ID\(([_A-Za-z\.]*)\)', type)
       if m:
-        result[name] = Arg('ID', type, comment)
+        result[name] = Arg('ID', type, comment, is_variadic)
         remaining_args = groups[-1].lstrip().lstrip(',').lstrip()
         continue
 
-      result[name] = Arg(type, type, comment)
+      result[name] = Arg(type, type, comment, is_variadic)
       # Strip whitespace and comma and parse the next arg.
       remaining_args = groups[-1].lstrip().lstrip(',').lstrip()
 
@@ -236,6 +263,10 @@ class TableViewDocParser(AbstractDocParser):
 
     or_replace, perfetto_or_virtual, type, self.name, schema = doc.obj_match
 
+    # Skip internal artifacts early if not including them
+    if is_internal(self.name) and not self.options.include_internal:
+      return None
+
     if or_replace is not None:
       self._error(
           f'{type} "{self.name}": CREATE OR REPLACE is not allowed in stdlib '
@@ -243,10 +274,8 @@ class TableViewDocParser(AbstractDocParser):
           f'use CREATE instead.')
       return
 
-    if _is_internal(self.name):
-      return None
-
-    if type.lower() == "table" and not perfetto_or_virtual:
+    if (type.lower() == "table" and not perfetto_or_virtual and
+        self.name not in CREATE_TABLE_ALLOWLIST):
       self._error(
           f'{type} "{self.name}": Can only expose CREATE PERFETTO tables')
       return
@@ -289,8 +318,8 @@ class FunctionDocParser(AbstractDocParser):
           f'as standard library modules can only included once. Please just '
           f'use CREATE instead.')
 
-    # Ignore internal functions.
-    if _is_internal(self.name):
+    # Ignore internal functions unless explicitly requested.
+    if is_internal(self.name) and not self.options.include_internal:
       return None
 
     name = self._parse_name()
@@ -300,7 +329,8 @@ class FunctionDocParser(AbstractDocParser):
                   f' (should be {name.casefold()})')
 
     ret_desc = None if ret_comment is None else parse_comment(ret_comment)
-    if not ret_desc:
+    # Internal artifacts don't need return descriptions
+    if not ret_desc and not is_internal(name):
       self._error(f'Function "{name}": return description is missing')
 
     return Function(
@@ -308,7 +338,7 @@ class FunctionDocParser(AbstractDocParser):
         desc=self._parse_desc_not_empty(doc.description),
         args=self._parse_args(args, ObjKind.function),
         return_type=ret_type,
-        return_desc=ret_desc,
+        return_desc=ret_desc if ret_desc else "",
     )
 
 
@@ -341,8 +371,8 @@ class TableFunctionDocParser(AbstractDocParser):
           f'use CREATE instead.')
       return
 
-    # Ignore internal functions.
-    if _is_internal(self.name):
+    # Ignore internal functions unless explicitly requested.
+    if is_internal(self.name) and not self.options.include_internal:
       return None
 
     name = self._parse_name()
@@ -390,8 +420,8 @@ class MacroDocParser(AbstractDocParser):
           f'as standard library modules can only included once. Please just '
           f'use CREATE instead.')
 
-    # Ignore internal macros.
-    if _is_internal(self.name):
+    # Ignore internal macros unless explicitly requested.
+    if is_internal(self.name) and not self.options.include_internal:
       return None
 
     name = self._parse_name()
@@ -553,6 +583,8 @@ def parse_file(
                         [], [], [], module_doc)
 
   # Parse the extracted docs.
+  # Note: We collect errors from all parsers even if the result is None (filtered out),
+  # to ensure validation errors are reported for internal artifacts.
   errors: List[str] = []
   table_views: List[TableOrView] = []
   functions: List[Function] = []
