@@ -214,9 +214,16 @@ class TestApi(unittest.TestCase):
         self.assertEqual(row.dur, dur_result[num])
 
   def test_simple_resolver(self):
+    sample_path = example_android_trace_path()
     dur = [178646, 178646, 178646, 178646]
     source = ['generator', 'path', 'path_resolver', 'file']
-    expected = pd.DataFrame(list(zip(dur, source)), columns=['dur', 'source'])
+
+    # Only path and path_resolver will resolve to PathUriResolver so those will have the _path added
+    # to their metadata
+    path = [None, sample_path, sample_path, None]
+
+    expected = pd.DataFrame(
+        list(zip(dur, source, path)), columns=['dur', 'source', '_path'])
 
     with create_batch_tp(
         traces='simple:path={}'.format(example_android_trace_path())) as btp:
@@ -247,9 +254,11 @@ class TestApi(unittest.TestCase):
         float('nan'), 'recursive_path', 'recursive_path', 'recursive_obj',
         'recursive_obj'
     ]
+    sample_path = example_android_trace_path()
+    path = [None, None, sample_path, None, sample_path]
     expected = pd.DataFrame(
-        list(zip(dur, source, root_source)),
-        columns=['dur', 'source', 'root_source'])
+        list(zip(dur, source, root_source, path)),
+        columns=['dur', 'source', 'root_source', '_path'])
 
     uri = 'recursive:path={};skip_resolve_file=true'.format(
         example_android_trace_path())
@@ -533,7 +542,7 @@ class TestApi(unittest.TestCase):
   def test_metadata_from_path(self):
     # When loading a trace directly from a path, metadata should be empty
     with create_tp(trace=example_android_trace_path()) as tp:
-      self.assertEqual(tp.metadata, {})
+      self.assertEqual(tp.metadata, {"_path": example_android_trace_path()})
 
   def test_metadata_from_file(self):
     # When loading a trace from a file object, metadata should be empty
@@ -578,10 +587,12 @@ class TestApi(unittest.TestCase):
         bin_path=os.environ["SHELL_PATH"], resolver_registry=registry)
 
     with TraceProcessor(trace='metadata_test:', config=config) as tp:
-      self.assertEqual(tp.metadata, {
-          'test_key': 'test_value',
-          'trace_id': '12345'
-      })
+      self.assertEqual(
+          tp.metadata, {
+              'test_key': 'test_value',
+              'trace_id': '12345',
+              '_path': example_android_trace_path()
+          })
 
   def test_metadata_from_resolver_merged(self):
     # Test that metadata is merged when using nested resolvers
@@ -631,6 +642,7 @@ class TestApi(unittest.TestCase):
       expected_metadata = {
           'outer_key': 'outer_value',
           'inner_key': 'inner_value',
-          'shared_key': 'from_inner'
+          'shared_key': 'from_inner',
+          '_path': example_android_trace_path()
       }
       self.assertEqual(tp.metadata, expected_metadata)
