@@ -24,6 +24,10 @@ namespace perfetto::shlib {
 namespace {
 
 using perfetto::internal::TrackEventInternal;
+// All interned string messages for track events must have this field number
+// structure.
+static constexpr uint32_t kInternedStringIidFieldNumber = 1;
+static constexpr uint32_t kInternedStringNameFieldNumber = 2;
 
 protos::pbzero::TrackEvent::Type EventType(int32_t type) {
   using Type = protos::pbzero::TrackEvent::Type;
@@ -42,7 +46,8 @@ protos::pbzero::TrackEvent::Type EventType(int32_t type) {
 }
 
 // Appends the fields described by `fields` to `msg`.
-void AppendHlProtoFields(protozero::Message* msg,
+void AppendHlProtoFields(TrackEventIncrementalState* incr,
+                         protozero::Message* msg,
                          PerfettoTeHlProtoField* const* fields) {
   for (PerfettoTeHlProtoField* const* p = fields; *p != nullptr; p++) {
     switch ((*p)->type) {
@@ -51,6 +56,26 @@ void AppendHlProtoFields(protozero::Message* msg,
         msg->AppendString(field->header.id, field->str);
         break;
       }
+      case PERFETTO_TE_HL_PROTO_TYPE_CSTR_INTERNED: {
+        auto field = reinterpret_cast<PerfettoTeHlProtoFieldCstrInterned*>(*p);
+        PERFETTO_DCHECK(field->interned_type_id != 0);
+        if (field->interned_type_id) {
+          const char* str = field->str;
+          size_t len = strlen(field->str);
+          auto res = incr->iids.FindOrAssign(
+              static_cast<int32_t>(field->interned_type_id), str, len);
+          if (res.newly_assigned) {
+            auto* ser = incr->serialized_interned_data
+                            ->BeginNestedMessage<protozero::Message>(
+                                field->interned_type_id);
+            ser->AppendVarInt(kInternedStringIidFieldNumber, res.iid);
+            ser->AppendString(kInternedStringNameFieldNumber, field->str);
+          }
+          msg->AppendVarInt(field->header.id, res.iid);
+        }
+        // If interned_type_id is zero, this is a user error, we drop the packet
+        break;
+      }
       case PERFETTO_TE_HL_PROTO_TYPE_BYTES: {
         auto field = reinterpret_cast<PerfettoTeHlProtoFieldBytes*>(*p);
         msg->AppendBytes(field->header.id, field->buf, field->len);
@@ -60,7 +85,7 @@ void AppendHlProtoFields(protozero::Message* msg,
         auto field = reinterpret_cast<PerfettoTeHlProtoFieldNested*>(*p);
         auto* nested =
             msg->BeginNestedMessage<protozero::Message>(field->header.id);
-        AppendHlProtoFields(nested, field->fields);
+        AppendHlProtoFields(incr, nested, field->fields);
         break;
       }
       case PERFETTO_TE_HL_PROTO_TYPE_VARINT: {
@@ -248,7 +273,7 @@ void WriteTrackEvent(TrackEventIncrementalState* incr,
       const auto* fields =
           reinterpret_cast<const struct PerfettoTeHlExtraProtoFields&>(extra)
               .fields;
-      AppendHlProtoFields(event, fields);
+      AppendHlProtoFields(incr, event, fields);
     }
   }
 }
@@ -294,7 +319,7 @@ uint64_t EmitProtoTrack(uint64_t uuid,
     auto packet = trace_writer->NewTracePacket();
     auto* track_descriptor = packet->set_track_descriptor();
     track_descriptor->set_uuid(uuid);
-    AppendHlProtoFields(track_descriptor, fields);
+    AppendHlProtoFields(incr_state, track_descriptor, fields);
   }
   return uuid;
 }
@@ -310,7 +335,7 @@ uint64_t EmitProtoTrackWithParentUuid(
     auto* track_descriptor = packet->set_track_descriptor();
     track_descriptor->set_uuid(uuid);
     track_descriptor->set_parent_uuid(parent_uuid);
-    AppendHlProtoFields(track_descriptor, fields);
+    AppendHlProtoFields(incr_state, track_descriptor, fields);
   }
   return uuid;
 }
