@@ -346,7 +346,7 @@ UniquePid ProcessTracker::StartNewProcessInternal(
   if (timestamp) {
     prr.set_start_ts(*timestamp);
   }
-  prr.set_name(process_name);
+  UpdateProcessName(upid, process_name, ProcessNamePriority::kSystem);
 
   if (parent_upid) {
     prr.set_parent_upid(*parent_upid);
@@ -415,7 +415,7 @@ void ProcessTracker::SetProcessMetadata(UniquePid upid,
   auto& process_table = *context_->storage->mutable_process_table();
   auto prr = process_table[upid];
   StringId proc_name_id = context_->storage->InternString(name);
-  prr.set_name(proc_name_id);
+  UpdateProcessName(upid, proc_name_id, ProcessNamePriority::kSystem);
   prr.set_cmdline(context_->storage->InternString(cmdline));
 }
 
@@ -430,11 +430,19 @@ void ProcessTracker::SetProcessUid(UniquePid upid, uint32_t uid) {
   rr.set_android_user_id(uid / 100000);
 }
 
-void ProcessTracker::SetProcessNameIfUnset(UniquePid upid,
-                                           StringId process_name_id) {
+void ProcessTracker::UpdateProcessName(UniquePid upid,
+                                       StringId process_name_id,
+                                       ProcessNamePriority priority) {
+  if (process_name_id.is_null())
+    return;
+
   auto& pt = *context_->storage->mutable_process_table();
-  if (auto rr = pt[upid]; !rr.name().has_value()) {
-    rr.set_name(process_name_id);
+  if (PERFETTO_UNLIKELY(process_name_priorities_.size() <= upid)) {
+    process_name_priorities_.resize(upid + 1);
+  }
+  if (priority >= process_name_priorities_[upid]) {
+    pt[upid].set_name(process_name_id);
+    process_name_priorities_[upid] = priority;
   }
 }
 
@@ -462,7 +470,7 @@ void ProcessTracker::UpdateThreadNameAndMaybeProcessName(
   auto prr = pt[*opt_upid];
   if (prr.pid() == trr.tid()) {
     PERFETTO_DCHECK(trr.is_main_thread());
-    prr.set_name(thread_name);
+    UpdateProcessName(*opt_upid, thread_name, ProcessNamePriority::kSystem);
   }
 }
 
@@ -639,6 +647,13 @@ void ProcessTracker::SetMainThread(UniqueTid utid, bool is_main_thread) {
   trr.set_is_main_thread(is_main_thread);
 }
 
+void ProcessTracker::SetIdleThread(UniqueTid utid, bool is_idle) {
+  auto& thread_table = *context_->storage->mutable_thread_table();
+
+  auto trr = thread_table[utid];
+  trr.set_is_idle(is_idle);
+}
+
 void ProcessTracker::SetPidZeroIsUpidZeroIdleProcess() {
   // Create a mapping from (t|p)id 0 -> u(t|p)id for the idle process.
   tids_.Insert(0, std::vector<UniqueTid>{swapper_utid_});
@@ -658,13 +673,14 @@ ArgsTracker::BoundInserter ProcessTracker::AddArgsToThread(UniqueTid utid) {
   return args_tracker_.AddArgsToThread(utid);
 }
 
-void ProcessTracker::NotifyEndOfFile() {
+void ProcessTracker::OnEventsFullyExtracted() {
   args_tracker_.Flush();
   tids_.Clear();
   pids_.Clear();
   pending_assocs_.clear();
   pending_parent_assocs_.clear();
   thread_name_priorities_.clear();
+  process_name_priorities_.clear();
   trusted_pids_.clear();
   namespaced_threads_.clear();
   namespaced_processes_.clear();
