@@ -22,7 +22,6 @@
 #include <utility>
 
 #include "perfetto/base/logging.h"
-#include "perfetto/ext/base/murmur_hash.h"
 #include "src/trace_processor/importers/common/args_translation_table.h"
 #include "src/trace_processor/importers/common/import_logs_tracker.h"
 #include "src/trace_processor/importers/common/slice_tracker.h"
@@ -184,7 +183,6 @@ std::optional<SliceId> SliceTracker::StartSlice(
   std::optional<tables::SliceTable::RowReference> parent_ref =
       depth == 0 ? std::nullopt
                  : std::make_optional(stack.back().row.ToRowReference(slices));
-  int64_t parent_stack_id = parent_ref ? parent_ref->stack_id() : 0;
   std::optional<tables::SliceTable::Id> parent_id =
       parent_ref ? std::make_optional(parent_ref->id()) : std::nullopt;
 
@@ -205,8 +203,6 @@ std::optional<SliceId> SliceTracker::StartSlice(
   // Post fill all the relevant columns. All the other columns should have
   // been filled by the inserter.
   ref.set_depth(static_cast<uint32_t>(depth));
-  ref.set_parent_stack_id(parent_stack_id);
-  ref.set_stack_id(GetStackHash(stack));
   if (parent_id)
     ref.set_parent_id(*parent_id);
 
@@ -469,27 +465,6 @@ bool SliceTracker::MaybeCloseStack(int64_t new_ts,
   return true;
 }
 
-int64_t SliceTracker::GetStackHash(const SlicesStack& stack) {
-  PERFETTO_DCHECK(!stack.empty());
-
-  const auto& slices = context_->storage->slice_table();
-
-  base::MurmurHashCombiner hash;
-  for (const auto& i : stack) {
-    auto ref = i.row.ToRowReference(slices);
-    hash.Combine(ref.category());
-    hash.Combine(ref.name());
-  }
-
-  // For clients which don't have an integer type (i.e. Javascript), returning
-  // hashes which have the top 11 bits set leads to numbers which are
-  // unrepresenatble. This means that clients cannot filter using this number as
-  // it will be meaningless when passed back to us. For this reason, make sure
-  // that the hash is always less than 2^53 - 1.
-  constexpr uint64_t kSafeBitmask = (1ull << 53) - 1;
-  return static_cast<int64_t>(hash.digest() & kSafeBitmask);
-}
-
 void SliceTracker::StackPop(TrackId track_id) {
   auto& stack = stacks_[track_id].slice_stack;
   MaybeAddTranslatableArgs(stack.back());
