@@ -23,6 +23,7 @@
 
 #include "src/trace_processor/importers/common/args_tracker.h"
 #include "src/trace_processor/importers/common/args_translation_table.h"
+#include "src/trace_processor/importers/common/machine_tracker.h"
 #include "src/trace_processor/importers/common/slice_tracker.h"
 #include "src/trace_processor/importers/common/slice_translation_table.h"
 #include "src/trace_processor/storage/trace_storage.h"
@@ -64,6 +65,8 @@ class SliceTrackerTest : public ::testing::Test {
     context_.storage = std::make_unique<TraceStorage>();
     context_.global_args_tracker =
         std::make_unique<GlobalArgsTracker>(context_.storage.get());
+    context_.machine_tracker =
+        std::make_unique<MachineTracker>(&context_, kDefaultMachineId);
     context_.args_translation_table =
         std::make_unique<ArgsTranslationTable>(context_.storage.get());
     context_.slice_translation_table =
@@ -253,7 +256,6 @@ TEST_F(SliceTrackerTest, TwoSliceDetailed) {
   EXPECT_EQ(sr0.category().value_or(kNullStringId).raw_id(), 0u);
   EXPECT_EQ(sr0.name().value_or(kNullStringId).raw_id(), 1u);
   EXPECT_EQ(sr0.depth(), 0u);
-  EXPECT_EQ(sr0.parent_stack_id(), 0);
 
   auto sr1 = slices[1];
   EXPECT_EQ(sr1.ts(), 3);
@@ -262,9 +264,6 @@ TEST_F(SliceTrackerTest, TwoSliceDetailed) {
   EXPECT_EQ(sr1.category().value_or(kNullStringId).raw_id(), 0u);
   EXPECT_EQ(sr1.name().value_or(kNullStringId).raw_id(), 2u);
   EXPECT_EQ(sr1.depth(), 1u);
-  EXPECT_NE(sr1.stack_id(), 0);
-
-  EXPECT_EQ(sr0.stack_id(), sr1.parent_stack_id());
 }
 
 TEST_F(SliceTrackerTest, Scoped) {
