@@ -107,6 +107,7 @@ class FtraceParser {
                                 uint32_t pid,
                                 protozero::ConstBytes);
   void ParseDpuDispDpuUnderrun(int64_t timestamp, protozero::ConstBytes);
+  void ParseGramCollision(int64_t timestamp, protozero::ConstBytes);
   void ParseDpuDispVblankIrqEnable(int64_t timestamp, protozero::ConstBytes);
   void ParseG2dTracingMarkWrite(int64_t timestamp,
                                 uint32_t pid,
@@ -346,6 +347,12 @@ class FtraceParser {
                               protozero::ConstBytes blob);
   void ParseMaliGpuPowerState(int64_t ts, protozero::ConstBytes blob);
   void ParseDmabufRssStat(int64_t ts, uint32_t pid, protozero::ConstBytes blob);
+  void ParseFwtpPerfettoCounter(protozero::ConstBytes blob);
+  void ParseFwtpPerfettoSlice(int64_t ts, protozero::ConstBytes blob);
+  void ParseF2fsWriteCheckpoint(int64_t ts,
+                                uint32_t pid,
+                                protozero::ConstBytes blob);
+  void ParseGpuPowerState(int64_t ts, protozero::ConstBytes blob);
 
   TraceProcessorContext* context_;
   GenericFtraceTracker* generic_tracker_;
@@ -442,6 +449,17 @@ class FtraceParser {
   const StringId disp_vblank_irq_enable_output_id_arg_name_;
   const StringId hrtimer_id_;
   const StringId local_timer_id_;
+  const StringId f2fs_checkpoint_name_id_;
+  const StringId f2fs_reason_str_arg_id_;
+  const StringId f2fs_reason_int_arg_id_;
+  const StringId f2fs_dev_arg_id_;
+  const StringId f2fs_checkpoint_unknown_reason_id_;
+  const StringId gpu_power_state_unknown_id_;
+  const StringId gpu_power_state_off_id_;
+  const StringId gpu_power_state_pg_id_;
+  const StringId gpu_power_state_on_id_;
+  const StringId ddic_underrun_id_;
+  std::array<StringId, 8> f2fs_checkpoint_reason_ids_;
 
   std::vector<StringId> syscall_arg_name_ids_;
 
