@@ -50,8 +50,10 @@
 #include "protos/perfetto/trace/ftrace/cpm_trace.pbzero.h"
 #include "protos/perfetto/trace/ftrace/ftrace_event.pbzero.h"
 #include "protos/perfetto/trace/ftrace/ftrace_event_bundle.pbzero.h"
+#include "protos/perfetto/trace/ftrace/fwtp_ftrace.pbzero.h"
 #include "protos/perfetto/trace/ftrace/power.pbzero.h"
 #include "protos/perfetto/trace/ftrace/thermal_exynos.pbzero.h"
+#include "src/trace_processor/util/clock_synchronizer.h"
 
 namespace perfetto::trace_processor {
 
@@ -139,7 +141,8 @@ base::Status FtraceTokenizer::TokenizeFtraceBundle(
   // Deal with ftrace recorded using a clock that isn't our preferred default
   // (boottime). Do a best-effort fit to the "primary trace clock" based on
   // per-bundle timestamp snapshots.
-  ClockTracker::ClockId clock_id = BuiltinClock::BUILTIN_CLOCK_BOOTTIME;
+  ClockTracker::ClockId clock_id =
+      ClockId::Machine(BuiltinClock::BUILTIN_CLOCK_BOOTTIME);
   if (decoder.has_ftrace_clock()) {
     ASSIGN_OR_RETURN(clock_id,
                      HandleFtraceClockSnapshot(decoder, packet_sequence_id));
@@ -285,6 +288,18 @@ void FtraceTokenizer::TokenizeFtraceEvent(
     TokenizeFtraceParamSetValueCpm(cpu, std::move(event), std::move(state));
     return;
   }
+  if (PERFETTO_UNLIKELY(
+          event_id ==
+          protos::pbzero::FtraceEvent::kFwtpPerfettoCounterFieldNumber)) {
+    TokenizeFtraceFwtpPerfettoCounter(cpu, std::move(event), std::move(state));
+    return;
+  }
+  if (PERFETTO_UNLIKELY(
+          event_id ==
+          protos::pbzero::FtraceEvent::kFwtpPerfettoSliceFieldNumber)) {
+    TokenizeFtraceFwtpPerfettoSlice(cpu, std::move(event), std::move(state));
+    return;
+  }
 
   std::optional<int64_t> timestamp = context_->clock_tracker->ToTraceTime(
       clock_id, static_cast<int64_t>(raw_timestamp));
@@ -429,13 +444,14 @@ FtraceTokenizer::HandleFtraceClockSnapshot(
     protos::pbzero::FtraceEventBundle::Decoder& decoder,
     uint32_t packet_sequence_id) {
   // Convert from ftrace clock enum to a clock id for trace parsing.
-  ClockTracker::ClockId clock_id = BuiltinClock::BUILTIN_CLOCK_BOOTTIME;
+  ClockTracker::ClockId clock_id =
+      ClockId::Machine(BuiltinClock::BUILTIN_CLOCK_BOOTTIME);
   switch (decoder.ftrace_clock()) {
     case FtraceClock::FTRACE_CLOCK_UNSPECIFIED:
-      clock_id = BuiltinClock::BUILTIN_CLOCK_BOOTTIME;
+      clock_id = ClockId::Machine(BuiltinClock::BUILTIN_CLOCK_BOOTTIME);
       break;
     case FtraceClock::FTRACE_CLOCK_MONO_RAW:
-      clock_id = BuiltinClock::BUILTIN_CLOCK_MONOTONIC_RAW;
+      clock_id = ClockId::Machine(BuiltinClock::BUILTIN_CLOCK_MONOTONIC_RAW);
       break;
     case FtraceClock::FTRACE_CLOCK_GLOBAL:
     case FtraceClock::FTRACE_CLOCK_LOCAL:
@@ -449,8 +465,8 @@ FtraceTokenizer::HandleFtraceClockSnapshot(
       // cpu0 as recorded in the bundle. Note: the timestamps will be in the
       // future relative to the data covered by the bundle, as the timestamping
       // is done at ftrace read time.
-      clock_id = ClockTracker::SequenceToGlobalClock(packet_sequence_id,
-                                                     kSequenceScopedClockId);
+      clock_id = ClockId::Sequence(context_->trace_id().value,
+                                   packet_sequence_id, kSequenceScopedClockId);
       break;
     default:
       return base::ErrStatus(
@@ -461,12 +477,17 @@ FtraceTokenizer::HandleFtraceClockSnapshot(
   // duplicates since multiple sequential ftrace bundles can share a snapshot.
   if (decoder.has_ftrace_timestamp() && decoder.has_boot_timestamp() &&
       latest_ftrace_clock_snapshot_ts_ != decoder.ftrace_timestamp()) {
-    PERFETTO_DCHECK(clock_id != BuiltinClock::BUILTIN_CLOCK_BOOTTIME);
+    PERFETTO_DCHECK(clock_id !=
+                    ClockId::Machine(BuiltinClock::BUILTIN_CLOCK_BOOTTIME));
     int64_t ftrace_timestamp = decoder.ftrace_timestamp();
-    context_->clock_tracker->AddSnapshot(
-        {ClockTracker::ClockTimestamp(clock_id, ftrace_timestamp),
-         ClockTracker::ClockTimestamp(BuiltinClock::BUILTIN_CLOCK_BOOTTIME,
-                                      decoder.boot_timestamp())});
+    auto x = context_->clock_tracker->AddSnapshot({
+        ClockTracker::ClockTimestamp(clock_id, ftrace_timestamp),
+        ClockTracker::ClockTimestamp(
+            ClockId::Machine(BuiltinClock::BUILTIN_CLOCK_BOOTTIME),
+            decoder.boot_timestamp()),
+    });
+    PERFETTO_ELOG("%s", x.ok() ? "Added ftrace clock snapshot"
+                               : x.status().message().c_str());
     latest_ftrace_clock_snapshot_ts_ = ftrace_timestamp;
   }
   return clock_id;
@@ -494,7 +515,7 @@ void FtraceTokenizer::TokenizeFtraceGpuWorkPeriod(
   // Enforce clock type for the event data to be CLOCK_MONOTONIC_RAW
   // as specified, to calculate the timestamp correctly.
   std::optional<int64_t> timestamp = context_->clock_tracker->ToTraceTime(
-      BuiltinClock::BUILTIN_CLOCK_MONOTONIC_RAW,
+      ClockId::Machine(BuiltinClock::BUILTIN_CLOCK_MONOTONIC_RAW),
       static_cast<int64_t>(raw_timestamp));
 
   // ClockTracker will increment some error stats if it failed to convert the
@@ -553,6 +574,54 @@ void FtraceTokenizer::TokenizeFtraceParamSetValueCpm(
       cpu, timestamp, TracePacketData{std::move(event), std::move(state)});
 }
 
+void FtraceTokenizer::TokenizeFtraceFwtpPerfettoCounter(
+    uint32_t cpu,
+    TraceBlobView event,
+    RefPtr<PacketSequenceStateGeneration> state) {
+  // Special handling of valid fwtp_perfetto_counter tracepoint events which
+  // contains the right timestamp value nested inside the event data.
+  auto ts_field = GetFtraceEventField(
+      protos::pbzero::FtraceEvent::kFwtpPerfettoCounterFieldNumber, event);
+  if (!ts_field.has_value())
+    return;
+
+  protos::pbzero::FwtpPerfettoCounterFtraceEvent::Decoder
+      fwtp_perfetto_counter_event(ts_field.value().data(),
+                                  ts_field.value().size());
+  if (!fwtp_perfetto_counter_event.has_timestamp()) {
+    context_->storage->IncrementStats(stats::ftrace_bundle_tokenizer_errors);
+    return;
+  }
+  int64_t timestamp =
+      static_cast<int64_t>(fwtp_perfetto_counter_event.timestamp());
+  module_context_->PushFtraceEvent(
+      cpu, timestamp, TracePacketData{std::move(event), std::move(state)});
+}
+
+void FtraceTokenizer::TokenizeFtraceFwtpPerfettoSlice(
+    uint32_t cpu,
+    TraceBlobView event,
+    RefPtr<PacketSequenceStateGeneration> state) {
+  // Special handling of valid fwtp_perfetto_slice tracepoint events which
+  // contains the right timestamp value nested inside the event data.
+  auto ts_field = GetFtraceEventField(
+      protos::pbzero::FtraceEvent::kFwtpPerfettoSliceFieldNumber, event);
+  if (!ts_field.has_value())
+    return;
+
+  protos::pbzero::FwtpPerfettoSliceFtraceEvent::Decoder
+      fwtp_perfetto_slice_event(ts_field.value().data(),
+                                ts_field.value().size());
+  if (!fwtp_perfetto_slice_event.has_timestamp()) {
+    context_->storage->IncrementStats(stats::ftrace_bundle_tokenizer_errors);
+    return;
+  }
+  int64_t timestamp =
+      static_cast<int64_t>(fwtp_perfetto_slice_event.timestamp());
+  module_context_->PushFtraceEvent(
+      cpu, timestamp, TracePacketData{std::move(event), std::move(state)});
+}
+
 std::optional<protozero::Field> FtraceTokenizer::GetFtraceEventField(
     uint32_t event_id,
     const TraceBlobView& event) {
