@@ -28,17 +28,14 @@
 #include <utility>
 #include <vector>
 
-#include "perfetto/base/build_config.h"
 #include "perfetto/base/flat_set.h"
 #include "perfetto/base/logging.h"
 #include "perfetto/base/status.h"
 #include "perfetto/ext/base/status_macros.h"
 #include "perfetto/ext/base/status_or.h"
-#include "perfetto/public/compiler.h"
 #include "perfetto/trace_processor/ref_counted.h"
 #include "perfetto/trace_processor/trace_blob_view.h"
 #include "protos/perfetto/common/builtin_clock.pbzero.h"
-#include "protos/perfetto/trace/clock_snapshot.pbzero.h"
 #include "protos/third_party/simpleperf/record_file.pbzero.h"
 #include "src/trace_processor/importers/common/clock_tracker.h"
 #include "src/trace_processor/importers/common/metadata_tracker.h"
@@ -67,6 +64,7 @@
 #include "src/trace_processor/storage/stats.h"
 #include "src/trace_processor/types/variadic.h"
 #include "src/trace_processor/util/build_id.h"
+#include "src/trace_processor/util/clock_synchronizer.h"
 #include "src/trace_processor/util/trace_blob_view_reader.h"
 
 namespace perfetto::trace_processor::perf_importer {
@@ -112,7 +110,6 @@ bool ReadTime(const Record& record, std::optional<uint64_t>& time) {
     return reader.Skip(reader.size_left() - *offset) &&
            reader.ReadOptional(time);
   }
-
   std::optional<size_t> offset = record.attr->time_offset_from_start();
   if (!offset.has_value()) {
     time = std::nullopt;
@@ -210,9 +207,6 @@ PerfDataTokenizer::ParseHeader() {
   feature_ids_ = ExtractFeatureIds(header_.flags, header_.flags1);
   feature_headers_section_ = {header_.data.end(),
                               feature_ids_.size() * sizeof(PerfFile::Section)};
-  context_->clock_tracker->SetTraceTimeClock(
-      protos::pbzero::ClockSnapshot::Clock::MONOTONIC);
-
   PERFETTO_CHECK(buffer_.PopFrontUntil(sizeof(PerfFile::Header)));
   parsing_state_ = ParsingState::kParseAttrs;
   return ParsingResult::kSuccess;
@@ -251,8 +245,8 @@ PerfDataTokenizer::ParseAttrs() {
 
   ASSIGN_OR_RETURN(perf_invocation_, builder.Build());
   if (perf_invocation_->HasPerfClock()) {
-    context_->clock_tracker->SetTraceTimeClock(
-        protos::pbzero::BUILTIN_CLOCK_PERF);
+    context_->clock_tracker->SetGlobalClock(
+        ClockId::Machine(protos::pbzero::BUILTIN_CLOCK_PERF));
   }
   parsing_state_ = ParsingState::kSeekRecords;
   return ParsingResult::kSuccess;
@@ -358,28 +352,44 @@ std::optional<int64_t> PerfDataTokenizer::ExtractTraceTimestamp(
   if (!ReadTime(record, time)) {
     return std::nullopt;
   }
-  // TODO(449973773): `*time > 0` is a temporary hack to work around the fact
-  // that some perf record types which actually don't have a timestamp. They
-  // should have been procesed during tokenization time (e.g. MMAP/MMAP2/COMM)
-  // but were incorrectly written to be handled with at parsing time. So by
-  // setting trace_ts to `latest_timestamp_`, we don't try and convert a zero
-  // timestamp accidentally, leading to negative timestamps in some clocks.
-  std::optional<int64_t> trace_ts =
-      time && *time > 0
-          ? context_->clock_tracker->ToTraceTime(record.attr->clock_id(),
-                                                 static_cast<int64_t>(*time))
-          : std::optional<int64_t>(latest_timestamp_);
-  if (PERFETTO_LIKELY(trace_ts.has_value())) {
-    latest_timestamp_ = std::max(latest_timestamp_, *trace_ts);
+  if (!time || *time == 0) {
+    // Record has no timestamp - return nullopt to signal it should be buffered
+    return std::nullopt;
   }
-  return trace_ts;
+  return context_->clock_tracker->ToTraceTime(record.attr->clock_id(),
+                                              static_cast<int64_t>(*time));
 }
 
 void PerfDataTokenizer::MaybePushRecord(Record record) {
   std::optional<int64_t> trace_ts = ExtractTraceTimestamp(record);
+
+  // Track minimum timestamp for records without timestamps
   if (trace_ts) {
-    stream_->Push(*trace_ts, std::move(record));
+    min_timestamp_ = std::min(min_timestamp_.value_or(*trace_ts), *trace_ts);
   }
+  // Buffer COMM records until end of file to make sure regardless of whether
+  // or not they have timestamps, they are sent in the correct order.
+  if (record.header.type == PERF_RECORD_COMM) {
+    buffered_comm_records_.push_back({std::move(record), trace_ts});
+    return;
+  }
+  // Record has no timestamp (e.g. MMAP, MMAP2). Buffer it until the end
+  // of file and send with the minimum timestamp seen in the file.
+  if (!trace_ts) {
+    pending_records_without_timestamp_.push_back(std::move(record));
+    return;
+  }
+
+  // Flush all pending records without timestamps with the minimum timestamp.
+  // We do this here to make sure that if `record` depends on any of the
+  // pending records, we process them in order.
+  for (auto& pending : pending_records_without_timestamp_) {
+    stream_->Push(*min_timestamp_, std::move(pending));
+  }
+  pending_records_without_timestamp_.clear();
+
+  // Now push the current record.
+  stream_->Push(*trace_ts, std::move(record));
 }
 
 base::StatusOr<PerfDataTokenizer::ParsingResult>
@@ -562,12 +572,33 @@ base::Status PerfDataTokenizer::ProcessItraceStartRecord(Record record) {
   return base::OkStatus();
 }
 
-base::Status PerfDataTokenizer::NotifyEndOfFile() {
+base::Status PerfDataTokenizer::OnPushDataToSorter() {
+  // Phase 1: Validate parsing is complete
   if (parsing_state_ != ParsingState::kDone) {
     return base::ErrStatus("Premature end of perf file.");
   }
-  RETURN_IF_ERROR(perf_tracker_.NotifyEndOfFile());
+
+  // Flush all buffered COMM records in file order
+  for (auto& comm : buffered_comm_records_) {
+    int64_t ts;
+    if (comm.timestamp.has_value()) {
+      ts = *comm.timestamp;
+    } else if (min_timestamp_.has_value()) {
+      ts = *min_timestamp_;
+    } else {
+      // Skip records without timestamps if we never saw a minimum timestamp
+      continue;
+    }
+    stream_->Push(ts, std::move(comm.record));
+  }
+  buffered_comm_records_.clear();
+
   return base::OkStatus();
 }
 
+void PerfDataTokenizer::OnEventsFullyExtracted() {
+  // Phase 3: Finalize tracker
+  perf_tracker_.OnEventsFullyExtracted();
+}
+
 }  // namespace perfetto::trace_processor::perf_importer
