@@ -86,6 +86,11 @@ StringId JankTypeBitmaskToStringId(TraceProcessorContext* context,
     jank_reasons.emplace_back("Non Animating");
   if (jank_type & FrameTimelineEvent::JANK_DISPLAY_NOT_ON)
     jank_reasons.emplace_back("Display not ON");
+  if (jank_type & FrameTimelineEvent::JANK_DISPLAY_MODE_CHANGE_IN_PROGRESS)
+    jank_reasons.emplace_back("ModeChange in progress");
+  if (jank_type &
+      FrameTimelineEvent::JANK_DISPLAY_POWER_MODE_CHANGE_IN_PROGRESS)
+    jank_reasons.emplace_back("PowerModeChange in progress");
 
   std::string jank_str(
       std::accumulate(jank_reasons.begin(), jank_reasons.end(), std::string(),
@@ -214,6 +219,8 @@ FrameTimelineEventParser::FrameTimelineEventParser(
           context->storage->InternString("Jank severity type")),
       jank_severity_score_id_(
           context->storage->InternString("Jank Severity Score (experimental)")),
+      jank_debug_metadata_id_(
+          context->storage->InternString("Jank Metadata (debugging only)")),
       layer_name_id_(context->storage->InternString("Layer name")),
       prediction_type_id_(context->storage->InternString("Prediction type")),
       jank_tag_id_(context->storage->InternString("Jank tag")),
@@ -229,10 +236,8 @@ FrameTimelineEventParser::FrameTimelineEventParser(
           context->storage->InternString("Buffer Stuffing")),
       jank_tag_sf_stuffing_id_(
           context->storage->InternString("SurfaceFlinger Stuffing")),
-      jank_tag_none_animating_id_(
-          context->storage->InternString("Non Animating")),
-      jank_tag_display_not_on_id_(
-          context->storage->InternString("Display not ON")) {}
+      jank_tag_none_perceivable_id_(
+          context->storage->InternString("Non-perceivable Jank")) {}
 
 void FrameTimelineEventParser::ParseExpectedDisplayFrameStart(int64_t timestamp,
                                                               ConstBytes blob) {
@@ -272,10 +277,11 @@ StringId FrameTimelineEventParser::CalculateDisplayFrameJankTag(
     jank_tag = jank_tag_sf_stuffing_id_;
   } else if (jank_type == FrameTimelineEvent::JANK_DROPPED) {
     jank_tag = jank_tag_dropped_id_;
-  } else if (jank_type == FrameTimelineEvent::JANK_NON_ANIMATING) {
-    jank_tag = jank_tag_none_animating_id_;
-  } else if (jank_type == FrameTimelineEvent::JANK_DISPLAY_NOT_ON) {
-    jank_tag = jank_tag_display_not_on_id_;
+  } else if (jank_type == FrameTimelineEvent::JANK_NON_ANIMATING ||
+             jank_type == FrameTimelineEvent::JANK_DISPLAY_NOT_ON ||
+             jank_type == FrameTimelineEvent::
+                              JANK_DISPLAY_POWER_MODE_CHANGE_IN_PROGRESS) {
+    jank_tag = jank_tag_none_perceivable_id_;
   } else {
     jank_tag = jank_tag_none_id_;
   }
@@ -296,6 +302,7 @@ void FrameTimelineEventParser::ParseActualDisplayFrameStart(int64_t timestamp,
   int64_t cookie = event.cookie();
   int64_t token = event.token();
   double jank_severity_score = static_cast<double>(event.jank_severity_score());
+  double jank_debug_metadata = static_cast<double>(event.jank_debug_metadata());
   double present_delay_millis =
       static_cast<double>(event.present_delay_millis());
   StringId name_id =
@@ -380,6 +387,8 @@ void FrameTimelineEventParser::ParseActualDisplayFrameStart(int64_t timestamp,
         inserter->AddArg(jank_tag_id_, Variadic::String(jank_tag));
         inserter->AddArg(jank_tag_experimental_id_,
                          Variadic::String(jank_tag_experimental));
+        inserter->AddArg(jank_debug_metadata_id_,
+                         Variadic::Real(jank_debug_metadata));
       });
 
   // SurfaceFrames will always be parsed before the matching DisplayFrame
@@ -461,13 +470,16 @@ StringId FrameTimelineEventParser::CalculateSurfaceFrameJankTag(
     jank_tag = jank_tag_other_id_;
   } else if (jank_type == FrameTimelineEvent::JANK_BUFFER_STUFFING) {
     jank_tag = jank_tag_buffer_stuffing_id_;
+  } else if (jank_type == FrameTimelineEvent::JANK_SF_STUFFING) {
+    jank_tag = jank_tag_sf_stuffing_id_;
   } else if (present_type_opt.has_value() &&
              *present_type_opt == FrameTimelineEvent::PRESENT_DROPPED) {
     jank_tag = jank_tag_dropped_id_;
-  } else if (jank_type == FrameTimelineEvent::JANK_NON_ANIMATING) {
-    jank_tag = jank_tag_none_animating_id_;
-  } else if (jank_type == FrameTimelineEvent::JANK_DISPLAY_NOT_ON) {
-    jank_tag = jank_tag_display_not_on_id_;
+  } else if (jank_type == FrameTimelineEvent::JANK_NON_ANIMATING ||
+             jank_type == FrameTimelineEvent::JANK_DISPLAY_NOT_ON ||
+             jank_type == FrameTimelineEvent::
+                              JANK_DISPLAY_POWER_MODE_CHANGE_IN_PROGRESS) {
+    jank_tag = jank_tag_none_perceivable_id_;
   } else {
     jank_tag = jank_tag_none_id_;
   }
@@ -490,6 +502,7 @@ void FrameTimelineEventParser::ParseActualSurfaceFrameStart(int64_t timestamp,
   int64_t token = event.token();
   int64_t display_frame_token = event.display_frame_token();
   double jank_severity_score = static_cast<double>(event.jank_severity_score());
+  double jank_debug_metadata = static_cast<double>(event.jank_debug_metadata());
   double present_delay_millis =
       static_cast<double>(event.present_delay_millis());
   double vsync_resynced_jitter_millis =
@@ -606,6 +619,8 @@ void FrameTimelineEventParser::ParseActualSurfaceFrameStart(int64_t timestamp,
         inserter->AddArg(jank_tag_experimental_id_,
                          Variadic::String(jank_tag_experimental));
         inserter->AddArg(is_buffer_id_, Variadic::String(is_buffer));
+        inserter->AddArg(jank_debug_metadata_id_,
+                         Variadic::Real(jank_debug_metadata));
       });
 
   if (opt_slice_id) {
