@@ -26,6 +26,7 @@
 #include "perfetto/ext/base/string_view.h"
 #include "protos/perfetto/trace/profiling/heap_graph.pbzero.h"
 #include "src/trace_processor/containers/string_pool.h"
+#include "src/trace_processor/importers/common/machine_tracker.h"
 #include "src/trace_processor/importers/common/process_tracker.h"
 #include "src/trace_processor/storage/trace_storage.h"
 #include "src/trace_processor/tables/profiler_tables_py.h"
@@ -72,6 +73,8 @@ TEST(HeapGraphTrackerTest, PopulateNativeSize) {
 
   TraceProcessorContext context;
   context.storage = std::make_unique<TraceStorage>();
+  context.machine_tracker =
+      std::make_unique<MachineTracker>(&context, kDefaultMachineId);
   context.process_tracker = std::make_unique<ProcessTracker>(&context);
   context.process_tracker->GetOrCreateProcess(kPid);
 
@@ -210,6 +213,8 @@ TEST(HeapGraphTrackerTest, BuildFlamegraph) {
 
   TraceProcessorContext context;
   context.storage.reset(new TraceStorage());
+  context.machine_tracker =
+      std::make_unique<MachineTracker>(&context, kDefaultMachineId);
   context.process_tracker.reset(new ProcessTracker(&context));
   context.process_tracker->GetOrCreateProcess(kPid);
 
@@ -338,6 +343,8 @@ TEST(HeapGraphTrackerTest, BuildFlamegraphWeakReferences) {
 
   TraceProcessorContext context;
   context.storage.reset(new TraceStorage());
+  context.machine_tracker =
+      std::make_unique<MachineTracker>(&context, kDefaultMachineId);
   context.process_tracker.reset(new ProcessTracker(&context));
   context.process_tracker->GetOrCreateProcess(kPid);
 
@@ -439,6 +446,191 @@ TEST(HeapGraphTrackerTest, BuildFlamegraphWeakReferences) {
   EXPECT_THAT(counts, UnorderedElementsAre(1, 1));
 }
 
+constexpr uint64_t kRoot = 1;
+
+class HeapGraphStabilityTest : public ::testing::Test {
+ public:
+  class Helper {
+   public:
+    Helper() {
+      context_.storage.reset(new TraceStorage());
+      context_.machine_tracker =
+          std::make_unique<MachineTracker>(&context_, kDefaultMachineId);
+      context_.process_tracker.reset(new ProcessTracker(&context_));
+      context_.process_tracker->GetOrCreateProcess(kPid);
+      tracker_ = std::make_unique<HeapGraphTracker>(context_.storage.get());
+
+      tracker_->AddInternedLocationName(
+          kSeqId, kLocation, context_.storage->InternString("location"));
+      tracker_->AddInternedFieldName(kSeqId, kField, base::StringView("foo"));
+    }
+
+    uint64_t GetOrCreateTypeId(const std::string& name) {
+      if (auto it = class_name_to_id_.find(name);
+          it != class_name_to_id_.end()) {
+        return it->second;
+      }
+      uint64_t id = next_type_id_++;
+      class_name_to_id_[name] = id;
+      RegisterType(id, name);
+      return id;
+    }
+
+    void RegisterType(uint64_t id, const std::string& name) {
+      tracker_->AddInternedType(
+          kSeqId, id, context_.storage->InternString(base::StringView(name)),
+          kLocation, 0, {}, 0, 0, false,
+          protos::pbzero::HeapGraphType::KIND_NORMAL);
+    }
+
+    void RegisterObject(uint64_t id, uint64_t type_id) {
+      object_type_ids_[id] = type_id;
+    }
+
+    void Link(uint64_t parent_id, uint64_t child_id) {
+      if (parent_id == kRoot) {
+        roots_.push_back(child_id);
+      } else {
+        object_refs_[parent_id].push_back(child_id);
+      }
+    }
+
+    void BuildFlamegraph() {
+      for (const auto& [id, type_id] : object_type_ids_) {
+        HeapGraphTracker::SourceObject obj;
+        obj.object_id = id;
+        obj.self_size = 1;  // Default size
+        obj.type_id = type_id;
+
+        auto it = object_refs_.find(id);
+        if (it != object_refs_.end()) {
+          obj.referred_objects = it->second;
+          obj.field_name_ids.resize(obj.referred_objects.size(), kField);
+        }
+
+        tracker_->AddObject(kSeqId, kPid, kTimestamp, std::move(obj));
+      }
+
+      HeapGraphTracker::SourceRoot root;
+      root.root_type = protos::pbzero::HeapGraphRoot::ROOT_UNKNOWN;
+      for (uint64_t id : roots_) {
+        root.object_ids.push_back(id);
+      }
+      tracker_->AddRoot(kSeqId, kPid, kTimestamp, root);
+
+      tracker_->FinalizeAllProfiles();
+      flame_ = tracker_->BuildFlamegraph(kPid, kTimestamp);
+      ASSERT_NE(flame_, nullptr);
+    }
+
+    void AssertParent(const std::string& child_name,
+                      const std::string& expected_parent_name) {
+      bool found_child = false;
+      for (auto it = flame_->IterateRows(); it; ++it) {
+        auto name = context_.storage->string_pool().Get(it.name());
+        if (name.ToStdString() == child_name) {
+          found_child = true;
+          auto parent_id = it.parent_id();
+          ASSERT_TRUE(parent_id.has_value());
+
+          std::optional<std::string> parent_name;
+          for (auto pit = flame_->IterateRows(); pit; ++pit) {
+            if (pit.id().value == parent_id->value) {
+              parent_name =
+                  context_.storage->string_pool().Get(pit.name()).ToStdString();
+              break;
+            }
+          }
+          ASSERT_TRUE(parent_name.has_value());
+          EXPECT_THAT(*parent_name, testing::HasSubstr(expected_parent_name));
+        }
+      }
+      EXPECT_TRUE(found_child) << "Child " << child_name << " not found";
+    }
+
+    uint64_t root_id = 0;
+
+   private:
+    static constexpr uint64_t kSeqId = 1;
+    static constexpr UniquePid kPid = 1;
+    static constexpr int64_t kTimestamp = 1;
+    static constexpr uint64_t kLocation = 0;
+    static constexpr uint64_t kField = 1;
+
+    TraceProcessorContext context_;
+    std::unique_ptr<HeapGraphTracker> tracker_;
+    std::unique_ptr<tables::ExperimentalFlamegraphTable> flame_;
+
+    std::map<uint64_t, uint64_t> object_type_ids_;
+    std::map<uint64_t, std::vector<uint64_t>> object_refs_;
+    std::vector<uint64_t> roots_;
+    std::map<std::string, uint64_t> class_name_to_id_;
+    uint64_t next_type_id_ = 1;
+  };
+};
+
+struct ObjectInfo {
+  uint64_t id;
+  std::string class_name;
+  std::vector<uint64_t> parents;
+};
+
+struct ShortestPathTestCase {
+  std::string name;
+  std::vector<ObjectInfo> objects;
+
+  static std::string ToString(
+      const testing::TestParamInfo<ShortestPathTestCase>& info) {
+    return info.param.name;
+  }
+};
+
+class ShortestPathStabilityTest
+    : public HeapGraphStabilityTest,
+      public testing::WithParamInterface<ShortestPathTestCase> {};
+
+TEST_P(ShortestPathStabilityTest, Run) {
+  const auto& test = GetParam();
+  Helper helper;
+
+  for (const auto& obj : test.objects) {
+    uint64_t type_id = helper.GetOrCreateTypeId(obj.class_name);
+    helper.RegisterObject(obj.id, type_id);
+    for (uint64_t parent : obj.parents) {
+      helper.Link(parent, obj.id);
+    }
+  }
+
+  helper.BuildFlamegraph();
+  helper.AssertParent("Child", "A_Parent");
+}
+
+INSTANTIATE_TEST_SUITE_P(
+    StabilityCases,
+    ShortestPathStabilityTest,
+    testing::Values(
+        ShortestPathTestCase{
+            "Aid_greater_than_Bid__B_references_earlier_A",
+            {{/* id= */ 2, /* name= */ "B_Parent", /* parents= */ {kRoot}},
+             {/* id= */ 3, /* name= */ "A_Parent", /* parents= */ {kRoot}},
+             {/* id= */ 4, /* name= */ "Child", /* parents= */ {2, 3}}}},
+        ShortestPathTestCase{
+            "Aid_greater_than_Bid__A_references_earlier_B",
+            {{/* id= */ 2, /* name= */ "B_Parent", /* parents= */ {kRoot}},
+             {/* id= */ 3, /* name= */ "A_Parent", /* parents= */ {kRoot}},
+             {/* id= */ 4, /* name= */ "Child", /* parents= */ {3, 2}}}},
+        ShortestPathTestCase{
+            "Aid_less_than_Bid__B_references_earlier_A",
+            {{/* id= */ 2, /* name= */ "A_Parent", /* parents= */ {kRoot}},
+             {/* id= */ 3, /* name= */ "B_Parent", /* parents= */ {kRoot}},
+             {/* id= */ 4, /* name= */ "Child", /* parents= */ {3, 2}}}},
+        ShortestPathTestCase{
+            "Aid_less_than_Bid__A_references_earlier_B",
+            {{/* id= */ 2, /* name= */ "A_Parent", /* parents= */ {kRoot}},
+             {/* id= */ 3, /* name= */ "B_Parent", /* parents= */ {kRoot}},
+             {/* id= */ 4, /* name= */ "Child", /* parents= */ {2, 3}}}}),
+    &ShortestPathTestCase::ToString);
+
 constexpr char kArray[] = "X[]";
 constexpr char kDoubleArray[] = "X[][]";
 constexpr char kNoArray[] = "X";
