@@ -23,7 +23,6 @@
 #include <optional>
 #include <string>
 #include <utility>
-#include <variant>
 #include <vector>
 
 #include "perfetto/base/logging.h"
@@ -32,9 +31,8 @@
 #include "perfetto/ext/base/status_or.h"
 #include "perfetto/trace_processor/basic_types.h"
 #include "src/trace_processor/containers/string_pool.h"
-#include "src/trace_processor/dataframe/dataframe.h"
+#include "src/trace_processor/core/dataframe/dataframe.h"
 #include "src/trace_processor/perfetto_sql/engine/dataframe_module.h"
-#include "src/trace_processor/perfetto_sql/engine/dataframe_shared_storage.h"
 #include "src/trace_processor/perfetto_sql/engine/runtime_table_function.h"
 #include "src/trace_processor/perfetto_sql/engine/static_table_function_module.h"
 #include "src/trace_processor/perfetto_sql/intrinsics/table_functions/static_table_function.h"
@@ -66,28 +64,17 @@ class PerfettoSqlEngine {
     SqliteEngine::PreparedStatement stmt;
     ExecutionStats stats;
   };
-  struct UnfinalizedStaticTable {
+  struct StaticTable {
     dataframe::Dataframe* dataframe;
     std::string name;
   };
-  struct FinalizedStaticTable {
-    DataframeSharedStorage::DataframeHandle handle;
-    std::string name;
-  };
-  PerfettoSqlEngine(StringPool* pool,
-                    DataframeSharedStorage* storage,
-                    bool enable_extra_checks);
+  PerfettoSqlEngine(StringPool* pool, bool enable_extra_checks);
 
   // Initializes the static tables and functions in the engine.
   base::Status InitializeStaticTablesAndFunctions(
-      const std::vector<UnfinalizedStaticTable>& unfinalized_tables,
-      std::vector<FinalizedStaticTable> finalized_tables,
+      const std::vector<StaticTable>& tables,
       std::vector<std::unique_ptr<StaticTableFunction>> functions);
 
-  // Finalizes all the static tables owned by this engine and makes them
-  // sharable in the `DataframeSharedStorage` passed in the constructor.
-  void FinalizeAndShareAllStaticTables();
-
   // Executes all the statements in |sql| and returns a |ExecutionResult|
   // object. The metadata will reference all the statements executed and the
   // |ScopedStmt| be empty.
@@ -238,11 +225,18 @@ class PerfettoSqlEngine {
     packages_.Insert(name, std::move(package));
   }
 
+  // Removes a SQL package.
+  void ErasePackage(const std::string& name) { packages_.Erase(name); }
+
   // Fetches registered SQL package.
   sql_modules::RegisteredPackage* FindPackage(const std::string& name) {
     return packages_.Find(name);
   }
 
+  // Finds a package that owns the given module key (i.e., whose name is a
+  // prefix of the key).
+  sql_modules::RegisteredPackage* FindPackageForModule(const std::string& key);
+
   // Returns the number of objects (tables, views, functions etc) registered
   // with SQLite.
   uint64_t SqliteRegisteredObjectCount() {
@@ -281,11 +275,39 @@ class PerfettoSqlEngine {
                                              SqlSource sql);
 
  private:
-  using UnfinalizedOrFinalizedStaticTable =
-      std::variant<DataframeSharedStorage::DataframeHandle,
-                   dataframe::Dataframe*>;
-  void RegisterStaticTable(UnfinalizedOrFinalizedStaticTable,
-                           const std::string&);
+  enum class FrameType { kRoot, kInclude, kWildcard };
+
+  // Result of processing a single frame iteration.
+  enum class FrameResult {
+    kContinue,     // Frame still has work, continue processing it
+    kFrameDone,    // Frame completed, should be popped
+    kReturnResult  // Root frame completed with result
+  };
+
+  // Represents the execution state for a single SQL source being executed.
+  struct ExecutionFrame {
+    FrameType type = FrameType::kRoot;
+
+    // For root and include frames: the SQL being executed
+    SqlSource sql_source;
+    std::unique_ptr<PerfettoSqlParser> parser;
+    ExecutionStats accumulated_stats;
+    std::optional<SqliteEngine::PreparedStatement> current_stmt;
+
+    // For include frames: metadata needed to complete the include
+    std::string include_key;
+    sql_modules::RegisteredPackage::ModuleFile* file_ptr = nullptr;
+    SqlSource traceback_sql;
+
+    // For wildcard frames: modules to include (processed in order)
+    std::vector<
+        std::pair<std::string, sql_modules::RegisteredPackage::ModuleFile*>>
+        wildcard_modules;
+    size_t wildcard_index = 0;
+    SqlSource wildcard_traceback_sql;
+  };
+
+  void RegisterStaticTable(dataframe::Dataframe*, const std::string&);
   void RegisterStaticTableFunction(std::unique_ptr<StaticTableFunction> fn);
 
   base::Status ExecuteCreateFunction(const PerfettoSqlParser::CreateFunction&);
@@ -344,6 +366,14 @@ class PerfettoSqlEngine {
                                  const std::string& key,
                                  const PerfettoSqlParser&);
 
+  // Implementation of ExecuteUntilLastStatement. Separated to handle
+  // re-entrant Execute() calls from statement handlers.
+  base::StatusOr<ExecutionResult> ExecuteUntilLastStatementImpl(SqlSource);
+
+  // Processes a single iteration of the frame at the given index.
+  // May push new frames onto the stack (for includes/wildcards).
+  base::StatusOr<FrameResult> ProcessFrame(size_t frame_idx);
+
   // Called when a transaction is committed by SQLite; that is, the result of
   // running some SQL is considered "perm".
   //
@@ -362,19 +392,14 @@ class PerfettoSqlEngine {
 
   StringPool* pool_ = nullptr;
 
-  // Storage for shared Dataframe objects.
-  //
-  // Note that this class can be shared between multiple PerfettoSqlEngine
-  // instances which are operating on different threads.
-  DataframeSharedStorage* dataframe_shared_storage_;
-
   // If true, engine will perform additional consistency checks when e.g.
   // creating tables and views.
   const bool enable_extra_checks_;
 
-  // A stack which keeps track of the modules which are being included. Used to
-  // know when dataframes should be shared.
-  std::vector<std::string> module_include_stack_;
+  // Execution stack for iterative (non-recursive) processing of SQL sources.
+  // When an INCLUDE statement is encountered, the included module's SQL is
+  // pushed onto this stack and executed before continuing with the current SQL.
+  std::vector<ExecutionFrame> execution_stack_;
 
   uint64_t function_count_ = 0;
   uint64_t aggregate_function_count_ = 0;
@@ -415,8 +440,8 @@ base::Status PerfettoSqlEngine::RegisterFunction(
   function_count_++;
   const char* name = args.name ? args.name : Function::kName;
   int argc = args.argc.has_value() ? args.argc.value() : Function::kArgCount;
-  return engine_->RegisterFunction(name, argc, Function::Step, ctx, nullptr,
-                                   args.deterministic);
+  return RegisterFunctionAndAddToRegistry(name, argc, Function::Step, ctx,
+                                          nullptr, args.deterministic);
 }
 
 template <typename Function>
@@ -426,7 +451,7 @@ base::Status PerfettoSqlEngine::RegisterFunction(
   function_count_++;
   const char* name = args.name ? args.name : Function::kName;
   int argc = args.argc.has_value() ? args.argc.value() : Function::kArgCount;
-  return engine_->RegisterFunction(
+  return RegisterFunctionAndAddToRegistry(
       name, argc, Function::Step, ctx.release(),
       [](void* ptr) {
         std::unique_ptr<typename Function::UserData>(
