@@ -14,9 +14,9 @@
 -- limitations under the License.
 
 -- It's very typical to filter the flow table on either incoming or outgoing slice ids.
-CREATE PERFETTO INDEX flow_in ON flow(slice_in);
+CREATE PERFETTO INDEX flow_in ON __intrinsic_flow(slice_in);
 
-CREATE PERFETTO INDEX flow_out ON flow(slice_out);
+CREATE PERFETTO INDEX flow_out ON __intrinsic_flow(slice_out);
 
 CREATE PERFETTO INDEX slice_parent_id ON __intrinsic_slice(parent_id);
 
