@@ -219,6 +219,20 @@ namespace perfetto::trace_processor::stats {
       "https://perfetto.dev/docs/concepts/buffers"                             \
       "#incremental-state-in-trace-packets"),                                  \
   F(traced_buf_write_wrap_count,          kIndexed, kInfo,     kTrace,    ""), \
+  F(traced_buf_v2s_packets_seen,          kIndexed, kInfo,     kTrace,         \
+       "Shadow mode: total packets read."),                                    \
+  F(traced_buf_v2s_packets_in_both,       kIndexed, kInfo,     kTrace,         \
+       "Shadow mode: packets found in both V1 and V2 buffers."),               \
+  F(traced_buf_v2s_packets_only_v1,       kIndexed, kInfo,     kTrace,         \
+       "Shadow mode: packets found only in V1 buffer."),                       \
+  F(traced_buf_v2s_packets_only_v2,       kIndexed, kInfo,     kTrace,         \
+       "Shadow mode: packets found only in V2 buffer."),                       \
+  F(traced_buf_v2s_patches_attempted,     kIndexed, kInfo,     kTrace,         \
+       "Shadow mode: patch operations attempted."),                            \
+  F(traced_buf_v2s_v1_patches_succeeded,  kIndexed, kInfo,     kTrace,         \
+       "Shadow mode: patches that succeeded on V1 buffer."),                   \
+  F(traced_buf_v2s_v2_patches_succeeded,  kIndexed, kInfo,     kTrace,         \
+       "Shadow mode: patches that succeeded on V2 buffer."),                   \
   F(traced_clone_started_timestamp_ns,    kSingle,  kInfo,     kTrace,         \
     "The timestamp when the clone snapshot operation for this trace started"), \
   F(traced_clone_trigger_timestamp_ns,    kSingle,  kInfo,     kTrace,         \
@@ -282,6 +296,11 @@ namespace perfetto::trace_processor::stats {
       "time clock. Both clocks exist in snapshots, but never together or "     \
       "via a common intermediate clock. Ensure ClockSnapshots link all used "  \
       "clocks to the trace time clock."),                                      \
+  F(clock_sync_mixed_clock_sources,         kSingle,  kError,    kAnalysis,      \
+      "A non-primary trace file used both the primary trace's clock "          \
+      "snapshots and its own for timestamp conversion. Timestamps "            \
+      "converted before the first own clock snapshot used the primary "        \
+      "trace's clocks which may differ."),                                     \
   F(clock_sync_cache_miss,                kSingle,  kInfo,     kAnalysis, ""), \
   F(process_tracker_errors,               kSingle,  kError,    kAnalysis, ""), \
   F(namespaced_thread_missing_process,    kSingle,  kError,    kAnalysis,      \
@@ -338,6 +357,8 @@ namespace perfetto::trace_processor::stats {
   F(metatrace_overruns,                   kSingle,  kError,    kTrace,    ""), \
   F(packages_list_has_parse_errors,       kSingle,  kError,    kTrace,    ""), \
   F(packages_list_has_read_errors,        kSingle,  kError,    kTrace,    ""), \
+  F(user_list_errors,       kSingle,  kError,    kTrace,                       \
+        "Errors occurred while reading or parsing the user.list file."),       \
   F(game_intervention_has_parse_errors,   kSingle,  kError,    kTrace,         \
        "One or more parsing errors occurred. This could result from "          \
        "unknown game more or intervention added to the file to be parsed."),   \
@@ -796,7 +817,28 @@ namespace perfetto::trace_processor::stats {
       "incomplete. Some objects and references may be missing from the heap "  \
       "graph. This typically occurs when the profiled process crashes or is "  \
       "killed before completing the profile. To get complete profiles, ensure "\
-      "the process finishes cleanly or increase the profiling timeout.")
+      "the process finishes cleanly or increase the profiling timeout."),      \
+  F(primes_unknown_edge_type, kSingle, kDataLoss, kAnalysis,                   \
+    "A Primes TraceEdge was received which did not contain a known slice type "\
+    "(Begin, End, Mark)."),                                                    \
+  F(primes_executor_not_found, kSingle, kDataLoss, kAnalysis,                  \
+    "A valid executor_id was not found for the given edge's parent_id, so the "\
+    "slice was dropped."),                                                     \
+  F(primes_end_without_matching_begin, kSingle, kInfo, kAnalysis,              \
+    "A SliceEnd event was seen without its corresponding SliceBegin."),        \
+  F(primes_missing_entity_details, kSingle, kInfo, kAnalysis,                  \
+    "The entity_details field was missing from an edge that requires it."),    \
+  F(primes_missing_parent_id, kSingle, kInfo, kAnalysis,                       \
+    "The parent_id field was missing from an edge that requires it."),         \
+  F(primes_malformed_timestamp, kSingle, kDataLoss, kAnalysis,                 \
+    "The timestamp for an edge or trace was not able to be parsed"),           \
+  F(protovm_abort, kSingle,  kError, kAnalysis,                                \
+      "A ProtoVM instance aborted the execution while applying the patch. "    \
+      "This might be due to inconsistencies between VM program logic and "     \
+      "actual patch format."),                                                 \
+  F(protovm_registration_error, kSingle,  kError, kAnalysis,                   \
+    "Failed to find the sequence IDs corresponding to a ProtoVM's producer "   \
+    "ID. Such mapping should be provided by the TraceProvenance packet.")
 // clang-format on
 
 enum Type {
