@@ -34,7 +34,8 @@ from python.generators.trace_processor_table.public import WrappingSqlView
 MACHINE_TABLE = Table(
     python_module=__file__,
     class_name='MachineTable',
-    sql_name='machine',
+    sql_name='__intrinsic_machine',
+    wrapping_sql_view=WrappingSqlView('machine'),
     columns=[
         C('raw_id', CppUint32()),
         C(
@@ -77,6 +78,16 @@ MACHINE_TABLE = Table(
             CppOptional(CppInt64()),
             cpp_access=CppAccess.READ_AND_LOW_PERF_WRITE,
         ),
+        C(
+            'system_ram_bytes',
+            CppOptional(CppInt64()),
+            cpp_access=CppAccess.READ_AND_LOW_PERF_WRITE,
+        ),
+        C(
+            'system_ram_gb',
+            CppOptional(CppInt64()),
+            cpp_access=CppAccess.READ_AND_LOW_PERF_WRITE,
+        ),
     ],
     tabledoc=TableDoc(
         doc='''
@@ -122,6 +133,15 @@ MACHINE_TABLE = Table(
                 '''
                   The Android SDK version used in the machine.
                 ''',
+            'system_ram_bytes':
+                '''
+                  Total system RAM in bytes available to the machine.
+                ''',
+            'system_ram_gb':
+                '''
+                  Total system RAM in gigabytes (rounded) available to the
+                  machine.
+                ''',
         }))
 
 PROCESS_TABLE = Table(
@@ -191,7 +211,7 @@ PROCESS_TABLE = Table(
             cpp_access=CppAccess.READ_AND_HIGH_PERF_WRITE,
             cpp_access_duration=CppAccessDuration.POST_FINALIZATION,
         ),
-        C('machine_id', CppOptional(CppTableId(MACHINE_TABLE))),
+        C('machine_id', CppTableId(MACHINE_TABLE)),
     ],
     wrapping_sql_view=WrappingSqlView(view_name='process',),
     tabledoc=TableDoc(
@@ -304,10 +324,14 @@ THREAD_TABLE = Table(
             sql_access=SqlAccess.HIGH_PERF,
             cpp_access=CppAccess.READ_AND_HIGH_PERF_WRITE,
         ),
-        C('is_idle', CppUint32()),
+        C(
+            'is_idle',
+            CppUint32(),
+            cpp_access=CppAccess.READ_AND_HIGH_PERF_WRITE,
+        ),
         C(
             'machine_id',
-            CppOptional(CppTableId(MACHINE_TABLE)),
+            CppTableId(MACHINE_TABLE),
             cpp_access=CppAccess.READ,
         ),
         C(
@@ -402,7 +426,7 @@ CPU_TABLE = Table(
             CppString(),
             cpp_access=CppAccess.READ_AND_LOW_PERF_WRITE,
         ),
-        C('machine_id', CppOptional(CppTableId(MACHINE_TABLE))),
+        C('machine_id', CppTableId(MACHINE_TABLE)),
         C(
             'capacity',
             CppOptional(CppUint32()),
@@ -607,10 +631,69 @@ ARG_TABLE = Table(
             'value_type': ''''''
         }))
 
+TRACE_FILE_TABLE = Table(
+    python_module=__file__,
+    class_name='TraceFileTable',
+    sql_name='__intrinsic_trace_file',
+    columns=[
+        C(
+            'parent_id',
+            CppOptional(CppSelfTableId()),
+            cpp_access=CppAccess.READ_AND_LOW_PERF_WRITE,
+        ),
+        C(
+            'name',
+            CppOptional(CppString()),
+            cpp_access=CppAccess.READ_AND_LOW_PERF_WRITE,
+        ),
+        C('size', CppInt64(), cpp_access=CppAccess.READ_AND_LOW_PERF_WRITE),
+        C(
+            'trace_type',
+            CppString(),
+            cpp_access=CppAccess.READ_AND_LOW_PERF_WRITE,
+        ),
+        C(
+            'processing_order',
+            CppOptional(CppInt64()),
+            sql_access=SqlAccess.HIGH_PERF,
+            cpp_access=CppAccess.READ_AND_HIGH_PERF_WRITE,
+        ),
+        C(
+            'is_container',
+            CppUint32(),
+            cpp_access=CppAccess.READ_AND_LOW_PERF_WRITE,
+        ),
+    ],
+    wrapping_sql_view=WrappingSqlView('trace_file'),
+    tabledoc=TableDoc(
+        doc='''
+            Metadata related to the trace file parsed. Note the order in which
+            the files appear in this table corresponds to the order in which
+            they are read and sent to the tokenization stage.
+        ''',
+        group='Misc',
+        columns={
+            'parent_id':
+                '''
+                  Parent file. E.g. files contained in a zip file will point to
+                  the zip file.
+                ''',
+            'name':
+                '''File name, if known, NULL otherwise''',
+            'size':
+                '''Size in bytes''',
+            'trace_type':
+                '''Trace type''',
+            'processing_order':
+                '''In which order where the files were processed.''',
+            'is_container':
+                '''Whether the file is a container (e.g. zip, gzip)''',
+        }))
+
 METADATA_TABLE = Table(
     python_module=__file__,
     class_name='MetadataTable',
-    sql_name='metadata',
+    sql_name='__intrinsic_metadata',
     columns=[
         C(
             'name',
@@ -633,6 +716,16 @@ METADATA_TABLE = Table(
             cpp_access=CppAccess.READ_AND_HIGH_PERF_WRITE,
             cpp_access_duration=CppAccessDuration.POST_FINALIZATION,
         ),
+        C(
+            'machine_id',
+            CppOptional(CppTableId(MACHINE_TABLE)),
+            cpp_access=CppAccess.READ_AND_HIGH_PERF_WRITE,
+        ),
+        C(
+            'trace_id',
+            CppOptional(CppTableId(TRACE_FILE_TABLE)),
+            cpp_access=CppAccess.READ_AND_HIGH_PERF_WRITE,
+        ),
     ],
     tabledoc=TableDoc(
         doc='''''',
@@ -641,13 +734,16 @@ METADATA_TABLE = Table(
             'name': '''''',
             'key_type': '''''',
             'int_value': '''''',
-            'str_value': ''''''
+            'str_value': '''''',
+            'machine_id': '''''',
+            'trace_id': '''''',
         }))
 
 FILEDESCRIPTOR_TABLE = Table(
     python_module=__file__,
     class_name='FiledescriptorTable',
-    sql_name='filedescriptor',
+    sql_name='__intrinsic_filedescriptor',
+    wrapping_sql_view=WrappingSqlView('filedescriptor'),
     columns=[
         C('ufd', CppInt64()),
         C('fd', CppInt64()),
@@ -684,7 +780,8 @@ number.'''
 EXP_MISSING_CHROME_PROC_TABLE = Table(
     python_module=__file__,
     class_name='ExpMissingChromeProcTable',
-    sql_name='experimental_missing_chrome_processes',
+    sql_name='__intrinsic_experimental_missing_chrome_processes',
+    wrapping_sql_view=WrappingSqlView('experimental_missing_chrome_processes'),
     columns=[
         C('upid', CppUint32()),
         C('reliable_from', CppOptional(CppInt64())),
@@ -717,7 +814,8 @@ CPU_FREQ_TABLE = Table(
 CLOCK_SNAPSHOT_TABLE = Table(
     python_module=__file__,
     class_name='ClockSnapshotTable',
-    sql_name='clock_snapshot',
+    sql_name='__intrinsic_clock_snapshot',
+    wrapping_sql_view=WrappingSqlView('clock_snapshot'),
     columns=[
         C(
             'ts',
@@ -737,7 +835,7 @@ CLOCK_SNAPSHOT_TABLE = Table(
           cpp_access=CppAccess.READ,
           cpp_access_duration=CppAccessDuration.POST_FINALIZATION),
         C('snapshot_id', CppUint32()),
-        C('machine_id', CppOptional(CppTableId(MACHINE_TABLE))),
+        C('machine_id', CppTableId(MACHINE_TABLE)),
     ],
     tabledoc=TableDoc(
         doc='''
@@ -766,50 +864,6 @@ otherwise.''',
                 ''',
         }))
 
-TRACE_FILE_TABLE = Table(
-    python_module=__file__,
-    class_name='TraceFileTable',
-    sql_name='__intrinsic_trace_file',
-    columns=[
-        C('parent_id', CppOptional(CppSelfTableId())),
-        C('name', CppOptional(CppString())),
-        C('size', CppInt64(), cpp_access=CppAccess.READ_AND_LOW_PERF_WRITE),
-        C(
-            'trace_type',
-            CppString(),
-            cpp_access=CppAccess.READ_AND_LOW_PERF_WRITE,
-        ),
-        C(
-            'processing_order',
-            CppOptional(CppInt64()),
-            sql_access=SqlAccess.HIGH_PERF,
-            cpp_access=CppAccess.READ_AND_HIGH_PERF_WRITE,
-        ),
-    ],
-    wrapping_sql_view=WrappingSqlView('trace_file'),
-    tabledoc=TableDoc(
-        doc='''
-            Metadata related to the trace file parsed. Note the order in which
-            the files appear in this table corresponds to the order in which
-            they are read and sent to the tokenization stage.
-        ''',
-        group='Misc',
-        columns={
-            'parent_id':
-                '''
-                  Parent file. E.g. files contained in a zip file will point to
-                  the zip file.
-                ''',
-            'name':
-                '''File name, if known, NULL otherwise''',
-            'size':
-                '''Size in bytes''',
-            'trace_type':
-                '''Trace type''',
-            'processing_order':
-                '''In which order where the files were processed.''',
-        }))
-
 BUILD_FLAGS_TABLE = Table(
     python_module=__file__,
     class_name='BuildFlagsTable',
@@ -851,11 +905,10 @@ TRACE_IMPORT_LOGS_TABLE = Table(
     class_name='TraceImportLogsTable',
     sql_name='__intrinsic_trace_import_logs',
     columns=[
-        C('trace_id', CppUint32()),
+        C('trace_id', CppTableId(TRACE_FILE_TABLE)),
         C('ts', CppOptional(CppInt64())),
         C('byte_offset', CppOptional(CppInt64())),
-        C('severity', CppString()),
-        C('name', CppString()),
+        C('stat_key', CppInt64()),
         C(
             'arg_set_id',
             CppOptional(CppUint32()),
