@@ -83,6 +83,8 @@ WINSCOPE_TRACE_RECT_TABLE = Table(
         C('is_visible', CppInt64()),
         C('opacity', CppOptional(CppDouble())),
         C('transform_id', CppTableId(WINSCOPE_TRANSFORM_TABLE)),
+        C('border_width', CppOptional(CppDouble())),
+        C('border_color', CppOptional(CppUint32())),
     ],
     tabledoc=TableDoc(
         doc='WinscopeTraceRect',
@@ -104,6 +106,10 @@ WINSCOPE_TRACE_RECT_TABLE = Table(
                 'Opacity',
             'transform_id':
                 'Used to match trace rect to transform in __intrinsic_winscope_transform',
+            'border_width':
+                'Border width',
+            'border_color':
+                'Border color',
         }))
 
 INPUTMETHOD_CLIENTS_TABLE = Table(
@@ -190,7 +196,8 @@ INPUTMETHOD_SERVICE_TABLE = Table(
 SURFACE_FLINGER_LAYERS_SNAPSHOT_TABLE = Table(
     python_module=__file__,
     class_name='SurfaceFlingerLayersSnapshotTable',
-    sql_name='surfaceflinger_layers_snapshot',
+    sql_name='__intrinsic_surfaceflinger_layers_snapshot',
+    wrapping_sql_view=WrappingSqlView('surfaceflinger_layers_snapshot'),
     columns=[
         C('ts', CppInt64(), ColumnFlag.SORTED),
         C(
@@ -259,7 +266,8 @@ SURFACE_FLINGER_DISPLAY_TABLE = Table(
 SURFACE_FLINGER_LAYER_TABLE = Table(
     python_module=__file__,
     class_name='SurfaceFlingerLayerTable',
-    sql_name='surfaceflinger_layer',
+    sql_name='__intrinsic_surfaceflinger_layer',
+    wrapping_sql_view=WrappingSqlView('surfaceflinger_layer'),
     columns=[
         C('snapshot_id', CppTableId(SURFACE_FLINGER_LAYERS_SNAPSHOT_TABLE)),
         C(
@@ -349,7 +357,8 @@ WINSCOPE_FILL_REGION_TABLE = Table(
 SURFACE_FLINGER_TRANSACTIONS_TABLE = Table(
     python_module=__file__,
     class_name='SurfaceFlingerTransactionsTable',
-    sql_name='surfaceflinger_transactions',
+    sql_name='__intrinsic_surfaceflinger_transactions',
+    wrapping_sql_view=WrappingSqlView('surfaceflinger_transactions'),
     columns=[
         C('ts', CppInt64(), ColumnFlag.SORTED),
         C(
@@ -571,7 +580,8 @@ VIEWCAPTURE_INTERNED_DATA_TABLE = Table(
 WINDOW_MANAGER_SHELL_TRANSITIONS_TABLE = Table(
     python_module=__file__,
     class_name='WindowManagerShellTransitionsTable',
-    sql_name='window_manager_shell_transitions',
+    sql_name='__intrinsic_window_manager_shell_transitions',
+    wrapping_sql_view=WrappingSqlView('window_manager_shell_transitions'),
     columns=[
         C('ts', CppInt64(), cpp_access=CppAccess.READ_AND_LOW_PERF_WRITE),
         C('transition_id', CppInt64()),
@@ -610,6 +620,21 @@ WINDOW_MANAGER_SHELL_TRANSITIONS_TABLE = Table(
             CppOptional(CppInt64()),
             cpp_access=CppAccess.READ_AND_LOW_PERF_WRITE,
         ),
+        C(
+            'wm_abort_time_ns',
+            CppOptional(CppInt64()),
+            cpp_access=CppAccess.READ_AND_LOW_PERF_WRITE,
+        ),
+        C(
+            'merge_time_ns',
+            CppOptional(CppInt64()),
+            cpp_access=CppAccess.READ_AND_LOW_PERF_WRITE,
+        ),
+        C(
+            'create_time_ns',
+            CppOptional(CppInt64()),
+            cpp_access=CppAccess.READ_AND_LOW_PERF_WRITE,
+        ),
         C(
             'handler',
             CppOptional(CppInt64()),
@@ -658,6 +683,12 @@ WINDOW_MANAGER_SHELL_TRANSITIONS_TABLE = Table(
                 'Transition finish time',
             'shell_abort_time_ns':
                 'Transition shell abort time',
+            'wm_abort_time_ns':
+                'Transition wm abort time',
+            'merge_time_ns':
+                'Transition merge time',
+            'create_time_ns':
+                'Transition create time',
             'handler':
                 'Handler id',
             'status':
@@ -673,7 +704,9 @@ WINDOW_MANAGER_SHELL_TRANSITIONS_TABLE = Table(
 WINDOW_MANAGER_SHELL_TRANSITION_HANDLERS_TABLE = Table(
     python_module=__file__,
     class_name='WindowManagerShellTransitionHandlersTable',
-    sql_name='window_manager_shell_transition_handlers',
+    sql_name='__intrinsic_window_manager_shell_transition_handlers',
+    wrapping_sql_view=WrappingSqlView(
+        'window_manager_shell_transition_handlers'),
     columns=[
         C('handler_id', CppInt64()),
         C('handler_name', CppString()),
@@ -793,7 +826,7 @@ WINDOW_MANAGER_WINDOW_CONTAINER_TABLE = Table(
         ),
         C(
             'base64_proto_id',
-            CppUint32(),
+            CppOptional(CppUint32()),
             cpp_access=CppAccess.READ_AND_LOW_PERF_WRITE,
             cpp_access_duration=CppAccessDuration.POST_FINALIZATION,
         ),
@@ -829,7 +862,9 @@ WINDOW_MANAGER_WINDOW_CONTAINER_TABLE = Table(
         ),
         C(
             'container_type',
-            CppString(),
+            CppOptional(CppString()),
+            cpp_access=CppAccess.READ_AND_LOW_PERF_WRITE,
+            cpp_access_duration=CppAccessDuration.POST_FINALIZATION,
         ),
         C(
             'name_override',
@@ -868,7 +903,8 @@ WINDOW_MANAGER_WINDOW_CONTAINER_TABLE = Table(
 PROTOLOG_TABLE = Table(
     python_module=__file__,
     class_name='ProtoLogTable',
-    sql_name='protolog',
+    sql_name='__intrinsic_protolog',
+    wrapping_sql_view=WrappingSqlView('protolog'),
     columns=[
         C(
             'ts',
