@@ -23,6 +23,8 @@
 
 #include "perfetto/base/logging.h"
 #include "perfetto/ext/base/crash_keys.h"
+#include "perfetto/ext/base/file_utils.h"
+#include "perfetto/ext/base/unix_socket.h"
 #include "perfetto/ext/base/utils.h"
 #include "perfetto/ext/base/watchdog.h"
 #include "perfetto/ext/base/weak_ptr.h"
@@ -51,6 +53,7 @@
 #include "src/traced/probes/statsd_client/statsd_binder_data_source.h"
 #include "src/traced/probes/sys_stats/sys_stats_data_source.h"
 #include "src/traced/probes/system_info/system_info_data_source.h"
+#include "src/traced/probes/user_list/user_list_data_source.h"
 
 namespace perfetto {
 namespace {
@@ -298,6 +301,18 @@ ProbesProducer::CreateDSInstance<SystemInfoDataSource>(
       std::make_unique<CpuFreqInfo>());
 }
 
+template <>
+std::unique_ptr<ProbesDataSource>
+ProbesProducer::CreateDSInstance<UserListDataSource>(
+    TracingSessionID session_id,
+    const DataSourceConfig& config) {
+  auto buffer_id = static_cast<BufferID>(config.target_buffer());
+  return std::unique_ptr<ProbesDataSource>(new UserListDataSource(
+      config, session_id,
+      endpoint_->CreateTraceWriter(buffer_id,
+                                   perfetto::BufferExhaustedPolicy::kStall)));
+}
+
 template <>
 std::unique_ptr<ProbesDataSource>
 ProbesProducer::CreateDSInstance<InitialDisplayStateDataSource>(
@@ -369,6 +384,7 @@ constexpr const DataSourceTraits kAllDataSources[] = {
 #endif
     Ds<SysStatsDataSource>(),
     Ds<SystemInfoDataSource>(),
+    Ds<UserListDataSource>(),
 };
 
 }  // namespace
@@ -376,6 +392,7 @@ constexpr const DataSourceTraits kAllDataSources[] = {
 void ProbesProducer::OnConnect() {
   PERFETTO_DCHECK(state_ == kConnecting);
   state_ = kConnected;
+  sock_inotify_.reset();
   ResetConnectionBackoff();
   PERFETTO_LOG("Connected to the service");
 
@@ -419,13 +436,23 @@ void ProbesProducer::OnConnect() {
 void ProbesProducer::OnDisconnect() {
   PERFETTO_DCHECK(state_ == kConnected || state_ == kConnecting);
   PERFETTO_LOG("Disconnected from tracing service");
+
   if (state_ == kConnected)
     return task_runner_->PostTask([this] { this->Restart(); });
 
   state_ = kNotConnected;
   IncreaseConnectionBackoff();
-  task_runner_->PostDelayedTask([this] { this->Connect(); },
-                                connection_backoff_ms_);
+
+  auto reconnect_task = [weak_this = weak_factory_.GetWeakPtr()] {
+    if (weak_this && weak_this->state_ == kNotConnected)
+      weak_this->Connect();
+  };
+
+  task_runner_->PostDelayedTask(reconnect_task, connection_backoff_ms_);
+  if (!sock_inotify_) {
+    sock_inotify_ = base::WatchUnixSocketCreation(task_runner_, socket_name_,
+                                                  reconnect_task);
+  }
 }
 
 void ProbesProducer::SetupDataSource(DataSourceInstanceID instance_id,
