@@ -357,7 +357,7 @@ void ProducerIPCClientImpl::OnServiceRequest(
 #endif
     if (use_shmem_emulation_) {
       PERFETTO_CHECK(!ipc_shared_memory);
-      // Need to create an emulated shmem buffer when the transport deosn't
+      // Need to create an emulated shmem buffer when the transport doesn't
       // support it.
       ipc_shared_memory = InProcessSharedMemory::Create(
           /*size=*/InProcessSharedMemory::kShmemEmulationSize);
@@ -474,6 +474,11 @@ void ProducerIPCClientImpl::RegisterTraceWriter(uint32_t writer_id,
         "Cannot RegisterTraceWriter(), not connected to tracing service");
     return;
   }
+  if (use_shmem_emulation_ &&
+      smb_scraping_mode_ == TracingService::ProducerSMBScrapingMode::kEnabled) {
+    writers_for_scraping_[static_cast<WriterID>(writer_id)] =
+        static_cast<BufferID>(target_buffer);
+  }
   protos::gen::RegisterTraceWriterRequest req;
   req.set_trace_writer_id(writer_id);
   req.set_target_buffer(target_buffer);
@@ -488,6 +493,10 @@ void ProducerIPCClientImpl::UnregisterTraceWriter(uint32_t writer_id) {
         "Cannot UnregisterTraceWriter(), not connected to tracing service");
     return;
   }
+  if (use_shmem_emulation_ &&
+      smb_scraping_mode_ == TracingService::ProducerSMBScrapingMode::kEnabled) {
+    writers_for_scraping_.erase(static_cast<WriterID>(writer_id));
+  }
   protos::gen::UnregisterTraceWriterRequest req;
   req.set_trace_writer_id(writer_id);
   producer_port_->UnregisterTraceWriter(
@@ -594,7 +603,23 @@ bool ProducerIPCClientImpl::IsShmemProvidedByProducer() const {
 }
 
 void ProducerIPCClientImpl::NotifyFlushComplete(FlushRequestID req_id) {
-  return shared_memory_arbiter_->NotifyFlushComplete(req_id);
+  shared_memory_arbiter_->NotifyFlushComplete(req_id);
+
+  // NB: For producers using SMB emulation, the actual value of
+  // ProducerSMBScrapingMode::kDefault in the service-side is unknown on the
+  // producer-side. Therefore we only do producer-side SMB scraping if the
+  // scraping mode is explicitly enabled.
+  if (use_shmem_emulation_ &&
+      smb_scraping_mode_ == TracingService::ProducerSMBScrapingMode::kEnabled) {
+    // Producer-side SMB scraping should occur after the flush is complete.
+    // Most often the NotifyFlushComplete method is called at the end of a
+    // producer's Flush method which also very commonly posts a flush pending
+    // commit data request to the producer IPC client's task runner. Posting
+    // the SMB scraping task into the task runner should push the SMB scraping
+    // after the pending flush data request.
+    shared_memory_arbiter_->ScrapeEmulatedSharedMemoryBuffer(
+        writers_for_scraping_);
+  }
 }
 
 SharedMemory* ProducerIPCClientImpl::shared_memory() const {
