@@ -940,7 +940,7 @@ TraceBufferV1::TraceBufferV1(CloneCtor, const TraceBufferV1& src)
   //    old behavior.
   // 2. To support properly cloning of write_into_file sessions (b/382209797).
   index_ = ChunkMap(src.index_);
-  if (!base::flags::buffer_clone_preserve_read_iter) {
+  if constexpr (!PERFETTO_FLAGS(BUFFER_CLONE_PRESERVE_READ_ITER)) {
     for (auto& kv : index_) {
       ChunkMeta& chunk_meta = kv.second;
       chunk_meta.num_fragments_read = 0;
