@@ -389,7 +389,7 @@ class AndroidMetrics(TestSuite):
         wattson_app_startup_rails {
           metric_version: 4
           power_model_version: 1
-          is_crude_estimate: false
+          is_crude_estimate: 0
           period_info {
             period_id: 1
             period_dur: 385136434
@@ -457,7 +457,7 @@ class AndroidMetrics(TestSuite):
         wattson_trace_rails {
           metric_version: 4
           power_model_version: 1
-          is_crude_estimate: false
+          is_crude_estimate: 0
           period_info {
             period_id: 1
             period_dur: 61793079818
@@ -501,7 +501,7 @@ class AndroidMetrics(TestSuite):
         wattson_trace_rails {
           metric_version: 4
           power_model_version: 1
-          is_crude_estimate: true
+          is_crude_estimate: 1
           period_info {
             period_id: 1
             period_dur: 16532191699
@@ -587,7 +587,7 @@ class AndroidMetrics(TestSuite):
         wattson_markers_rails {
           metric_version: 4
           power_model_version: 1
-          is_crude_estimate: false
+          is_crude_estimate: 0
           period_info {
             period_id: 1
             period_dur: 2031888905
