@@ -172,6 +172,7 @@ def add_actual_surface_frame_events(ts,
                                     token,
                                     cookie=None,
                                     jank=None,
+                                    jank_score=0,
                                     on_time_finish_override=None,
                                     display_frame_token_override=None,
                                     layer_name=LAYER):
@@ -195,7 +196,8 @@ def add_actual_surface_frame_events(ts,
       gpu_composition=0,
       jank_type=jank_type,
       prediction_type=3,
-      layer_name=layer_name)
+      layer_name=layer_name,
+      jank_score=jank_score)
   trace.add_frame_end_event(ts=ts + dur, cookie=100002 + cookie)
 
 
@@ -283,6 +285,21 @@ trace.add_atrace_counter(
     ts=150_200_000, tid=PID, pid=PID, buf="J<FIRST_CUJ>#missedAppFrames", cnt=5)
 trace.add_atrace_counter(
     ts=150_300_000, tid=PID, pid=PID, buf="J<FIRST_CUJ>#missedSfFrames", cnt=1)
+trace.add_atrace_counter(
+    ts=150_300_000, tid=PID, pid=PID, buf="J<FIRST_CUJ>#totalAnimTime", cnt=500)
+trace.add_atrace_counter(
+    ts=150_300_000,
+    tid=PID,
+    pid=PID,
+    buf="J<FIRST_CUJ>#weightedAppJank",
+    cnt=225)
+trace.add_atrace_counter(
+    ts=150_300_000,
+    tid=PID,
+    pid=PID,
+    buf="J<FIRST_CUJ>#weightedSfJank",
+    cnt=421)
+
 trace.add_atrace_counter(
     ts=150_400_000,
     tid=PID,
@@ -320,6 +337,24 @@ trace.add_atrace_counter(
     pid=PID,
     buf="J<SHADE_ROW_EXPAND>#maxSuccessiveMissedFrames",
     cnt=5)
+trace.add_atrace_counter(
+    ts=950_300_000,
+    tid=PID,
+    pid=PID,
+    buf="J<SHADE_ROW_EXPAND>#totalAnimTime",
+    cnt=250)
+trace.add_atrace_counter(
+    ts=950_300_000,
+    tid=PID,
+    pid=PID,
+    buf="J<SHADE_ROW_EXPAND>#weightedAppJank",
+    cnt=1025)
+trace.add_atrace_counter(
+    ts=950_300_000,
+    tid=PID,
+    pid=PID,
+    buf="J<SHADE_ROW_EXPAND>#weightedSfJank",
+    cnt=142)
 trace.add_atrace_counter(
     ts=950_400_000,
     tid=PID,
@@ -673,34 +708,36 @@ add_expected_display_frame_events(ts=1_016_000_000, dur=20_000_000, token=20)
 add_actual_display_frame_events(ts=1_016_000_000, dur=10_000_000, token=20)
 
 add_expected_surface_frame_events(ts=8_000_000, dur=20_000_000, token=20)
-add_actual_surface_frame_events(ts=8_000_000, dur=28_000_000, token=20, jank=66)
+add_actual_surface_frame_events(
+    ts=8_000_000, dur=28_000_000, token=20, jank=66, jank_score=10)
 
 add_expected_display_frame_events(ts=1_032_000_000, dur=16_000_000, token=30)
 add_actual_display_frame_events(ts=1_032_000_000, dur=16_000_000, token=30)
 
 add_expected_surface_frame_events(ts=30_000_000, dur=20_000_000, token=30)
 add_actual_surface_frame_events(
-    ts=30_000_000, dur=25_000_000, token=30, jank=64)
+    ts=30_000_000, dur=25_000_000, token=30, jank=64, jank_score=2)
 
 add_expected_display_frame_events(ts=1_048_000_000, dur=16_000_000, token=40)
 add_actual_display_frame_events(ts=1_048_000_000, dur=16_000_000, token=40)
 
 add_expected_surface_frame_events(ts=40_000_000, dur=20_000_000, token=40)
 add_actual_surface_frame_events(
-    ts=40_000_000, dur=40_000_000, token=40, jank=64)
+    ts=40_000_000, dur=40_000_000, token=40, jank=64, jank_score=3)
 
 add_expected_display_frame_events(ts=1_070_000_000, dur=16_000_000, token=60)
 add_actual_display_frame_events(ts=1_070_000_000, dur=16_000_000, token=60)
 
 add_expected_surface_frame_events(ts=70_000_000, dur=20_000_000, token=60)
 add_actual_surface_frame_events(
-    ts=70_000_000, dur=10_000_000, token=60, jank=64)
+    ts=70_000_000, dur=10_000_000, token=60, jank=64, jank_score=4)
 add_actual_surface_frame_events(
     ts=70_000_000,
     dur=20_000_000,
     token=60,
     cookie=62,
     jank=64,
+    jank_score=5,
     # second layer produced frame later so was picked up by the next SF frame
     display_frame_token_override=190)
 
@@ -709,7 +746,7 @@ add_actual_display_frame_events(ts=1_100_000_000, dur=16_000_000, token=90)
 
 add_expected_surface_frame_events(ts=100_000_000, dur=20_000_000, token=90)
 add_actual_surface_frame_events(
-    ts=100_000_000, dur=23_000_000, token=90, jank=64)
+    ts=100_000_000, dur=23_000_000, token=90, jank=64, jank_score=6)
 
 add_expected_display_frame_events(ts=1_200_000_000, dur=16_000_000, token=100)
 add_actual_display_frame_events(
@@ -717,7 +754,7 @@ add_actual_display_frame_events(
 
 add_expected_surface_frame_events(ts=200_000_000, dur=20_000_000, token=100)
 add_actual_surface_frame_events(
-    ts=200_000_000, dur=22_000_000, token=100, jank=34)
+    ts=200_000_000, dur=22_000_000, token=100, jank=34, jank_score=7)
 
 add_expected_surface_frame_events(ts=300_000_000, dur=20_000_000, token=110)
 add_actual_surface_frame_events(
@@ -728,6 +765,7 @@ add_actual_surface_frame_events(
     dur=80_000_000,
     token=110,
     jank=64,
+    jank_score=8,
     layer_name="TX - JankyLayer#1")
 
 add_expected_surface_frame_events(ts=400_000_000, dur=20_000_000, token=120)
@@ -736,23 +774,24 @@ add_actual_surface_frame_events(
     dur=61_000_000,
     token=120,
     jank=128,
+    jank_score=9,
     on_time_finish_override=1)
 
 # Multiple layers but only one of them janked (the one we care about)
 add_expected_surface_frame_events(ts=500_000_000, dur=20_000_000, token=130)
 add_actual_surface_frame_events(ts=500_000_000, dur=2_000_000, token=130)
 add_actual_surface_frame_events(
-    ts=550_000_000, dur=6_000_000, token=130, cookie=132, jank=64)
+    ts=550_000_000, dur=6_000_000, token=130, cookie=132, jank=64, jank_score=1)
 
 # Single layer but actual frame event is slighly after doFrame start
 add_expected_surface_frame_events(ts=600_000_000, dur=20_000_000, token=140)
 add_actual_surface_frame_events(
-    ts=608_600_000, dur=17_000_000, token=140, jank=64)
+    ts=608_600_000, dur=17_000_000, token=140, jank=64, jank_score=11)
 
 # Surface flinger stuffing frame not classified as missed
 add_expected_surface_frame_events(ts=650_000_000, dur=20_000_000, token=145)
 add_actual_surface_frame_events(
-    ts=650_000_000, dur=20_000_000, token=145, jank=512)
+    ts=650_000_000, dur=20_000_000, token=145, jank=512, jank_score=12)
 
 add_expected_surface_frame_events(ts=700_000_000, dur=20_000_000, token=150)
 add_actual_surface_frame_events(ts=700_500_000, dur=14_500_000, token=150)
@@ -762,6 +801,6 @@ add_expected_surface_frame_events(ts=800_000_000, dur=20_000_000, token=160)
 
 add_expected_surface_frame_events(ts=1_100_000_000, dur=20_000_000, token=1000)
 add_actual_surface_frame_events(
-    ts=1_100_000_000, dur=500_000_000, token=1000, jank=64)
+    ts=1_100_000_000, dur=500_000_000, token=1000, jank=64, jank_score=13)
 
 sys.stdout.buffer.write(trace.trace.SerializeToString())
