@@ -14,7 +14,7 @@
 -- limitations under the License.
 
 SELECT
-  EXTRACT_ARG(s.arg_set_id, 'track_event.log_message') AS log_message,
+  EXTRACT_ARG(s.arg_set_id, 'track_event.log_message.message') AS log_message,
   EXTRACT_ARG(s.arg_set_id, 'track_event.log_message.function_name') AS function_name,
   EXTRACT_ARG(s.arg_set_id, 'track_event.log_message.file_name') AS file_name,
   EXTRACT_ARG(s.arg_set_id, 'track_event.log_message.line_number') AS line_number
