@@ -1181,7 +1181,7 @@ class GenericKernelParser(TestSuite):
         """,
         out=Csv("""
         "ts","cpu","value","name","type","machine_id"
-        359831239274,0,1500000.000000,"cpufreq","cpu_frequency","[NULL]"
+        359831239274,0,1500000.000000,"cpufreq","cpu_frequency",0
         360831239274,1,2500000.000000,"cpufreq","cpu_frequency",1
         """))
 
@@ -1325,6 +1325,12 @@ class GenericKernelParser(TestSuite):
                 tid: 5678
                 pid: 22
                 comm: "task2"
+              },
+              {
+                tid: 9012
+                pid: 33
+                comm: "idle"
+                is_idle: true
               }
             ]
           }
@@ -1345,4 +1351,5 @@ class GenericKernelParser(TestSuite):
         0,0,"swapper",0,1,1
         1,1234,"task1",2,1,0
         2,5678,"task2",3,0,0
+        3,9012,"idle",4,0,1
         """))
