@@ -1618,3 +1618,217 @@ class JsonParser(TestSuite):
           "02 03 ",2000,2000
           "Routine Identifier",2000,2000
         """))
+
+  def test_json_thread_state(self):
+    return DiffTestBlueprint(
+        trace=Json('''
+          {
+            "traceEvents": [
+              {
+                "name": "process_name",
+                "ph": "M",
+                "pid": 100,
+                "args": {"name": "TestProcess"}
+              },
+              {
+                "name": "thread_name",
+                "ph": "M",
+                "pid": 100,
+                "tid": 101,
+                "args": {"name": "TestThread"}
+              },
+              {
+                "name": "Running",
+                "cat": "thread_state",
+                "ph": "T",
+                "ts": 1000,
+                "dur": 5000,
+                "pid": 100,
+                "tid": 101,
+                "args": {
+                  "cpu": 0
+                }
+              },
+              {
+                "name": "S",
+                "cat": "thread_state",
+                "ph": "T",
+                "ts": 6000,
+                "dur": 10000,
+                "pid": 100,
+                "tid": 101,
+                "args": {
+                  "io_wait": 0
+                }
+              },
+              {
+                "name": "D",
+                "cat": "thread_state",
+                "ph": "T",
+                "ts": 16000,
+                "dur": 8000,
+                "pid": 100,
+                "tid": 101,
+                "args": {
+                  "io_wait": 1,
+                  "blocked_function": "wait_on_page_locked"
+                }
+              },
+              {
+                "name": "R",
+                "cat": "thread_state",
+                "ph": "T",
+                "ts": 24000,
+                "dur": 2000,
+                "pid": 100,
+                "tid": 101
+              },
+              {
+                "name": "Running",
+                "cat": "thread_state",
+                "ph": "T",
+                "ts": 26000,
+                "dur": 4000,
+                "pid": 100,
+                "tid": 101,
+                "args": {
+                  "cpu": 1
+                }
+              }
+            ]
+          }
+        '''),
+        query='''
+          SELECT
+            ts.ts,
+            ts.dur,
+            ts.state,
+            ts.io_wait,
+            ts.blocked_function,
+            thread.tid,
+            process.pid
+          FROM thread_state ts
+          JOIN thread USING (utid)
+          JOIN process USING (upid)
+          ORDER BY ts.ts
+        ''',
+        out=Csv("""
+        "ts","dur","state","io_wait","blocked_function","tid","pid"
+        1000000,5000000,"Running","[NULL]","[NULL]",101,100
+        6000000,10000000,"S",0,"[NULL]",101,100
+        16000000,8000000,"D",1,"wait_on_page_locked",101,100
+        24000000,2000000,"R","[NULL]","[NULL]",101,100
+        26000000,4000000,"Running","[NULL]","[NULL]",101,100
+        """))
+
+  def test_json_thread_state_sched(self):
+    return DiffTestBlueprint(
+        trace=Json('''
+          {
+            "traceEvents": [
+              {
+                "name": "process_name",
+                "ph": "M",
+                "pid": 100,
+                "args": {"name": "TestProcess"}
+              },
+              {
+                "name": "thread_name",
+                "ph": "M",
+                "pid": 100,
+                "tid": 101,
+                "args": {"name": "TestThread"}
+              },
+              {
+                "name": "Running",
+                "cat": "thread_state",
+                "ph": "T",
+                "ts": 1000,
+                "dur": 5000,
+                "pid": 100,
+                "tid": 101,
+                "args": {
+                  "cpu": 0
+                }
+              },
+              {
+                "name": "S",
+                "cat": "thread_state",
+                "ph": "T",
+                "ts": 6000,
+                "dur": 10000,
+                "pid": 100,
+                "tid": 101,
+                "args": {
+                  "io_wait": 0
+                }
+              },
+              {
+                "name": "Running",
+                "cat": "thread_state",
+                "ph": "T",
+                "ts": 26000,
+                "dur": 4000,
+                "pid": 100,
+                "tid": 101,
+                "args": {
+                  "cpu": 1
+                }
+              }
+            ]
+          }
+        '''),
+        query='''
+          SELECT
+            sched.ts,
+            sched.dur,
+            cpu.cpu,
+            thread.tid,
+            process.pid
+          FROM sched
+          JOIN thread USING (utid)
+          JOIN process USING (upid)
+          JOIN cpu USING (ucpu)
+          ORDER BY sched.ts
+        ''',
+        out=Csv("""
+        "ts","dur","cpu","tid","pid"
+        1000000,5000000,0,101,100
+        26000000,4000000,1,101,100
+        """))
+
+  def test_json_metadata_missing_or_invalid_ts(self):
+    return DiffTestBlueprint(
+        trace=Json('''
+          [
+            {
+              "ph": "M",
+              "pid": 100,
+              "name": "process_name",
+              "args": { "name": "MissingTs" }
+            },
+            {
+              "ph": "M",
+              "pid": 101,
+              "ts": null,
+              "name": "process_name",
+              "args": { "name": "NullTs" }
+            },
+            {
+              "ph": "M",
+              "pid": 102,
+              "ts": "invalid",
+              "name": "process_name",
+              "args": { "name": "InvalidTs" }
+            }
+          ]
+        '''),
+        query='''
+          SELECT pid, name FROM process WHERE name IS NOT NULL ORDER BY pid
+        ''',
+        out=Csv("""
+          "pid","name"
+          100,"MissingTs"
+          101,"NullTs"
+          102,"InvalidTs"
+        """))
