@@ -483,6 +483,54 @@ class Parsing(TestSuite):
         """,
         out=Path('android_sched_and_ps_stats.out'))
 
+  def test_shadow_buf_stats(self):
+    return DiffTestBlueprint(
+        trace=TextProto(r"""
+          packet {
+            trusted_uid: 158158
+            trusted_packet_sequence_id: 1
+            trace_stats {
+              buffer_stats {
+                buffer_size: 131072
+                bytes_written: 55459840
+                bytes_overwritten: 55328768
+                bytes_read: 131072
+                padding_bytes_cleared: 0
+                chunks_written: 13540
+                chunks_overwritten: 13508
+                chunks_read: 32
+                write_wrap_count: 423
+                patches_succeeded: 1861
+                patches_failed: 12349
+                readaheads_succeeded: 11
+                shadow_buffer_stats {
+                  packets_seen: 57
+                  packets_in_both: 55
+                  packets_only_v1: 0
+                  packets_only_v2: 0
+                  patches_attempted: 13499
+                  v1_patches_succeeded: 1150
+                  v2_patches_succeeded: 1150
+                }
+              }
+            }
+          }
+        """),
+        query="""
+        SELECT name, source, value
+        FROM stats WHERE name GLOB 'traced_buf_v2s*';
+        """,
+        out=Csv("""
+        "name","source","value"
+        "traced_buf_v2s_packets_seen","trace",57
+        "traced_buf_v2s_packets_in_both","trace",55
+        "traced_buf_v2s_packets_only_v1","trace",0
+        "traced_buf_v2s_packets_only_v2","trace",0
+        "traced_buf_v2s_patches_attempted","trace",13499
+        "traced_buf_v2s_v1_patches_succeeded","trace",1150
+        "traced_buf_v2s_v2_patches_succeeded","trace",1150
+        """))
+
   # Syscalls
   def test_sys_syscall(self):
     return DiffTestBlueprint(
@@ -581,20 +629,24 @@ class Parsing(TestSuite):
         }
         packet {
           system_info {
-            android_build_fingerprint: "the fingerprint"
+            android_build_fingerprint: "google/coral/coral:12/SP1A.210812.015/7679548:userdebug/dev-keys"
           }
         }
         """),
         query="""
-        SELECT name, str_value FROM metadata WHERE str_value IS NOT NULL ORDER BY name;
+        SELECT name, str_value, int_value FROM metadata WHERE str_value IS NOT NULL OR int_value IS NOT NULL ORDER BY name;
         """,
         out=Csv("""
-        "name","str_value"
-        "android_build_fingerprint","the fingerprint"
+        "name","str_value","int_value"
+        "android_build_fingerprint","google/coral/coral:12/SP1A.210812.015/7679548:userdebug/dev-keys","[NULL]"
+        "android_incremental_build","7679548","[NULL]"
+        "android_sdk_version","[NULL]",31
         "trace_config_pbtxt","trace_uuid_msb: 1314564453825188563
-        trace_uuid_lsb: -6605018796207623390"
-        "trace_type","proto"
-        "trace_uuid","123e4567-e89b-12d3-a456-426655443322"
+        trace_uuid_lsb: -6605018796207623390","[NULL]"
+        "trace_size_bytes","[NULL]",124
+        "trace_time_clock_id","[NULL]",6
+        "trace_type","proto","[NULL]"
+        "trace_uuid","123e4567-e89b-12d3-a456-426655443322","[NULL]"
         """))
 
   def test_triggers_packets_trigger_packet_trace(self):
@@ -979,6 +1031,38 @@ class Parsing(TestSuite):
         5,"com.google.android.gms",10100,1,"com.google.android.gms",1234,0
         """))
 
+  # package metadata with user type
+  def test_package_metadata_user_type(self):
+    return DiffTestBlueprint(
+        trace=DataPath('user_package_metadata.pftrace'),
+        query="""
+        INCLUDE PERFETTO MODULE android.process_metadata;
+
+        SELECT
+          package_name,
+          process_name,
+          uid,
+          user_id,
+          user_type
+        FROM
+          android_process_metadata
+        WHERE package_name IS NOT NULL
+        LIMIT 10;
+        """,
+        out=Csv("""
+        "package_name","process_name","uid","user_id","user_type"
+        "com.google.android.bluetooth","/vendor/bin/hw/android.hardware.bluetooth@1.1-service.synabtlinux",1002,0,"android.os.usertype.system.HEADLESS"
+        "com.google.android.googlequicksearchbox","com.google.android.googlequicksearchbox:search",10180,0,"android.os.usertype.system.HEADLESS"
+        "com.google.android.apps.turbo","com.google.android.apps.turbo:aab",10164,11,"android.os.usertype.full.SECONDARY"
+        "com.google.android.apps.messaging","com.google.android.apps.messaging",10171,11,"android.os.usertype.full.SECONDARY"
+        "com.google.android.googlequicksearchbox","com.google.android.googlequicksearchbox:googleapp",10180,0,"android.os.usertype.system.HEADLESS"
+        "com.google.android.googlequicksearchbox","com.google.android.googlequicksearchbox:interactor",10180,11,"android.os.usertype.full.SECONDARY"
+        "com.android.systemui","com.android.systemui",10252,0,"android.os.usertype.system.HEADLESS"
+        "com.google.android.apps.nest.dockmanager.app","com.google.android.apps.nest.dockmanager.app",10187,0,"android.os.usertype.system.HEADLESS"
+        "com.google.android.flipendo","com.google.android.flipendo",10268,0,"android.os.usertype.system.HEADLESS"
+        "com.google.android.pixelsystemservice","com.google.android.pixelsystemservice",10269,0,"android.os.usertype.system.HEADLESS"
+        """))
+
   # Flow events importing from json
   def test_flow_events_json_v1(self):
     return DiffTestBlueprint(
@@ -1668,25 +1752,25 @@ class Parsing(TestSuite):
         out=Csv("""
         "ts","cpu","utid","machine_id"
         5230311628979,0,1,1
-        5230315517287,0,11,1
+        5230315517287,0,50,1
         5230315524649,0,1,1
-        5230315676788,0,10,1
+        5230315676788,0,49,1
         5230315684911,0,1,1
-        5230319663217,0,10,1
+        5230319663217,0,49,1
         5230319684310,0,1,1
-        5230323692459,0,10,1
-        5230323726976,0,11,1
+        5230323692459,0,49,1
+        5230323726976,0,50,1
         5230323764556,0,1,1
-        5230327702466,0,10,1
+        5230327702466,0,49,1
         5230327736100,0,1,1
-        5230331761483,0,10,1
-        5230331800905,0,11,1
+        5230331761483,0,49,1
+        5230331800905,0,50,1
         5230331837332,0,1,1
-        5230421799455,0,10,1
+        5230421799455,0,49,1
         5230421810047,0,1,1
-        5230422048874,0,1306,"[NULL]"
-        5230422153284,0,1306,"[NULL]"
-        5230425693562,0,10,1
+        5230422048874,0,1305,0
+        5230422153284,0,1305,0
+        5230425693562,0,49,1
         """))
 
   # Kernel idle tasks created by /sbin/init should be filtered.
