@@ -476,3 +476,120 @@ class Profiling(TestSuite):
         273574993553025,4672142.000000,0,0,0
         273574993553025,1144537.000000,0,0,0
         """))
+
+  def test_perf_sample_counter_set(self):
+    return DiffTestBlueprint(
+        trace=TextProto(r"""
+        packet {
+          clock_snapshot {
+            primary_trace_clock: BUILTIN_CLOCK_BOOTTIME
+            clocks {
+              clock_id: 6
+              timestamp: 273574904041306
+            }
+            clocks {
+              clock_id: 2
+              timestamp: 1737644264730557105
+            }
+            clocks {
+              clock_id: 4
+              timestamp: 106208706668231
+            }
+            clocks {
+              clock_id: 1
+              timestamp: 1737644264734590878
+            }
+            clocks {
+              clock_id: 3
+              timestamp: 106208710702167
+            }
+            clocks {
+              clock_id: 5
+              timestamp: 106208710702494
+            }
+          }
+          trusted_packet_sequence_id: 1
+        }
+        packet {
+          first_packet_on_sequence: true
+          timestamp: 273574983771490
+          timestamp_clock_id: 6
+          sequence_flags: 1
+          trace_packet_defaults {
+            timestamp_clock_id: 3
+            perf_sample_defaults {
+              timebase {
+                frequency: 100
+                counter: SW_CPU_CLOCK
+              }
+              followers {
+                counter: HW_CPU_CYCLES
+              }
+              followers {
+                counter: HW_INSTRUCTIONS
+              }
+            }
+          }
+          trusted_packet_sequence_id: 4
+          previous_packet_dropped: true
+        }
+        packet {
+          interned_data {
+            build_ids {
+              iid: 0
+              str: ""
+            }
+            mapping_paths {
+              iid: 0
+              str: ""
+            }
+            function_names {
+              iid: 0
+              str: ""
+            }
+          }
+          sequence_flags: 2
+          trusted_packet_sequence_id: 4
+        }
+        packet {
+          sequence_flags: 2
+          timestamp: 106208800213886
+          interned_data {
+            callstacks {
+              iid: 1
+            }
+          }
+          perf_sample {
+            cpu: 0
+            pid: 0
+            tid: 0
+            cpu_mode: MODE_KERNEL
+            timebase_count: 10020141
+            follower_counts: 4672142
+            follower_counts: 1144537
+            callstack_iid: 1
+          }
+          trusted_packet_sequence_id: 4
+        }
+        """),
+        query="""
+        -- Test the counter_set_id on perf_sample and the
+        -- __intrinsic_perf_counter_set table.
+        select
+          ps.id as sample_id,
+          ps.counter_set_id,
+          pcs.counter_id,
+          c.value
+        from
+          __intrinsic_perf_sample ps
+          join __intrinsic_perf_counter_set pcs
+            on ps.counter_set_id = pcs.perf_counter_set_id
+          join counter c on c.id = pcs.counter_id
+        order by ps.id, pcs.counter_id
+        """,
+        out=Csv("""
+        "sample_id","counter_set_id","counter_id","value"
+        0,0,0,10020141.000000
+        0,0,1,4672142.000000
+        0,0,2,1144537.000000
+        """))
