@@ -739,14 +739,14 @@ class LinuxCpu(TestSuite):
          """,
         out=Csv("""
           "ts","machine_id","state","cpu","idle_percentage","total_residency","time_slice"
-          200001000000,"[NULL]","C8",0,10.000000,100.000000,1000
-          200002000000,"[NULL]","C8",0,10.000000,100.000000,1000
-          200001000000,"[NULL]","C8",1,10.000000,100.000000,1000
-          200002000000,"[NULL]","C8",1,10.000000,100.000000,1000
-          200001000000,"[NULL]","C0",0,90.000000,900.000000,1000
-          200001000000,"[NULL]","C0",1,90.000000,900.000000,1000
-          200002000000,"[NULL]","C0",0,90.000000,900.000000,1000
-          200002000000,"[NULL]","C0",1,90.000000,900.000000,1000
+          200001000000,0,"C8",0,10.000000,100.000000,1000
+          200002000000,0,"C8",0,10.000000,100.000000,1000
+          200001000000,0,"C8",1,10.000000,100.000000,1000
+          200002000000,0,"C8",1,10.000000,100.000000,1000
+          200001000000,0,"C0",0,90.000000,900.000000,1000
+          200001000000,0,"C0",1,90.000000,900.000000,1000
+          200002000000,0,"C0",0,90.000000,900.000000,1000
+          200002000000,0,"C0",1,90.000000,900.000000,1000
          """))
 
   def test_linux_cpu_idle_time_in_state(self):
@@ -798,8 +798,8 @@ class LinuxCpu(TestSuite):
          """,
         out=Csv("""
           "ts","machine_id","state","idle_percentage","total_residency","time_slice"
-          200001000000,"[NULL]","C0",90.000000,900.000000,1000
-          200001000000,"[NULL]","C8",10.000000,100.000000,1000
-          200002000000,"[NULL]","C0",90.000000,900.000000,1000
-          200002000000,"[NULL]","C8",10.000000,100.000000,1000
+          200001000000,0,"C0",90.000000,900.000000,1000
+          200001000000,0,"C8",10.000000,100.000000,1000
+          200002000000,0,"C0",90.000000,900.000000,1000
+          200002000000,0,"C8",10.000000,100.000000,1000
          """))
