@@ -15,8 +15,7 @@
 import m from 'mithril';
 import {AsyncLimiter} from '../base/async_limiter';
 import {AsyncDisposableStack} from '../base/disposable_stack';
-import {assertExists} from '../base/logging';
-import {Monitor} from '../base/monitor';
+import {assertExists} from '../base/assert';
 import {uuidv4Sql} from '../base/uuid';
 import {Engine} from '../trace_processor/engine';
 import {
@@ -42,6 +41,7 @@ import {
 import {Trace} from '../public/trace';
 import {sqliteString} from '../base/string_utils';
 import {SharedAsyncDisposable} from '../base/shared_disposable';
+import {Monitor} from '../base/monitor';
 
 export interface QueryFlamegraphColumn {
   // The name of the column in SQL.
@@ -69,6 +69,10 @@ export interface QueryFlamegraphMetric {
   // the user suffixed with this.
   readonly unit: string;
 
+  // Label for the name column in copy stack table and tooltip.
+  // Examples: "Symbol", "Slice", "Class". Defaults to "Name".
+  readonly nameColumnLabel?: string;
+
   // SQL statement which need to be run in preparation for being able to execute
   // `statement`.
   readonly dependencySql?: string;
@@ -116,8 +120,18 @@ export interface QueryFlamegraphMetric {
   readonly optionalMarker?: FlamegraphOptionalMarker;
 }
 
-export interface QueryFlamegraphState {
-  state: FlamegraphState;
+export interface MetricsFromTableOrSubqueryOptions {
+  readonly tableOrSubquery: string;
+  readonly tableMetrics: ReadonlyArray<{
+    name: string;
+    unit: string;
+    columnName: string;
+  }>;
+  readonly dependencySql?: string;
+  readonly unaggregatableProperties?: ReadonlyArray<QueryFlamegraphColumn>;
+  readonly aggregatableProperties?: ReadonlyArray<AggQueryFlamegraphColumn>;
+  readonly optionalActions?: ReadonlyArray<FlamegraphOptionalAction>;
+  readonly nameColumnLabel?: string;
 }
 
 // Given a table and columns on those table (corresponding to metrics),
@@ -128,45 +142,61 @@ export interface QueryFlamegraphState {
 // columns specified by `tableMetrics[].name`, `unaggregatableProperties` and
 // `aggregatableProperties`.
 export function metricsFromTableOrSubquery(
-  tableOrSubquery: string,
-  tableMetrics: ReadonlyArray<{name: string; unit: string; columnName: string}>,
-  dependencySql?: string,
-  unaggregatableProperties?: ReadonlyArray<QueryFlamegraphColumn>,
-  aggregatableProperties?: ReadonlyArray<AggQueryFlamegraphColumn>,
-  optionalActions?: ReadonlyArray<FlamegraphOptionalAction>,
+  opts: MetricsFromTableOrSubqueryOptions,
 ): QueryFlamegraphMetric[] {
   const metrics = [];
-  for (const {name, unit, columnName} of tableMetrics) {
+  for (const {name, unit, columnName} of opts.tableMetrics) {
     metrics.push({
       name,
       unit,
-      dependencySql,
+      nameColumnLabel: opts.nameColumnLabel,
+      dependencySql: opts.dependencySql,
       statement: `
         select *, ${columnName} as value
-        from ${tableOrSubquery}
+        from ${opts.tableOrSubquery}
       `,
-      unaggregatableProperties,
-      aggregatableProperties,
-      optionalActions,
+      unaggregatableProperties: opts.unaggregatableProperties,
+      aggregatableProperties: opts.aggregatableProperties,
+      optionalActions: opts.optionalActions,
     });
   }
   return metrics;
 }
 
+interface QueryFlamegraphAttrs {
+  // The metrics to display in the flamegraph. If undefined, the flamegraph will
+  // show a loading state.
+  readonly metrics?: ReadonlyArray<QueryFlamegraphMetric>;
+
+  // The current state of the flamegraph (filters, view, selected metric, etc).
+  readonly state?: FlamegraphState;
+
+  // Callback invoked when the flamegraph state changes (e.g., user changes
+  // filters, selects a different metric, etc).
+  readonly onStateChange: (state: FlamegraphState) => void;
+}
+
+export interface QueryFlamegraphWithMetrics {
+  flamegraph: QueryFlamegraph;
+  metrics: ReadonlyArray<QueryFlamegraphMetric>;
+}
+
 // A Perfetto UI component which wraps the `Flamegraph` widget and fetches the
 // data for the widget by querying an `Engine`.
 export class QueryFlamegraph implements AsyncDisposable {
   private data?: FlamegraphQueryData;
-  private readonly selMonitor = new Monitor([() => this.state.state]);
   private readonly queryLimiter = new AsyncLimiter();
   private readonly dependencies: ReadonlyArray<
     SharedAsyncDisposable<AsyncDisposable>
   >;
+  private lastAttrs?: QueryFlamegraphAttrs;
+  private monitor = new Monitor([
+    () => this.lastAttrs?.metrics,
+    () => this.lastAttrs?.state,
+  ]);
 
   constructor(
     private readonly trace: Trace,
-    private readonly metrics: ReadonlyArray<QueryFlamegraphMetric>,
-    private state: QueryFlamegraphState,
     dependencies: ReadonlyArray<AsyncDisposable> = [],
   ) {
     this.dependencies = dependencies.map((d) => SharedAsyncDisposable.wrap(d));
@@ -178,37 +208,52 @@ export class QueryFlamegraph implements AsyncDisposable {
     }
   }
 
-  render() {
-    if (this.selMonitor.ifStateChanged()) {
-      const metric = assertExists(
-        this.metrics.find(
-          (x) => this.state.state.selectedMetricName === x.name,
-        ),
-      );
-      const engine = this.trace.engine;
-      const state = this.state;
+  render(attrs: QueryFlamegraphAttrs) {
+    const {metrics, state, onStateChange} = attrs;
+    this.lastAttrs = attrs;
+    if (this.monitor.ifStateChanged()) {
       this.data = undefined;
-      this.queryLimiter.schedule(async () => {
-        this.data = undefined;
-        // Clone all the dependencies to make sure the the are not dropped while
-        // this function is running, adding them to the trash to make sure they
-        // are disposed after this function returns, but note this won't
-        // actually drop the tables unless this class instances have also been
-        // disposed due to the SharedAsyncDisposable logic.
-        await using trash = new AsyncDisposableStack();
-        for (const dependency of this.dependencies ?? []) {
-          trash.use(dependency.clone());
-        }
-        this.data = await computeFlamegraphTree(engine, metric, state.state);
-      });
+      if (metrics && state) {
+        this.fetchData(metrics, state);
+      }
     }
     return m(Flamegraph, {
-      metrics: this.metrics,
+      metrics: metrics ?? [],
       data: this.data,
-      state: this.state.state,
-      onStateChange: (state) => {
-        this.state.state = state;
+      state: state ?? {
+        view: {kind: 'TOP_DOWN'},
+        selectedMetricName: '',
+        filters: [],
       },
+      onStateChange,
+    });
+  }
+
+  fetchData(
+    metrics: ReadonlyArray<QueryFlamegraphMetric>,
+    state: FlamegraphState,
+  ) {
+    const metric = assertExists(
+      metrics.find((x) => state.selectedMetricName === x.name),
+    );
+    const engine = this.trace.engine;
+    this.queryLimiter.schedule(async () => {
+      this.data = undefined;
+      // Clone all the dependencies to make sure the the are not dropped while
+      // this function is running, adding them to the trash to make sure they
+      // are disposed after this function returns, but note this won't
+      // actually drop the tables unless this class instances have also been
+      // disposed due to the SharedAsyncDisposable logic.
+      await using trash = new AsyncDisposableStack();
+      for (const dependency of this.dependencies ?? []) {
+        // If the dependency is disposed, it means that we have already ended
+        // up cleaning up the object so none of this matters. Just return.
+        if (dependency.isDisposed) {
+          return;
+        }
+        trash.use(dependency.clone());
+      }
+      this.data = await computeFlamegraphTree(engine, metric, state);
     });
   }
 }
@@ -481,7 +526,7 @@ async function computeFlamegraphTree(
   const nodes = [];
   for (; it.valid(); it.next()) {
     const properties = new Map<string, FlamegraphPropertyDefinition>();
-    for (const a of [...agg, ...unagg]) {
+    for (const a of unagg) {
       const r = it.get(a.name);
       if (r !== null) {
         const value = r as string;
@@ -489,6 +534,19 @@ async function computeFlamegraphTree(
           displayName: a.displayName,
           value,
           isVisible: a.isVisible ? a.isVisible(value) : true,
+          isAggregatable: false,
+        });
+      }
+    }
+    for (const a of agg) {
+      const r = it.get(a.name);
+      if (r !== null) {
+        const value = r as string;
+        properties.set(a.name, {
+          displayName: a.displayName,
+          value,
+          isVisible: a.isVisible ? a.isVisible(value) : true,
+          isAggregatable: true,
         });
       }
     }
@@ -542,10 +600,19 @@ async function computeFlamegraphTree(
 }
 
 function makeSqlFilter(x: string) {
-  if (x.startsWith('^') && x.endsWith('$')) {
-    return x.slice(1, -1);
+  const hasStart = x.startsWith('^');
+  const hasEnd = x.endsWith('$');
+  const pattern = x.slice(hasStart ? 1 : 0, hasEnd ? -1 : undefined);
+
+  if (hasStart && hasEnd) {
+    return pattern; // Exact match
+  } else if (hasStart) {
+    return `${pattern}%`; // Starts with
+  } else if (hasEnd) {
+    return `%${pattern}`; // Ends with
+  } else {
+    return `%${pattern}%`; // Contains
   }
-  return `%${x}%`;
 }
 
 function getPivotFilter(
