@@ -12,7 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-import {assertExists, assertTrue} from '../base/logging';
+import {assertExists, assertTrue} from '../base/assert';
 import {time, Time, TimeSpan} from '../base/time';
 import {cacheTrace} from './cache_manager';
 import {
@@ -49,7 +49,8 @@ import {Router} from '../core/router';
 import {TraceInfoImpl} from './trace_info_impl';
 import {base64Decode} from '../base/string_utils';
 import {parseUrlCommands} from './command_manager';
-import {isStartupCommandAllowed} from './startup_command_allowlist';
+import {HighPrecisionTimeSpan} from '../base/high_precision_time_span';
+import {sha1} from '../base/hash';
 
 const ENABLE_CHROME_RELIABLE_RANGE_ZOOM_FLAG = featureFlags.register({
   id: 'enableChromeReliableRangeZoom',
@@ -143,11 +144,10 @@ async function createEngine(
   if (app.httpRpc.newEngineMode === 'USE_HTTP_RPC_IF_AVAILABLE') {
     useRpc = (await HttpRpcEngine.checkConnection()).connected;
   }
+
   const descriptorBlobs: Uint8Array[] = [];
-  if (app.extraParsingDescriptors.length > 0) {
-    for (const b64Str of app.extraParsingDescriptors) {
-      descriptorBlobs.push(base64Decode(b64Str));
-    }
+  for (const b64Str of await app.protoDescriptors()) {
+    descriptorBlobs.push(base64Decode(b64Str));
   }
   let engine;
   if (useRpc) {
@@ -224,7 +224,8 @@ async function loadTraceIntoEngine(
     assertTrue(engine instanceof HttpRpcEngine);
     await engine.restoreInitialTables();
   }
-  for (const p of app.extraSqlPackages) {
+
+  for (const p of await app.sqlPackages()) {
     await engine.registerSqlPackages(p);
   }
 
@@ -232,33 +233,23 @@ async function loadTraceIntoEngine(
   const trace = new TraceImpl(app, engine, traceDetails);
   app.setActiveTrace(trace);
 
+  const hasJsonTrace = traceDetails.traceTypes.includes('json');
+
   const visibleTimeSpan = await computeVisibleTime(
     traceDetails.start,
     traceDetails.end,
-    trace.traceInfo.traceType === 'json',
+    hasJsonTrace,
     engine,
   );
 
-  trace.timeline.updateVisibleTime(visibleTimeSpan);
+  const newViewport = HighPrecisionTimeSpan.fromTime(
+    visibleTimeSpan.start,
+    visibleTimeSpan.end,
+  );
+  trace.timeline.setVisibleWindow(newViewport);
 
   const cacheUuid = traceDetails.cached ? traceDetails.uuid : '';
-
-  // Attempt to preserve the existing page, only add/change the local_cache_key.
-  //
-  // This is so that if the user opens a trace from a URL or has navigated to a
-  // page before opening a trace, we stay on that page. This allows links to
-  // e.g. #!/explore to work as expected.
-  //
-  // Only navigate to the timeline page if we are currently on the home page.
-  const route = Router.parseUrl(window.location.href);
-
-  let nextPage = route.page;
-  if (route.page === '/' || route.page === '') {
-    // Currently on the home page, navigate to the timeline page.
-    nextPage = '/viewer';
-  }
-
-  Router.navigate(`#!${nextPage}${route.subpage}?local_cache_key=${cacheUuid}`);
+  Router.navigate(`#!/viewer?local_cache_key=${cacheUuid}`);
 
   // Make sure the helper views are available before we start adding tracks.
   await includeSummaryTables(trace);
@@ -280,10 +271,7 @@ async function loadTraceIntoEngine(
 
   // Trace Processor doesn't support the reliable range feature for JSON
   // traces.
-  if (
-    trace.traceInfo.traceType !== 'json' &&
-    ENABLE_CHROME_RELIABLE_RANGE_ANNOTATION_FLAG.get()
-  ) {
+  if (!hasJsonTrace && ENABLE_CHROME_RELIABLE_RANGE_ANNOTATION_FLAG.get()) {
     const reliableRangeStart = await computeTraceReliableRangeStart(engine);
     if (reliableRangeStart > 0) {
       trace.notes.addNote({
@@ -337,7 +325,7 @@ async function loadTraceIntoEngine(
 
     // Set allowlist checking during startup if enforcement enabled
     if (enforceAllowlist) {
-      app.commands.setAllowlistCheck(isStartupCommandAllowed);
+      app.commands.setExecutingStartupCommands(true);
     }
 
     try {
@@ -356,7 +344,7 @@ async function loadTraceIntoEngine(
       }
     } finally {
       // Always restore default (allow all) behavior when done
-      app.commands.setAllowlistCheck(() => true);
+      app.commands.setExecutingStartupCommands(false);
     }
   }
 
@@ -443,6 +431,18 @@ async function computeVisibleTime(
   return new TimeSpan(visibleStart, visibleEnd);
 }
 
+// TODO(sashwinbalaji): Move session UUID generation to TraceProcessor.
+// computeGlobalUuid is a temporary measure to ensure multi-trace sessions
+// have a unique cache key. This prevents collisions where a multi-trace
+// session (e.g. a ZIP) would otherwise reuse the cache entry of its first
+// component trace if that trace was previously opened individually.
+async function computeGlobalUuid(uuids: string[]): Promise<string> {
+  if (uuids.length === 0) return '';
+  if (uuids.length === 1) return uuids[0];
+  const sortedUuids = [...uuids].sort();
+  return await sha1(sortedUuids.join(';'));
+}
+
 async function getTraceInfo(
   engine: Engine,
   app: AppImpl,
@@ -531,16 +531,27 @@ async function getTraceInfo(
       break;
   }
 
-  const traceType = await getTraceType(engine);
+  const traceTypes = await getTraceTypes(engine);
 
   const hasFtrace =
     (await engine.query(`select * from ftrace_event limit 1`)).numRows() > 0;
 
-  const uuidRes = await engine.query(`select str_value as uuid from metadata
-    where name = 'trace_uuid'`);
-  // trace_uuid can be missing from the TP tables if the trace is empty or in
-  // other similar edge cases.
-  const uuid = uuidRes.numRows() > 0 ? uuidRes.firstRow({uuid: STR}).uuid : '';
+  // UUIDs are not always present. We fall back to a combination of trace_id and trace_type
+  // to ensure we still have a unique component for the session hash.
+  const uuidRes = await engine.query(`
+    INCLUDE PERFETTO MODULE std.traceinfo.trace;
+    select ifnull(trace_uuid, trace_id || '-' || trace_type) as uuid from _metadata_by_trace
+  `);
+  const uuids: string[] = [];
+  for (
+    const itUuid = uuidRes.iter({uuid: STR});
+    itUuid.valid();
+    itUuid.next()
+  ) {
+    uuids.push(itUuid.uuid);
+  }
+  const uuid = await computeGlobalUuid(uuids);
+
   updateStatus(app, 'Caching trace...');
   const cached = await cacheTrace(traceSource, uuid);
 
@@ -557,7 +568,7 @@ async function getTraceInfo(
     unixOffset,
     importErrors: await getTraceErrors(engine),
     source: traceSource,
-    traceType,
+    traceTypes,
     hasFtrace,
     uuid,
     cached,
@@ -565,13 +576,19 @@ async function getTraceInfo(
   };
 }
 
-async function getTraceType(engine: Engine) {
-  const result = await engine.query(
-    `select str_value from metadata where name = 'trace_type'`,
-  );
+async function getTraceTypes(engine: Engine): Promise<string[]> {
+  const result = await engine.query(`
+    INCLUDE PERFETTO MODULE std.traceinfo.trace;
+    select distinct trace_type as str_value
+    from _metadata_by_trace
+  `);
 
-  if (result.numRows() === 0) return undefined;
-  return result.firstRow({str_value: STR}).str_value;
+  const traceTypes: string[] = [];
+  const it = result.iter({str_value: STR});
+  for (; it.valid(); it.next()) {
+    traceTypes.push(it.str_value);
+  }
+  return traceTypes;
 }
 
 async function getTraceTimeBounds(engine: Engine): Promise<TimeSpan> {
