@@ -12,6 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+import m from 'mithril';
 import {z} from 'zod';
 import {copyToClipboard} from '../../base/clipboard';
 import {formatTimezone, Time, time, timezoneOffsetMap} from '../../base/time';
@@ -19,7 +20,7 @@ import {exists} from '../../base/utils';
 import {JsonSettingsEditor} from '../../components/json_settings_editor';
 import {addQueryResultsTab} from '../../components/query_table/query_result_tab';
 import {AppImpl} from '../../core/app_impl';
-import {commandInvocationSchema} from '../../core/command_manager';
+import {commandInvocationSchema, macroSchema} from '../../core/command_manager';
 import {featureFlags} from '../../core/feature_flags';
 import {OmniboxMode} from '../../core/omnibox_manager';
 import {
@@ -42,7 +43,7 @@ import {DurationPrecision, TimestampFormat} from '../../public/timeline';
 import {getTimeSpanOfSelectionOrVisibleWindow} from '../../public/utils';
 import {Workspace} from '../../public/workspace';
 import {showModal} from '../../widgets/modal';
-import {assertExists} from '../../base/logging';
+import {assertExists} from '../../base/assert';
 import {Setting} from '../../public/settings';
 import {toggleHelp} from '../../frontend/help_modal';
 
@@ -132,13 +133,23 @@ function getOrPromptForTimestamp(tsRaw: unknown): time | undefined {
   return promptForTimestamp('Enter a timestamp');
 }
 
-const macroSchema = z.record(z.array(commandInvocationSchema));
-type MacroConfig = z.infer<typeof macroSchema>;
+// Type alias for macro array (inferred from the shared schema)
+const macrosConfigSchema = z.array(macroSchema);
+type MacrosConfig = z.infer<typeof macrosConfigSchema>;
+
+// Legacy macro schema (dictionary format) - deprecated, kept for migration
+export const legacyMacrosConfigSchema = z.record(
+  z.string(), // key: macro name
+  z.array(commandInvocationSchema).readonly(),
+);
+type LegacyMacrosConfig = z.infer<typeof legacyMacrosConfigSchema>;
 
 export default class CoreCommands implements PerfettoPlugin {
   static readonly id = 'dev.perfetto.CoreCommands';
 
-  static macrosSetting: Setting<MacroConfig> | undefined = undefined;
+  static macrosSetting: Setting<MacrosConfig> | undefined = undefined;
+  static legacyMacrosSetting: Setting<LegacyMacrosConfig> | undefined =
+    undefined;
 
   static onActivate(ctx: AppImpl) {
     // Register global commands (commands that are required even without a trace
@@ -168,20 +179,60 @@ export default class CoreCommands implements PerfettoPlugin {
       });
     }
 
-    const macroSettingsEditor = new JsonSettingsEditor<MacroConfig>({
-      schema: macroSchema,
+    // Register the new macros setting (array format)
+    const macroSettingsEditor = new JsonSettingsEditor<MacrosConfig>({
+      schema: macrosConfigSchema,
     });
     CoreCommands.macrosSetting = ctx.settings.register({
-      id: 'perfetto.CoreCommands#UserDefinedMacros',
+      id: 'perfetto.CoreCommands#Macros',
       name: 'Macros',
       description:
         'Custom command macros that execute multiple commands in sequence',
-      schema: macroSchema,
-      defaultValue: {},
+      schema: macrosConfigSchema,
+      defaultValue: [],
       requiresReload: true,
       render: (setting) => macroSettingsEditor.render(setting),
     });
 
+    // Register the legacy macros setting (dictionary format) - deprecated
+    CoreCommands.legacyMacrosSetting = ctx.settings.register({
+      id: 'perfetto.CoreCommands#UserDefinedMacros',
+      name: 'Macros (Legacy)',
+      description:
+        'Deprecated: This setting uses the old dictionary format. ' +
+        'Your macros have been automatically migrated to the new format above.',
+      schema: legacyMacrosConfigSchema,
+      defaultValue: {},
+      requiresReload: false,
+      render: () =>
+        m(
+          '.pf-legacy-macros-notice',
+          m(
+            'p',
+            'This setting is deprecated. Your macros have been automatically ' +
+              'migrated to the new "Macros" setting above.',
+          ),
+          m(
+            'p',
+            'Please use the new "Macros" setting to view and edit your macros.',
+          ),
+        ),
+    });
+
+    // Migration: if new setting is empty but legacy has data, migrate
+    const newMacros = CoreCommands.macrosSetting.get();
+    const legacyMacros = CoreCommands.legacyMacrosSetting.get();
+    if (newMacros.length === 0 && Object.keys(legacyMacros).length > 0) {
+      const migratedMacros: MacrosConfig = Object.entries(legacyMacros).map(
+        ([name, run]) => ({
+          id: `dev.perfetto.UserMacro.${name}`,
+          name,
+          run,
+        }),
+      );
+      CoreCommands.macrosSetting.set(migratedMacros);
+    }
+
     const input = document.createElement('input');
     input.classList.add('trace_file');
     input.setAttribute('type', 'file');
@@ -201,8 +252,9 @@ export default class CoreCommands implements PerfettoPlugin {
     });
     ctx.sidebar.addMenuItem({
       commandId: OPEN_TRACE_COMMAND_ID,
-      section: 'navigation',
+      section: 'trace_files',
       icon: 'folder_open',
+      sortOrder: 1,
     });
 
     const OPEN_LEGACY_COMMAND_ID = 'dev.perfetto.OpenTraceInLegacyUi';
@@ -217,7 +269,7 @@ export default class CoreCommands implements PerfettoPlugin {
     if (SHOW_OPEN_WITH_LEGACY_UI_BUTTON.get()) {
       ctx.sidebar.addMenuItem({
         commandId: OPEN_LEGACY_COMMAND_ID,
-        section: 'navigation',
+        section: 'trace_files',
         icon: 'filter_none',
       });
     }
@@ -235,18 +287,20 @@ export default class CoreCommands implements PerfettoPlugin {
     const app = AppImpl.instance;
 
     // Rgister macros from settings first.
-    registerMacros(ctx, assertExists(CoreCommands.macrosSetting).get());
+    const settingMacros = assertExists(CoreCommands.macrosSetting).get();
+    for (const macro of settingMacros) {
+      ctx.commands.registerMacro(macro);
+    }
 
     // Register the macros from extras at onTraceReady (the latest time
     // possible).
     ctx.onTraceReady.addListener(async (_) => {
-      // Await the promise: we've tried to be async as long as possible but
+      // Await the promises: we've tried to be async as long as possible but
       // now we need the extras to be loaded.
-      await app.extraLoadingPromise;
-      registerMacros(
-        ctx,
-        app.extraMacros.reduce((acc, macro) => ({...acc, ...macro}), {}),
-      );
+      const macros = await app.macros();
+      for (const macro of macros) {
+        ctx.commands.registerMacro(macro, macro.source);
+      }
     });
 
     ctx.commands.registerCommand({
@@ -346,7 +400,7 @@ export default class CoreCommands implements PerfettoPlugin {
       callback: (tsRaw: unknown) => {
         const ts = getOrPromptForTimestamp(tsRaw);
         if (ts !== undefined) {
-          ctx.timeline.panToTimestamp(ts);
+          ctx.timeline.panIntoView(ts, {align: 'center'});
         }
       },
     });
@@ -532,17 +586,20 @@ export default class CoreCommands implements PerfettoPlugin {
     ctx.commands.registerCommand({
       id: 'dev.perfetto.RunQueryAndShowTab',
       name: 'Runs an SQL query and opens results in a tab',
-      callback: async (rawSql: unknown) => {
+      callback: async (queryArg: unknown, titleArg?: unknown) => {
         const query =
-          typeof rawSql === 'string'
-            ? rawSql
+          typeof queryArg === 'string'
+            ? queryArg
             : await ctx.omnibox.prompt('Enter SQL...');
         if (!query) {
           return;
         }
+
+        const title = typeof titleArg === 'string' ? titleArg : 'Command Query';
+
         addQueryResultsTab(ctx, {
           query,
-          title: 'Command Query',
+          title,
         });
       },
     });
@@ -566,16 +623,10 @@ export default class CoreCommands implements PerfettoPlugin {
     ctx.commands.registerCommand({
       id: 'dev.perfetto.FocusSelection',
       name: 'Focus current selection',
-      callback: () => ctx.selection.scrollToSelection(),
+      callback: () => ctx.selection.scrollToSelection('focus'),
       defaultHotkey: 'F',
     });
 
-    ctx.commands.registerCommand({
-      id: 'dev.perfetto.ZoomOnSelection',
-      name: 'Zoom in on current selection',
-      callback: () => ctx.selection.zoomOnSelection(),
-    });
-
     ctx.commands.registerCommand({
       id: 'dev.perfetto.Deselect',
       name: 'Deselect',
@@ -698,13 +749,6 @@ export default class CoreCommands implements PerfettoPlugin {
       defaultHotkey: 'R',
     });
 
-    ctx.commands.registerCommand({
-      id: 'dev.perfetto.ToggleDrawer',
-      name: 'Toggle drawer',
-      defaultHotkey: 'Q',
-      callback: () => ctx.tabs.toggleTabPanelVisibility(),
-    });
-
     ctx.commands.registerCommand({
       id: 'dev.perfetto.CopyPinnedToWorkspace',
       name: 'Copy pinned tracks to workspace',
@@ -883,21 +927,3 @@ async function openWithLegacyUi(file: File) {
   }
   return await openInOldUIWithSizeCheck(file);
 }
-
-function registerMacros(trace: TraceImpl, config: MacroConfig) {
-  for (const [macroName, commands] of Object.entries(config)) {
-    trace.commands.registerCommand({
-      id: `dev.perfetto.UserMacro.${macroName}`,
-      name: macroName,
-      callback: async () => {
-        // Macros could run multiple commands, some of which might prompt the
-        // user in an optional way. But macros should be self-contained
-        // so we disable prompts during their execution.
-        using _ = trace.omnibox.disablePrompts();
-        for (const command of commands) {
-          await trace.commands.runCommand(command.id, ...command.args);
-        }
-      },
-    });
-  }
-}
