@@ -17,6 +17,7 @@ import {OmniboxMode} from '../../core/omnibox_manager';
 import {Trace} from '../../public/trace';
 import {PerfettoPlugin} from '../../public/plugin';
 import {AppImpl} from '../../core/app_impl';
+import {TraceImpl} from '../../core/trace_impl';
 import {getTimeSpanOfSelectionOrVisibleWindow} from '../../public/utils';
 import {exists, RequiredField} from '../../base/utils';
 import {LONG, NUM, NUM_NULL} from '../../trace_processor/query_result';
@@ -28,6 +29,7 @@ import {Time} from '../../base/time';
 export default class TrackUtilsPlugin implements PerfettoPlugin {
   static readonly id = 'dev.perfetto.TrackUtils';
   static dvorakSetting: Setting<boolean>;
+  static azertySetting: Setting<boolean>;
 
   static onActivate(app: App): void {
     TrackUtilsPlugin.dvorakSetting = app.settings.register({
@@ -41,35 +43,48 @@ export default class TrackUtilsPlugin implements PerfettoPlugin {
       requiresReload: true, // Hotkeys are registered on trace load.
     });
 
+    TrackUtilsPlugin.azertySetting = app.settings.register({
+      id: 'azertyMode',
+      defaultValue: false,
+      name: 'AZERTY layout',
+      description: 'Adjusts shortcuts for AZERTY keyboards (ZQSD navigation).',
+      schema: z.boolean(),
+      requiresReload: true,
+    });
+  }
+
+  async onTraceLoad(ctx: TraceImpl): Promise<void> {
+    ctx.commands.registerCommand({
+      id: 'dev.perfetto.ToggleDrawer',
+      name: 'Toggle drawer',
+      defaultHotkey: TrackUtilsPlugin.azertySetting.get() ? 'A' : 'Q',
+      callback: () => {
+        ctx.tabs.toggleTabPanelVisibility();
+      },
+    });
+
     // Register this command up front to block the print dialog from appearing
     // when pressing the hotkey before the trace is loaded.
-    app.commands.registerCommand({
+    ctx.commands.registerCommand({
       id: 'dev.perfetto.FindTrackByName',
       name: 'Find track by name',
       callback: async () => {
-        const trace = app.trace;
-        if (!trace) {
-          return;
-        }
-
-        const tracksWithUris = trace.currentWorkspace.flatTracksOrdered.filter(
+        const tracksWithUris = ctx.currentWorkspace.flatTracksOrdered.filter(
           (track) => track.uri !== undefined,
         ) as ReadonlyArray<RequiredField<TrackNode, 'uri'>>;
-        const track = await app.omnibox.prompt('Choose a track...', {
+        const track = await ctx.omnibox.prompt('Choose a track...', {
           values: tracksWithUris,
           getName: (track) => track.fullPath.join(' \u2023 '),
         });
         track &&
-          trace.selection.selectTrack(track.uri, {
+          ctx.selection.selectTrack(track.uri, {
             scrollToSelection: true,
           });
       },
       // This is analogous to the 'Find file' hotkey in VSCode.
       defaultHotkey: '!Mod+P',
     });
-  }
 
-  async onTraceLoad(ctx: Trace): Promise<void> {
     ctx.commands.registerCommand({
       id: 'dev.perfetto.RunQueryInSelectedTimeWindow',
       name: `Run query in selected time window`,
@@ -396,6 +411,36 @@ export default class TrackUtilsPlugin implements PerfettoPlugin {
       },
     });
 
+    ctx.commands.registerCommand({
+      id: 'dev.perfetto.AddNoteAtTimestamp',
+      name: 'Add note at nanosecond timestamp',
+      callback: async (timestampArg: unknown, noteTextArg: unknown) => {
+        const timestampStr =
+          typeof timestampArg === 'string'
+            ? timestampArg
+            : await ctx.omnibox.prompt('Enter timestamp...');
+        if (!timestampStr) return;
+
+        const noteText =
+          typeof noteTextArg === 'string'
+            ? noteTextArg
+            : await ctx.omnibox.prompt('Enter note text...');
+        if (noteText === undefined) return;
+
+        const timestamp = parseInt(timestampStr, 10);
+        if (isNaN(timestamp)) {
+          console.error(`invalid timestamp: ${timestampStr}`);
+          return;
+        }
+        const traceTime = Time.fromRaw(BigInt(timestamp));
+
+        ctx.notes.addNote({
+          timestamp: traceTime,
+          text: noteText,
+        });
+      },
+    });
+
     ctx.commands.registerCommand({
       id: 'dev.perfetto.SelectNextTrackEvent',
       name: 'Select next track event',
