@@ -44,7 +44,7 @@ import {exists, getOrCreate} from '../base/utils';
 import {classNames} from '../base/classnames';
 import {formatHotkey} from '../base/hotkeys';
 import {assetSrc} from '../base/assets';
-import {assertExists} from '../base/logging';
+import {assertExists} from '../base/assert';
 import {Icon} from '../widgets/icon';
 import {Button} from '../widgets/button';
 
@@ -99,7 +99,7 @@ function downloadTrace(trace: TraceImpl) {
   const filePickerAcceptTypes = [
     {
       description: 'Perfetto trace',
-      accept: {'*/*': ['.pftrace']},
+      accept: {'*/*': ['.pftrace', '.gz']},
     },
   ];
   if (src.type === 'URL') {
@@ -386,6 +386,21 @@ export class Sidebar implements m.ClassComponent {
       m(
         `header.pf-sidebar__channel--${getCurrentChannel()}`,
         m(`img[src=${assetSrc('assets/brand.png')}].pf-sidebar__brand`),
+        app.embedder.brandingBadge &&
+          m(
+            'span.pf-sidebar__branding-badge',
+            {style: {color: app.embedder.brandingBadge.color}},
+            app.embedder.brandingBadge.image
+              ? m('img.pf-sidebar__branding-badge-img', {
+                  src: app.embedder.brandingBadge.image,
+                })
+              : app.embedder.brandingBadge.icon &&
+                  m(Icon, {
+                    icon: app.embedder.brandingBadge.icon,
+                    className: 'pf-sidebar__branding-badge-icon',
+                  }),
+            app.embedder.brandingBadge.text,
+          ),
         m(Button, {
           icon: 'menu',
           className: 'pf-sidebar-button',
@@ -418,7 +433,8 @@ export class Sidebar implements m.ClassComponent {
         .filter((item) => item.section === sectionId),
     ];
 
-    // Add section-specific global and trace items
+    // Add section-specific global and trace items, and determine default collapsed state
+    let defaultCollapsed = false;
     switch (sectionId) {
       case 'current_trace':
         if (trace !== undefined) {
@@ -429,12 +445,14 @@ export class Sidebar implements m.ClassComponent {
         if (trace !== undefined) {
           allItems.push(...getConvertTraceItems(trace));
         }
+        defaultCollapsed = true;
         break;
       case 'support':
         allItems.push(...getSupportGlobalItems());
         if (trace !== undefined) {
           allItems.push(...getSupportTraceItems(trace));
         }
+        defaultCollapsed = true;
         break;
     }
 
@@ -445,7 +463,11 @@ export class Sidebar implements m.ClassComponent {
     // Don't render empty sections.
     if (menuItems.length === 0) return undefined;
 
-    const expanded = getOrCreate(this._sectionExpanded, sectionId, () => true);
+    const expanded = getOrCreate(
+      this._sectionExpanded,
+      sectionId,
+      () => !defaultCollapsed,
+    );
     return m(
       `section${expanded ? '.pf-sidebar__section--expanded' : ''}`,
       m(
