@@ -24,7 +24,6 @@
 
 import {hex} from 'color-convert';
 import m from 'mithril';
-import {canvasClip, canvasSave} from '../../base/canvas_utils';
 import {classNames} from '../../base/classnames';
 import {DisposableStack} from '../../base/disposable_stack';
 import {findRef, toHTMLElement} from '../../base/dom_utils';
@@ -32,11 +31,12 @@ import {
   HorizontalBounds,
   Rect2D,
   Size2D,
+  Transform2D,
   VerticalBounds,
 } from '../../base/geom';
 import {HighPrecisionTime} from '../../base/high_precision_time';
 import {HighPrecisionTimeSpan} from '../../base/high_precision_time_span';
-import {assertExists} from '../../base/logging';
+import {assertExists} from '../../base/assert';
 import {Time} from '../../base/time';
 import {TimeScale} from '../../base/time_scale';
 import {
@@ -46,6 +46,7 @@ import {
 import {PerfStats, runningStatStr} from '../../core/perf_stats';
 import {TraceImpl} from '../../core/trace_impl';
 import {TrackNode} from '../../public/workspace';
+import {SnapPoint} from '../../public/track';
 import {VirtualOverlayCanvas} from '../../widgets/virtual_overlay_canvas';
 import {
   COLOR_ACCENT,
@@ -73,15 +74,52 @@ import {Button, ButtonVariant} from '../../widgets/button';
 import {Intent} from '../../widgets/common';
 import {CursorTooltip} from '../../widgets/cursor_tooltip';
 import {CanvasColors} from '../../public/canvas_colors';
+import {Icons} from '../../base/semantic_icons';
+import {Renderer} from '../../base/renderer';
 
 const VIRTUAL_TRACK_SCROLLING = featureFlags.register({
   id: 'virtualTrackScrolling',
   name: 'Virtual track scrolling',
   description: `[Experimental] Use virtual scrolling in the timeline view to
     improve performance on large traces.`,
-  defaultValue: false,
+  defaultValue: true,
 });
 
+const WEBGL_RENDERING = featureFlags.register({
+  id: 'webglRendering',
+  name: 'WebGL rendering',
+  description: `Use WebGL for rendering track rectangles. Falls back to
+    Canvas 2D when disabled or unavailable.`,
+  defaultValue: true,
+});
+
+// Snap-to-boundaries feature constants
+const SNAP_THRESHOLD_PX = 15;
+const SNAP_ENABLED_DEFAULT = true;
+
+// Cache for CSS color to packed RGBA conversion
+const cssColorCache = new Map<string, number>();
+
+// Convert a CSS color string to packed RGBA (0xRRGGBBAA)
+function cssColorToRgba(cssColor: string): number {
+  const cached = cssColorCache.get(cssColor);
+  if (cached !== undefined) return cached;
+
+  // Use an offscreen canvas to parse CSS color
+  const canvas = document.createElement('canvas');
+  canvas.width = 1;
+  canvas.height = 1;
+  const ctx = canvas.getContext('2d')!;
+  ctx.fillStyle = cssColor;
+  ctx.fillRect(0, 0, 1, 1);
+  const imageData = ctx.getImageData(0, 0, 1, 1);
+  const [r, g, b, a] = imageData.data;
+  const packed = ((r << 24) | (g << 16) | (b << 8) | a) >>> 0;
+
+  cssColorCache.set(cssColor, packed);
+  return packed;
+}
+
 export interface TrackTreeViewAttrs {
   // Access to the trace, for accessing the track registry / selection manager.
   readonly trace: TraceImpl;
@@ -129,6 +167,8 @@ export class TrackTreeView implements m.ClassComponent<TrackTreeViewAttrs> {
   private areaDrag?: InProgressAreaSelection;
   private handleDrag?: InProgressHandleDrag;
   private canvasRect?: Rect2D;
+  private currentSnapPoint?: SnapPoint;
+  private snapEnabled = SNAP_ENABLED_DEFAULT;
 
   constructor({attrs}: m.Vnode<TrackTreeViewAttrs>) {
     this.trace = attrs.trace;
@@ -166,15 +206,22 @@ export class TrackTreeView implements m.ClassComponent<TrackTreeViewAttrs> {
       node: TrackNode,
       depth = 0,
       stickyTop = 0,
-    ): m.Children => {
+    ): {vnodes: m.Children; isVisible: boolean} => {
       // Skip nodes that don't match the filter and have no matching children.
-      if (!filterMatches(node)) return undefined;
+      if (!filterMatches(node)) return {vnodes: false, isVisible: false};
 
       if (node.headless) {
         // Headless nodes are invisible, just render children.
-        return node.children.map((track) => {
-          return renderTrack(track, depth, stickyTop);
-        });
+        const childNodes: m.Children = [];
+        let atLeastOneChildVisible = false;
+        for (const child of node.children) {
+          const {vnodes, isVisible} = renderTrack(child, depth, stickyTop);
+          childNodes.push(vnodes);
+          if (isVisible) {
+            atLeastOneChildVisible = true;
+          }
+        }
+        return {vnodes: childNodes, isVisible: atLeastOneChildVisible};
       }
 
       const trackView = new TrackView(trace, node, top);
@@ -188,28 +235,36 @@ export class TrackTreeView implements m.ClassComponent<TrackTreeViewAttrs> {
         ? stickyTop + trackView.height
         : stickyTop;
 
-      const children =
-        (node.expanded || filtersApplied) &&
-        node.hasChildren &&
-        node.children.map((track) =>
-          renderTrack(track, depth + 1, childStickyTop),
-        );
+      const childNodes: m.Children = [];
+      let atLeastOneChildVisible = false;
+      if ((node.expanded || filtersApplied) && node.hasChildren) {
+        for (const child of node.children) {
+          const {vnodes, isVisible} = renderTrack(
+            child,
+            depth + 1,
+            childStickyTop,
+          );
+          childNodes.push(vnodes);
+          if (isVisible) {
+            atLeastOneChildVisible = true;
+          }
+        }
+      }
 
-      const isTrackOnScreen = (() => {
-        if (VIRTUAL_TRACK_SCROLLING.get()) {
-          return this.canvasRect?.overlaps({
+      const isTrackOnScreen = VIRTUAL_TRACK_SCROLLING.get()
+        ? this.canvasRect?.overlaps({
             left: 0,
             right: 1,
-            ...trackView.verticalBounds,
-          });
-        } else {
-          return true;
-        }
-      })();
+            top: trackView.verticalBounds.top,
+            bottom: trackView.verticalBounds.bottom,
+          })
+        : true;
 
-      return trackView.renderDOM(
+      const isVisible = isTrackOnScreen || atLeastOneChildVisible;
+
+      const vnodes = trackView.renderDOM(
         {
-          lite: !Boolean(isTrackOnScreen),
+          lite: !Boolean(isVisible),
           scrollToOnCreate: scrollToNewTracks,
           reorderable: canReorderNodes,
           removable: canRemoveNodes,
@@ -223,11 +278,15 @@ export class TrackTreeView implements m.ClassComponent<TrackTreeViewAttrs> {
             this.hoveredTrackNode = undefined;
           },
         },
-        children,
+        childNodes,
       );
+
+      return {vnodes, isVisible};
     };
 
-    const trackVnodes = rootNode.children.map((track) => renderTrack(track));
+    const trackVnodes = rootNode.children
+      .map((track) => renderTrack(track))
+      .map(({vnodes}) => vnodes);
 
     // If there are no truthy vnode values, show "empty state" placeholder.
     if (trackVnodes.every((x) => !Boolean(x))) {
@@ -237,13 +296,13 @@ export class TrackTreeView implements m.ClassComponent<TrackTreeViewAttrs> {
           EmptyState,
           {
             className,
-            icon: 'filter_alt_off',
             title: `No tracks match track filter`,
           },
           m(Button, {
             intent: Intent.Primary,
             variant: ButtonVariant.Filled,
             label: 'Clear track filter',
+            icon: Icons.FilterOff,
             onclick: () => trace.tracks.filters.clearAll(),
           }),
         );
@@ -266,13 +325,15 @@ export class TrackTreeView implements m.ClassComponent<TrackTreeViewAttrs> {
         className: classNames(className, 'pf-track-tree'),
         overflowY: 'auto',
         overflowX: 'hidden',
-        onCanvasRedraw: ({ctx, virtualCanvasSize, canvasRect}) => {
+        enableWebGL: WEBGL_RENDERING.get(),
+        onCanvasRedraw: ({ctx, virtualCanvasSize, canvasRect, renderer}) => {
           this.drawCanvas(
             ctx,
             virtualCanvasSize,
             renderedTracks,
             canvasRect,
             rootNode,
+            renderer,
           );
 
           if (VIRTUAL_TRACK_SCROLLING.get()) {
@@ -360,6 +421,7 @@ export class TrackTreeView implements m.ClassComponent<TrackTreeViewAttrs> {
     renderedTracks: ReadonlyArray<TrackView>,
     floatingCanvasRect: Rect2D,
     rootNode: TrackNode,
+    renderer: Renderer,
   ) {
     const timelineRect = new Rect2D({
       left: TRACK_SHELL_WIDTH,
@@ -375,11 +437,15 @@ export class TrackTreeView implements m.ClassComponent<TrackTreeViewAttrs> {
 
     const start = performance.now();
 
-    // Save, translate & clip the canvas to the area of the timeline.
-    using _ = canvasSave(ctx);
-    canvasClip(ctx, timelineRect);
+    // Clip to the timeline area for WebGL rendering
+    using _clip = renderer.clip(
+      timelineRect.left,
+      timelineRect.top,
+      timelineRect.width,
+      timelineRect.height,
+    );
 
-    this.drawGridLines(ctx, timescale, timelineRect);
+    this.drawGridLines(renderer, timescale, timelineRect);
 
     const colors: CanvasColors = {
       COLOR_BORDER,
@@ -393,6 +459,7 @@ export class TrackTreeView implements m.ClassComponent<TrackTreeViewAttrs> {
       COLOR_TIMELINE_OVERLAY,
     };
 
+    // Render all track content (WebGL rectangles + Canvas 2D text)
     const tracksOnCanvas = this.drawTracks(
       renderedTracks,
       floatingCanvasRect,
@@ -401,6 +468,7 @@ export class TrackTreeView implements m.ClassComponent<TrackTreeViewAttrs> {
       timelineRect,
       visibleWindow,
       colors,
+      renderer,
     );
 
     renderFlows(this.trace, ctx, size, renderedTracks, rootNode, timescale);
@@ -419,32 +487,63 @@ export class TrackTreeView implements m.ClassComponent<TrackTreeViewAttrs> {
   }
 
   private drawGridLines(
-    ctx: CanvasRenderingContext2D,
+    renderer: Renderer,
     timescale: TimeScale,
-    size: Size2D,
+    timelineRect: Rect2D,
   ): void {
-    ctx.strokeStyle = COLOR_BORDER_SECONDARY;
-    ctx.lineWidth = 1;
+    if (timelineRect.width <= 0 || timescale.timeSpan.duration <= 0n) {
+      return;
+    }
 
-    if (size.width > 0 && timescale.timeSpan.duration > 0n) {
-      const maxMajorTicks = getMaxMajorTicks(size.width);
-      const offset = this.trace.timeline.getTimeAxisOrigin();
-      for (const {type, time} of generateTicks(
-        timescale.timeSpan.toTimeSpan(),
-        maxMajorTicks,
-        offset,
-      )) {
-        const px = Math.floor(timescale.timeToPx(time));
-        if (type === TickType.MAJOR) {
-          ctx.beginPath();
-          ctx.moveTo(px + 0.5, 0);
-          ctx.lineTo(px + 0.5, size.height);
-          ctx.stroke();
-        }
+    const maxMajorTicks = getMaxMajorTicks(timelineRect.width);
+    const offset = this.trace.timeline.getTimeAxisOrigin();
+
+    // Collect all major tick positions
+    const tickPositions: number[] = [];
+    for (const {type, time} of generateTicks(
+      timescale.timeSpan.toTimeSpan(),
+      maxMajorTicks,
+      offset,
+    )) {
+      if (type === TickType.MAJOR) {
+        tickPositions.push(Math.floor(timescale.timeToPx(time)));
       }
     }
+
+    if (tickPositions.length === 0) return;
+
+    // Create buffers for WebGL rendering
+    const count = tickPositions.length;
+    const xs = new Float32Array(count);
+    const ys = new Float32Array(count);
+    const ws = new Float32Array(count);
+    const colors = new Uint32Array(count);
+    const patterns = new Uint8Array(count);
+    const gridColor = cssColorToRgba(COLOR_BORDER_SECONDARY);
+
+    for (let i = 0; i < count; i++) {
+      xs[i] = tickPositions[i];
+      ys[i] = 0;
+      ws[i] = 1;
+      colors[i] = gridColor;
+      patterns[i] = 0;
+    }
+
+    renderer.drawRects(
+      {
+        xs,
+        ys,
+        ws,
+        h: timelineRect.height,
+        colors,
+        patterns,
+        count,
+      },
+      Transform2D.Identity,
+    );
   }
 
+  // Render all tracks - WebGL rectangles and Canvas 2D content in one pass
   private drawTracks(
     renderedTracks: ReadonlyArray<TrackView>,
     floatingCanvasRect: Rect2D,
@@ -453,6 +552,7 @@ export class TrackTreeView implements m.ClassComponent<TrackTreeViewAttrs> {
     timelineRect: Rect2D,
     visibleWindow: HighPrecisionTimeSpan,
     colors: CanvasColors,
+    renderer: Renderer,
   ) {
     let tracksOnCanvas = 0;
     for (const trackView of renderedTracks) {
@@ -471,6 +571,7 @@ export class TrackTreeView implements m.ClassComponent<TrackTreeViewAttrs> {
           this.perfStatsEnabled,
           this.trackPerfStats,
           colors,
+          renderer,
         );
         ++tracksOnCanvas;
       }
@@ -507,22 +608,40 @@ export class TrackTreeView implements m.ClassComponent<TrackTreeViewAttrs> {
                 new HighPrecisionTime(areaSelection.end),
               );
             }
-            this.handleDrag.currentTime = timescale.pxToHpTime(e.dragCurrent.x);
+
+            this.currentSnapPoint = undefined;
+            const currentTime = this.trySnapToTrack(
+              timescale.pxToHpTime(e.dragCurrent.x),
+              e.dragCurrent.y,
+              e.altKey,
+              timescale,
+              renderedTracks,
+            );
+
+            this.handleDrag.currentTime = currentTime;
             trace.timeline.selectedSpan = this.handleDrag
               .timeSpan()
               .toTimeSpan();
+            this.trace.raf.scheduleCanvasRedraw();
           },
           onDragEnd: (e) => {
-            const newStartTime = timescale
-              .pxToHpTime(e.dragCurrent.x)
-              .toTime('ceil');
+            this.currentSnapPoint = undefined;
+            const newStartTime = this.trySnapToTrack(
+              timescale.pxToHpTime(e.dragCurrent.x),
+              e.dragCurrent.y,
+              e.altKey,
+              timescale,
+              renderedTracks,
+            );
+
             trace.selection.selectArea({
               ...areaSelection,
-              end: Time.max(newStartTime, areaSelection.end),
-              start: Time.min(newStartTime, areaSelection.end),
+              end: Time.max(newStartTime.toTime('ceil'), areaSelection.end),
+              start: Time.min(newStartTime.toTime('ceil'), areaSelection.end),
             });
             trace.timeline.selectedSpan = undefined;
             this.handleDrag = undefined;
+            this.currentSnapPoint = undefined;
           },
         },
       },
@@ -543,22 +662,40 @@ export class TrackTreeView implements m.ClassComponent<TrackTreeViewAttrs> {
                 new HighPrecisionTime(areaSelection.start),
               );
             }
-            this.handleDrag.currentTime = timescale.pxToHpTime(e.dragCurrent.x);
+
+            this.currentSnapPoint = undefined;
+            const currentTime = this.trySnapToTrack(
+              timescale.pxToHpTime(e.dragCurrent.x),
+              e.dragCurrent.y,
+              e.altKey,
+              timescale,
+              renderedTracks,
+            );
+
+            this.handleDrag.currentTime = currentTime;
             trace.timeline.selectedSpan = this.handleDrag
               .timeSpan()
               .toTimeSpan();
+            this.trace.raf.scheduleCanvasRedraw();
           },
           onDragEnd: (e) => {
-            const newEndTime = timescale
-              .pxToHpTime(e.dragCurrent.x)
-              .toTime('ceil');
+            this.currentSnapPoint = undefined;
+            const newEndTime = this.trySnapToTrack(
+              timescale.pxToHpTime(e.dragCurrent.x),
+              e.dragCurrent.y,
+              e.altKey,
+              timescale,
+              renderedTracks,
+            );
+
             trace.selection.selectArea({
               ...areaSelection,
-              end: Time.max(newEndTime, areaSelection.start),
-              start: Time.min(newEndTime, areaSelection.start),
+              end: Time.max(newEndTime.toTime('ceil'), areaSelection.start),
+              start: Time.min(newEndTime.toTime('ceil'), areaSelection.start),
             });
             trace.timeline.selectedSpan = undefined;
             this.handleDrag = undefined;
+            this.currentSnapPoint = undefined;
           },
         },
       },
@@ -580,7 +717,19 @@ export class TrackTreeView implements m.ClassComponent<TrackTreeViewAttrs> {
                 e.dragStart.y,
               );
             }
-            this.areaDrag.update(e, timescale);
+
+            this.currentSnapPoint = undefined;
+            const currentTime = this.trySnapToTrack(
+              timescale.pxToHpTime(e.dragCurrent.x),
+              e.dragCurrent.y,
+              e.altKey,
+              timescale,
+              renderedTracks,
+            );
+
+            this.areaDrag.currentTime = currentTime;
+            this.areaDrag.currentY = e.dragCurrent.y;
+
             this.trace.raf.scheduleCanvasRedraw();
             trace.timeline.selectedSpan = this.areaDrag.timeSpan().toTimeSpan();
           },
@@ -591,7 +740,18 @@ export class TrackTreeView implements m.ClassComponent<TrackTreeViewAttrs> {
                 e.dragStart.y,
               );
             }
-            this.areaDrag?.update(e, timescale);
+
+            this.currentSnapPoint = undefined;
+            const currentTime = this.trySnapToTrack(
+              timescale.pxToHpTime(e.dragCurrent.x),
+              e.dragCurrent.y,
+              e.altKey,
+              timescale,
+              renderedTracks,
+            );
+
+            this.areaDrag.currentTime = currentTime;
+            this.areaDrag.currentY = e.dragCurrent.y;
 
             // Find the list of tracks that intersect this selection
             const trackUris = findTracksInRect(
@@ -611,6 +771,7 @@ export class TrackTreeView implements m.ClassComponent<TrackTreeViewAttrs> {
 
             trace.timeline.selectedSpan = undefined;
             this.areaDrag = undefined;
+            this.currentSnapPoint = undefined;
           },
         },
       },
@@ -618,6 +779,36 @@ export class TrackTreeView implements m.ClassComponent<TrackTreeViewAttrs> {
     ]);
   }
 
+  private trySnapToTrack(
+    targetTime: HighPrecisionTime,
+    y: number,
+    altKeyPressed: boolean,
+    timescale: TimeScale,
+    renderedTracks: ReadonlyArray<TrackView>,
+  ): HighPrecisionTime {
+    if (!this.snapEnabled || altKeyPressed) {
+      return targetTime;
+    }
+
+    const trackView = findTrackAtY(renderedTracks, y);
+    if (!trackView?.renderer?.track.getSnapPoint) {
+      return targetTime;
+    }
+
+    const snapPoint = trackView.renderer.track.getSnapPoint(
+      targetTime.toTime(),
+      SNAP_THRESHOLD_PX,
+      timescale,
+    );
+
+    if (snapPoint) {
+      this.currentSnapPoint = snapPoint;
+      return new HighPrecisionTime(snapPoint.time);
+    }
+
+    return targetTime;
+  }
+
   private updatePerfStats(
     renderTime: number,
     totalTracks: number,
@@ -635,29 +826,81 @@ export class TrackTreeView implements m.ClassComponent<TrackTreeViewAttrs> {
     size: Size2D,
   ) {
     if (this.areaDrag) {
+      const rect = this.areaDrag.rect(timescale);
+      const snapPx = this.currentSnapPoint
+        ? timescale.timeToPx(this.currentSnapPoint.time)
+        : undefined;
+
       ctx.strokeStyle = COLOR_ACCENT;
       ctx.lineWidth = 1;
-      const rect = this.areaDrag.rect(timescale);
-      ctx.strokeRect(rect.x, rect.y, rect.width, rect.height);
+      ctx.beginPath();
+
+      // Always draw top and bottom
+      ctx.moveTo(rect.x, rect.y);
+      ctx.lineTo(rect.x + rect.width, rect.y);
+      ctx.moveTo(rect.x, rect.y + rect.height);
+      ctx.lineTo(rect.x + rect.width, rect.y + rect.height);
+
+      // Draw left edge if not snapped
+      if (snapPx === undefined || Math.abs(snapPx - rect.x) > 1) {
+        ctx.moveTo(rect.x, rect.y);
+        ctx.lineTo(rect.x, rect.y + rect.height);
+      }
+
+      // Draw right edge if not snapped
+      if (
+        snapPx === undefined ||
+        Math.abs(snapPx - (rect.x + rect.width)) > 1
+      ) {
+        ctx.moveTo(rect.x + rect.width, rect.y);
+        ctx.lineTo(rect.x + rect.width, rect.y + rect.height);
+      }
+
+      ctx.stroke();
+
+      // Draw full-height dashed line if snapped
+      if (snapPx !== undefined) {
+        ctx.setLineDash([4, 4]);
+        ctx.beginPath();
+        ctx.moveTo(snapPx, 0);
+        ctx.lineTo(snapPx, size.height);
+        ctx.stroke();
+        ctx.setLineDash([]);
+      }
     }
 
     if (this.handleDrag) {
       const rect = this.handleDrag.hBounds(timescale);
+      const snapPx = this.currentSnapPoint
+        ? timescale.timeToPx(this.currentSnapPoint.time)
+        : undefined;
 
       ctx.strokeStyle = COLOR_ACCENT;
       ctx.lineWidth = 1;
 
+      // Draw left boundary
+      const leftSnapped =
+        snapPx !== undefined && Math.abs(snapPx - rect.left) < 1;
+      if (leftSnapped) {
+        ctx.setLineDash([4, 4]);
+      }
       ctx.beginPath();
       ctx.moveTo(rect.left, 0);
       ctx.lineTo(rect.left, size.height);
       ctx.stroke();
-      ctx.closePath();
+      ctx.setLineDash([]);
 
+      // Draw right boundary
+      const rightSnapped =
+        snapPx !== undefined && Math.abs(snapPx - rect.right) < 1;
+      if (rightSnapped) {
+        ctx.setLineDash([4, 4]);
+      }
       ctx.beginPath();
       ctx.moveTo(rect.right, 0);
       ctx.lineTo(rect.right, size.height);
       ctx.stroke();
-      ctx.closePath();
+      ctx.setLineDash([]);
     }
 
     const selection = this.trace.selection.selection;
@@ -754,6 +997,26 @@ export class TrackTreeView implements m.ClassComponent<TrackTreeViewAttrs> {
   }
 }
 
+/**
+ * Helper function to find a single track at a given Y coordinate.
+ *
+ * @param renderedTracks - The list of tracks and their positions.
+ * @param y - The Y coordinate to check.
+ * @returns - The track at the given Y coordinate, or undefined if none found.
+ */
+function findTrackAtY(
+  renderedTracks: ReadonlyArray<TrackView>,
+  y: number,
+): TrackView | undefined {
+  for (const trackView of renderedTracks) {
+    const {verticalBounds} = trackView;
+    if (y >= verticalBounds.top && y < verticalBounds.bottom) {
+      return trackView;
+    }
+  }
+  return undefined;
+}
+
 /**
  * Returns a list of track nodes that are contained within a given set of
  * vertical bounds.
