@@ -18,11 +18,10 @@ import {DetailsShell} from '../../widgets/details_shell';
 import {Timestamp} from '../../components/widgets/timestamp';
 import {Engine} from '../../trace_processor/engine';
 import {LONG, NUM, NUM_NULL, STR} from '../../trace_processor/query_result';
-import {Monitor} from '../../base/monitor';
-import {AsyncLimiter} from '../../base/async_limiter';
 import {
   escapeQuery,
   escapeSearchQuery,
+  escapeRegexQuery,
 } from '../../trace_processor/query_utils';
 import {Select} from '../../widgets/select';
 import {
@@ -44,12 +43,15 @@ import {classNames} from '../../base/classnames';
 import {TagInput} from '../../widgets/tag_input';
 import {Store} from '../../base/store';
 import {Trace} from '../../public/trace';
+import {Icons} from '../../base/semantic_icons';
+import {SerialTaskQueue, QuerySlot} from '../../base/query_slot';
 
 const ROW_H = 24;
 
 export interface LogFilteringCriteria {
   readonly minimumLevel: number;
   readonly tags: string[];
+  readonly isTagRegex?: boolean;
   readonly textEntry: string;
   readonly hideNonMatching: boolean;
   readonly machineExcludeList: number[];
@@ -86,56 +88,49 @@ interface LogEntries {
 
 export class LogPanel implements m.ClassComponent<LogPanelAttrs> {
   private readonly trace: Trace;
-  private entries?: LogEntries;
+  private readonly executor = new SerialTaskQueue();
+  private readonly viewQuery = new QuerySlot<AsyncDisposable>(this.executor);
+  private readonly entriesQuery = new QuerySlot<LogEntries>(this.executor);
   private pagination: Pagination = {
     offset: 0,
     count: 0,
   };
-  private readonly rowsMonitor: Monitor;
-  private readonly filterMonitor: Monitor;
-  private readonly queryLimiter = new AsyncLimiter();
 
   constructor({attrs}: m.CVnode<LogPanelAttrs>) {
     this.trace = attrs.trace;
-    this.rowsMonitor = new Monitor([
-      () => attrs.filterStore.state,
-      () => attrs.trace.timeline.visibleWindow.toTimeSpan().start,
-      () => attrs.trace.timeline.visibleWindow.toTimeSpan().end,
-    ]);
+  }
 
-    this.filterMonitor = new Monitor([() => attrs.filterStore.state]);
+  onremove() {
+    this.viewQuery.dispose();
+    this.entriesQuery.dispose();
   }
 
   view({attrs}: m.CVnode<LogPanelAttrs>) {
-    if (this.rowsMonitor.ifStateChanged()) {
-      this.scheduleDataReload(attrs);
-    }
+    const visibleSpan = attrs.trace.timeline.visibleWindow.toTimeSpan();
+    const filters = attrs.filterStore.state;
+    const pagination = this.pagination;
+    const engine = attrs.trace.engine;
 
-    const hasMachineIds = attrs.cache.uniqueMachineIds.length > 1;
-    const hasProcessNames =
-      this.entries &&
-      this.entries.processName.filter((name) => name).length > 0;
-    const totalEvents = this.entries?.totalEvents ?? 0;
+    // Query 1: Create the filtered_logs table (no staleOn = always-fresh)
+    const viewResult = this.viewQuery.use({
+      key: {filters},
+      queryFn: () => updateLogView(engine, filters),
+    });
 
-    const columns: GridColumn[] = [
-      ...(hasMachineIds
-        ? [{key: 'machine', header: m(GridHeaderCell, 'Machine')}]
-        : []),
-      {key: 'timestamp', header: m(GridHeaderCell, 'Timestamp')},
-      {key: 'pid', header: m(GridHeaderCell, 'PID')},
-      {key: 'tid', header: m(GridHeaderCell, 'TID')},
-      {key: 'level', header: m(GridHeaderCell, 'Level')},
-      ...(hasProcessNames
-        ? [{key: 'process', header: m(GridHeaderCell, 'Process')}]
-        : []),
-      {key: 'tag', header: m(GridHeaderCell, 'Tag')},
-      {
-        key: 'message',
-        // Allow the initial width of the message column to expand as needed.
-        maxInitialWidthPx: Infinity,
-        header: m(GridHeaderCell, 'Message'),
+    // Query 2: Read from the table (staleOn=['pagination'] for smooth scrolling)
+    const entriesResult = this.entriesQuery.use({
+      key: {
+        filters,
+        viewport: {start: visibleSpan.start, end: visibleSpan.end},
+        pagination,
       },
-    ];
+      retainOn: ['pagination', 'viewport'],
+      queryFn: () => updateLogEntries(engine, visibleSpan, pagination),
+      enabled: !!viewResult.data,
+    });
+
+    const entries = entriesResult.data;
+    const totalEvents = entries?.totalEvents ?? 0;
 
     return m(
       DetailsShell,
@@ -148,16 +143,50 @@ export class LogPanel implements m.ClassComponent<LogPanelAttrs> {
           store: attrs.filterStore,
         }),
       },
-      m(Grid, {
+      this.renderGrid(attrs.trace, entries, attrs.cache),
+    );
+  }
+
+  private renderGrid(
+    trace: Trace,
+    entries: LogEntries | undefined,
+    cache: LogPanelCache,
+  ) {
+    if (entries) {
+      const hasMachineIds = cache.uniqueMachineIds.length > 1;
+      const hasProcessNames =
+        entries.processName.filter((name) => name).length > 0;
+
+      const columns: GridColumn[] = [
+        ...(hasMachineIds
+          ? [{key: 'machine', header: m(GridHeaderCell, 'Machine')}]
+          : []),
+        {key: 'timestamp', header: m(GridHeaderCell, 'Timestamp')},
+        {key: 'pid', header: m(GridHeaderCell, 'PID')},
+        {key: 'tid', header: m(GridHeaderCell, 'TID')},
+        {key: 'level', header: m(GridHeaderCell, 'Level')},
+        ...(hasProcessNames
+          ? [{key: 'process', header: m(GridHeaderCell, 'Process')}]
+          : []),
+        {key: 'tag', header: m(GridHeaderCell, 'Tag')},
+        {
+          key: 'message',
+          // Allow the initial width of the message column to expand as needed.
+          maxInitialWidthPx: Infinity,
+          header: m(GridHeaderCell, 'Message'),
+        },
+      ];
+
+      return m(Grid, {
         className: 'pf-logs-panel',
         columns,
         rowData: {
-          data: this.renderRows(hasMachineIds, hasProcessNames),
-          total: this.entries?.totalEvents ?? 0,
-          offset: this.entries?.offset ?? 0,
+          data: this.renderRows(entries, hasMachineIds, hasProcessNames),
+          total: entries?.totalEvents ?? 0,
+          offset: entries?.offset ?? 0,
           onLoadData: (offset, count) => {
             this.pagination = {offset, count};
-            this.scheduleDataReload(attrs);
+            m.redraw();
           },
         },
         virtualization: {
@@ -166,60 +195,43 @@ export class LogPanel implements m.ClassComponent<LogPanelAttrs> {
         fillHeight: true,
         onRowHover: (rowIndex) => {
           // Calculate the actual row index from virtualization offset
-          const actualIndex = rowIndex - (this.entries?.offset ?? 0);
-          const timestamp = this.entries?.timestamps[actualIndex];
+          const actualIndex = rowIndex - (entries?.offset ?? 0);
+          const timestamp = entries?.timestamps[actualIndex];
           if (timestamp !== undefined) {
-            attrs.trace.timeline.hoverCursorTimestamp = timestamp;
+            trace.timeline.hoverCursorTimestamp = timestamp;
           }
         },
         onRowOut: () => {
-          attrs.trace.timeline.hoverCursorTimestamp = undefined;
+          trace.timeline.hoverCursorTimestamp = undefined;
         },
-      }),
-    );
-  }
-
-  private scheduleDataReload(attrs: LogPanelAttrs) {
-    const visibleSpan = attrs.trace.timeline.visibleWindow.toTimeSpan();
-    const filterStateChanged = this.filterMonitor.ifStateChanged();
-    const filterStoreState = attrs.filterStore.state;
-    const engine = attrs.trace.engine;
-    const pagination = this.pagination;
-
-    this.queryLimiter.schedule(async () => {
-      if (filterStateChanged) {
-        await updateLogView(engine, filterStoreState);
-      }
-
-      this.entries = await updateLogEntries(engine, visibleSpan, pagination);
-    });
+      });
+    } else {
+      return null;
+    }
   }
 
   private renderRows(
+    entries: LogEntries,
     hasMachineIds: boolean | undefined,
     hasProcessNames: boolean | undefined,
   ): ReadonlyArray<GridRow> {
-    if (!this.entries) {
-      return [];
-    }
-
     const trace = this.trace;
-    const machineIds = this.entries.machineIds;
-    const timestamps = this.entries.timestamps;
-    const pids = this.entries.pids;
-    const tids = this.entries.tids;
-    const priorities = this.entries.priorities;
-    const tags = this.entries.tags;
-    const messages = this.entries.messages;
-    const processNames = this.entries.processName;
+    const machineIds = entries.machineIds;
+    const timestamps = entries.timestamps;
+    const pids = entries.pids;
+    const tids = entries.tids;
+    const priorities = entries.priorities;
+    const tags = entries.tags;
+    const messages = entries.messages;
+    const processNames = entries.processName;
 
     const rows: GridRow[] = [];
-    for (let i = 0; i < this.entries.timestamps.length; i++) {
+    for (let i = 0; i < entries.timestamps.length; i++) {
       const priority = priorities[i];
       const priorityLetter = LOG_PRIORITIES[priority][0];
       const ts = timestamps[i];
       const priorityClass = `pf-logs-panel__row--${classForPriority(priority)}`;
-      const isHighlighted = this.entries.isHighlighted[i];
+      const isHighlighted = entries.isHighlighted[i];
       const className = classNames(
         priorityClass,
         isHighlighted && 'pf-logs-panel__row--highlighted',
@@ -332,7 +344,7 @@ interface FilterByTextWidgetAttrs {
 
 class FilterByTextWidget implements m.ClassComponent<FilterByTextWidgetAttrs> {
   view({attrs}: m.Vnode<FilterByTextWidgetAttrs>) {
-    const icon = attrs.hideNonMatching ? 'filter_alt' : 'filter_alt_off';
+    const icon = attrs.hideNonMatching ? Icons.Filter : Icons.FilterOff;
     const tooltip = attrs.hideNonMatching
       ? 'Show all logs and highlight matches'
       : 'Show only matching logs';
@@ -381,6 +393,16 @@ export class LogsFilters implements m.ClassComponent<LogsFiltersAttrs> {
           });
         },
       }),
+      m(Button, {
+        icon: 'regular_expression',
+        title: 'Use regular expression',
+        active: !!attrs.store.state.isTagRegex,
+        onclick: () => {
+          attrs.store.edit((draft) => {
+            draft.isTagRegex = !draft.isTagRegex;
+          });
+        },
+      }),
       m(LogTextWidget, {
         trace: attrs.trace,
         onChange: (text) => {
@@ -418,7 +440,7 @@ export class LogsFilters implements m.ClassComponent<LogsFiltersAttrs> {
 
     return m(PopupMultiSelect, {
       label: 'Filter by machine',
-      icon: 'filter_list_alt',
+      icon: Icons.Filter,
       position: PopupPosition.Top,
       options,
       onChange: (diffs: MultiSelectDiff[]) => {
@@ -482,7 +504,7 @@ async function updateLogEntries(
     isMsgHighlighted: NUM_NULL,
     isProcessHighlighted: NUM,
     processName: STR,
-    machineId: NUM_NULL,
+    machineId: NUM,
   });
   for (; it.valid(); it.next()) {
     timestamps.push(Time.fromRaw(it.ts));
@@ -495,7 +517,7 @@ async function updateLogEntries(
       it.isMsgHighlighted === 1 || it.isProcessHighlighted === 1,
     );
     processName.push(it.processName);
-    machineIds.push(it.machineId ?? 0); // Id 0 is for the primary VM
+    machineIds.push(it.machineId);
   }
 
   const queryRes = await engine.query(`
@@ -521,9 +543,10 @@ async function updateLogEntries(
   };
 }
 
-async function updateLogView(engine: Engine, filter: LogFilteringCriteria) {
-  await engine.query('drop view if exists filtered_logs');
-
+async function updateLogView(
+  engine: Engine,
+  filter: LogFilteringCriteria,
+): Promise<AsyncDisposable> {
   const globMatch = composeGlobMatch(filter.hideNonMatching, filter.textEntry);
   let selectedRows = `select prio, ts, pid, tid, tag, msg,
       process.name as process_name,
@@ -533,16 +556,29 @@ async function updateLogView(engine: Engine, filter: LogFilteringCriteria) {
       left join process using(upid)
       where prio >= ${filter.minimumLevel}`;
   if (filter.tags.length) {
-    selectedRows += ` and tag in (${serializeTags(filter.tags)})`;
+    if (filter.isTagRegex) {
+      const tagGlobClauses = filter.tags.map(
+        (pattern) => `tag glob ${escapeRegexQuery(pattern)}`,
+      );
+      selectedRows += ` and (${tagGlobClauses.join(' OR ')})`;
+    } else {
+      selectedRows += ` and tag in (${serializeTags(filter.tags)})`;
+    }
   }
   if (filter.machineExcludeList.length) {
-    selectedRows += ` and ifnull(process.machine_id, 0) not in (${filter.machineExcludeList.join(',')})`;
+    selectedRows += ` and process.machine_id not in (${filter.machineExcludeList.join(',')})`;
   }
 
   // We extract only the rows which will be visible.
-  await engine.query(`create view filtered_logs as select *
+  await engine.query(`create perfetto table filtered_logs as select *
     from (${selectedRows})
     where is_msg_chosen is 1 or is_process_chosen is 1`);
+
+  return {
+    async [Symbol.asyncDispose]() {
+      await engine.query('drop table filtered_logs');
+    },
+  };
 }
 
 function serializeTags(tags: string[]) {
