@@ -14,7 +14,6 @@
 
 import m from 'mithril';
 
-import {assertExists, assertFalse} from '../../base/logging';
 import {createPerfettoTable} from '../../trace_processor/sql_utils';
 import {extensions} from '../../components/extensions';
 import {time} from '../../base/time';
@@ -35,42 +34,17 @@ import {NUM} from '../../trace_processor/query_result';
 import {Button, ButtonVariant} from '../../widgets/button';
 import {Intent} from '../../widgets/common';
 import {DetailsShell} from '../../widgets/details_shell';
-import {Icon} from '../../widgets/icon';
 import {Modal, showModal} from '../../widgets/modal';
 import {
   Flamegraph,
-  FLAMEGRAPH_STATE_SCHEMA,
   FlamegraphState,
+  FLAMEGRAPH_STATE_SCHEMA,
   FlamegraphOptionalAction,
 } from '../../widgets/flamegraph';
-import {SqlTableDescription} from '../../components/widgets/sql/table/table_description';
-import {StandardColumn} from '../../components/widgets/sql/table/columns';
+import {SqlTableDefinition} from '../../components/widgets/sql/table/table_description';
 import {PerfettoSqlTypes} from '../../trace_processor/perfetto_sql_type';
 import {Stack} from '../../widgets/stack';
-import {Tooltip} from '../../widgets/tooltip';
-
-export enum ProfileType {
-  HEAP_PROFILE = 'heap_profile',
-  MIXED_HEAP_PROFILE = 'heap_profile:com.android.art,libc.malloc',
-  NATIVE_HEAP_PROFILE = 'heap_profile:libc.malloc',
-  JAVA_HEAP_SAMPLES = 'heap_profile:com.android.art',
-  JAVA_HEAP_GRAPH = 'graph',
-  PERF_SAMPLE = 'perf',
-  INSTRUMENTS_SAMPLE = 'instruments',
-}
-
-export function profileType(s: string): ProfileType {
-  if (s === 'heap_profile:libc.malloc,com.android.art') {
-    s = 'heap_profile:com.android.art,libc.malloc';
-  }
-  if (Object.values(ProfileType).includes(s as ProfileType)) {
-    return s as ProfileType;
-  }
-  if (s.startsWith('heap_profile')) {
-    return ProfileType.HEAP_PROFILE;
-  }
-  throw new Error('Unknown type ${s}');
-}
+import {ProfileDescriptor, ProfileType} from './common';
 
 interface Props {
   ts: time;
@@ -80,26 +54,59 @@ interface Props {
 export class HeapProfileFlamegraphDetailsPanel
   implements TrackEventDetailsPanel
 {
-  private readonly flamegraph: QueryFlamegraph;
+  private flamegraph: QueryFlamegraph;
   private readonly props: Props;
   private flamegraphModalDismissed = false;
 
-  readonly serialization: TrackEventDetailsPanelSerializeArgs<FlamegraphState>;
+  // TODO(lalitm): we should be able remove this around the 26Q2 timeframe
+  // We moved serialization from being attached to selections to instead being
+  // attached to the plugin that loaded the panel.
+  readonly serialization: TrackEventDetailsPanelSerializeArgs<
+    FlamegraphState | undefined
+  > = {
+    schema: FLAMEGRAPH_STATE_SCHEMA.optional(),
+    state: undefined,
+  };
+
+  readonly metrics: ReadonlyArray<QueryFlamegraphMetric>;
 
   constructor(
-    private trace: Trace,
-    private heapGraphIncomplete: boolean,
-    private upid: number,
-    profileType: ProfileType,
-    ts: time,
+    private readonly trace: Trace,
+    private readonly heapGraphIncomplete: boolean,
+    private readonly upid: number,
+    private readonly profileDescriptor: ProfileDescriptor,
+    private readonly ts: time,
+    private readonly tsEnd: time,
+    private state: FlamegraphState | undefined,
+    private readonly onStateChange: (state: FlamegraphState) => void,
   ) {
-    const metrics = flamegraphMetrics(trace, profileType, ts, upid);
-    this.serialization = {
-      schema: FLAMEGRAPH_STATE_SCHEMA,
-      state: Flamegraph.createDefaultState(metrics),
-    };
-    this.flamegraph = new QueryFlamegraph(trace, metrics, this.serialization);
-    this.props = {ts, type: profileType};
+    this.props = {ts, type: profileDescriptor.type};
+    this.flamegraph = new QueryFlamegraph(trace);
+    this.metrics = flamegraphMetrics(
+      this.trace,
+      this.profileDescriptor,
+      this.ts,
+      this.tsEnd,
+      this.upid,
+    );
+    if (this.state === undefined) {
+      this.state = Flamegraph.createDefaultState(this.metrics);
+      onStateChange(this.state);
+    }
+  }
+
+  async load() {
+    // If the state in the serialization is not undefined, we should read from
+    // it.
+    // TODO(lalitm): remove this in 26Q2 - see comment on `serialization`.
+    if (this.serialization.state !== undefined) {
+      this.state = Flamegraph.updateState(
+        this.serialization.state,
+        this.metrics,
+      );
+      this.onStateChange(this.state);
+      this.serialization.state = undefined;
+    }
   }
 
   render() {
@@ -111,23 +118,7 @@ export class HeapProfileFlamegraphDetailsPanel
         DetailsShell,
         {
           fillHeight: true,
-          title: m(
-            'span',
-            getFlamegraphTitle(type),
-            type === ProfileType.MIXED_HEAP_PROFILE && [
-              ' ', // Some space between title and icon
-              m(
-                Tooltip,
-                {
-                  trigger: m(Icon, {icon: 'warning', intent: Intent.Warning}),
-                },
-                m(
-                  '',
-                  'This is a mixed java/native heap profile, free()s are not visualized. To visualize free()s, remove "all_heaps: true" from the config.',
-                ),
-              ),
-            ],
-          ),
+          title: m('span', this.profileDescriptor.label),
           buttons: m(Stack, {orientation: 'horizontal', spacing: 'large'}, [
             m('span', `Snapshot time: `, m(Timestamp, {trace: this.trace, ts})),
             (type === ProfileType.NATIVE_HEAP_PROFILE ||
@@ -142,7 +133,14 @@ export class HeapProfileFlamegraphDetailsPanel
               }),
           ]),
         },
-        assertExists(this.flamegraph).render(),
+        this.flamegraph.render({
+          metrics: this.metrics,
+          state: this.state,
+          onStateChange: (state) => {
+            this.state = state;
+            this.onStateChange(state);
+          },
+        }),
       ),
     );
   }
@@ -184,83 +182,89 @@ export class HeapProfileFlamegraphDetailsPanel
 
 function flamegraphMetrics(
   trace: Trace,
-  type: ProfileType,
+  descriptor: ProfileDescriptor,
   ts: time,
+  tsEnd: time,
   upid: number,
 ): ReadonlyArray<QueryFlamegraphMetric> {
-  switch (type) {
+  switch (descriptor.type) {
     case ProfileType.NATIVE_HEAP_PROFILE:
-      return flamegraphMetricsForHeapProfile(ts, upid, [
-        {
-          name: 'Unreleased Malloc Size',
-          unit: 'B',
-          columnName: 'self_size',
-        },
-        {
-          name: 'Unreleased Malloc Count',
-          unit: '',
-          columnName: 'self_count',
-        },
-        {
-          name: 'Total Malloc Size',
-          unit: 'B',
-          columnName: 'self_alloc_size',
-        },
-        {
-          name: 'Total Malloc Count',
-          unit: '',
-          columnName: 'self_alloc_count',
-        },
-      ]);
-    case ProfileType.HEAP_PROFILE:
-      return flamegraphMetricsForHeapProfile(ts, upid, [
-        {
-          name: 'Unreleased Size',
-          unit: 'B',
-          columnName: 'self_size',
-        },
-        {
-          name: 'Unreleased Count',
-          unit: '',
-          columnName: 'self_count',
-        },
-        {
-          name: 'Total Size',
-          unit: 'B',
-          columnName: 'self_alloc_size',
-        },
-        {
-          name: 'Total Count',
-          unit: '',
-          columnName: 'self_alloc_count',
-        },
-      ]);
+      return flamegraphMetricsForHeapProfile(
+        ts,
+        tsEnd,
+        upid,
+        descriptor.heapName!,
+        [
+          {
+            name: 'Unreleased Malloc Size',
+            unit: 'B',
+            columnName: 'self_size',
+          },
+          {
+            name: 'Unreleased Malloc Count',
+            unit: '',
+            columnName: 'self_count',
+          },
+          {
+            name: 'Total Malloc Size',
+            unit: 'B',
+            columnName: 'self_alloc_size',
+          },
+          {
+            name: 'Total Malloc Count',
+            unit: '',
+            columnName: 'self_alloc_count',
+          },
+        ],
+      );
+    case ProfileType.GENERIC_HEAP_PROFILE:
+      return flamegraphMetricsForHeapProfile(
+        ts,
+        tsEnd,
+        upid,
+        descriptor.heapName!,
+        [
+          {
+            name: 'Unreleased Size',
+            unit: 'B',
+            columnName: 'self_size',
+          },
+          {
+            name: 'Unreleased Count',
+            unit: '',
+            columnName: 'self_count',
+          },
+          {
+            name: 'Total Size',
+            unit: 'B',
+            columnName: 'self_alloc_size',
+          },
+          {
+            name: 'Total Count',
+            unit: '',
+            columnName: 'self_alloc_count',
+          },
+        ],
+      );
     case ProfileType.JAVA_HEAP_SAMPLES:
-      return flamegraphMetricsForHeapProfile(ts, upid, [
-        {
-          name: 'Total Allocation Size',
-          unit: 'B',
-          columnName: 'self_size',
-        },
-        {
-          name: 'Total Allocation Count',
-          unit: '',
-          columnName: 'self_count',
-        },
-      ]);
-    case ProfileType.MIXED_HEAP_PROFILE:
-      return flamegraphMetricsForHeapProfile(ts, upid, [
-        {
-          name: 'Allocation Size (malloc + java)',
-          unit: 'B',
-          columnName: 'self_size',
-        },
-        {
-          name: 'Allocation Count (malloc + java)',
-          unit: '',
-          columnName: 'self_count',
-        },
-      ]);
+      return flamegraphMetricsForHeapProfile(
+        ts,
+        tsEnd,
+        upid,
+        descriptor.heapName!,
+        [
+          {
+            name: 'Total Allocation Size',
+            unit: 'B',
+            columnName: 'self_size',
+          },
+          {
+            name: 'Total Allocation Count',
+            unit: '',
+            columnName: 'self_count',
+          },
+        ],
+      );
     case ProfileType.JAVA_HEAP_GRAPH:
       return [
         {
@@ -406,21 +410,44 @@ function flamegraphMetrics(
           optionalRootActions: getHeapGraphRootOptionalActions(trace, true),
         },
       ];
-    case ProfileType.PERF_SAMPLE:
-      throw new Error('Perf sample not supported');
-    case ProfileType.INSTRUMENTS_SAMPLE:
-      throw new Error('Instruments sample not supported');
   }
 }
 
 function flamegraphMetricsForHeapProfile(
   ts: time,
+  tsEnd: bigint,
   upid: number,
+  heapName: string,
   metrics: {name: string; unit: string; columnName: string}[],
 ) {
-  return metricsFromTableOrSubquery(
-    `
+  return metricsFromTableOrSubquery({
+    tableOrSubquery: `
       (
+        -- Any selection overlap with an allocation slice includes the
+        -- slice in the result. Practically this means that we might need to
+        -- extend the right-side boundary.
+        with alloc_bound as (
+          select ts
+          from heap_profile_allocation
+          where ts >= ${tsEnd}
+            and upid = ${upid} and heap_name = '${heapName}'
+          order by ts asc
+          limit 1
+        ),
+        -- The native heap profiler data model is delta-encoded.
+        -- Unreleased allocations will be recorded across continuous dumps
+        -- and trace processor is responsible for deduplicating them.
+        -- If an allocation at ts1 is released in ts2, this will
+        -- be represented as an unmatched memory released in ts2 (negative size).
+        -- For the purposes of looking at ts2+ slices, we need to ignore
+        -- the negative sized data points.
+        alloc_class as (
+          select callsite_id, if(sum(count) > 0, 1, 0) as positive_alloc
+          from heap_profile_allocation
+          where ts >= ${ts} and ts <= ifnull((SELECT ts FROM alloc_bound), ${tsEnd})
+            and upid = ${upid} and heap_name = '${heapName}'
+          group by callsite_id
+        )
         select
           id,
           parent_id as parentId,
@@ -434,47 +461,30 @@ function flamegraphMetricsForHeapProfile(
         from _android_heap_profile_callstacks_for_allocations!((
           select
             callsite_id,
-            size,
-            count,
+            iif(positive_alloc, size, 0) as size,
+            iif(positive_alloc, count, 0) as count,
             max(size, 0) as alloc_size,
             max(count, 0) as alloc_count
           from heap_profile_allocation a
-          where a.ts <= ${ts} and a.upid = ${upid}
+          join alloc_class using (callsite_id)
+          where a.ts >= ${ts} and a.ts <= ifnull((SELECT ts FROM alloc_bound), ${tsEnd})
+            and a.upid = ${upid} and a.heap_name = '${heapName}'
         ))
       )
     `,
-    metrics,
-    'include perfetto module android.memory.heap_profile.callstacks',
-    [{name: 'mapping_name', displayName: 'Mapping'}],
-    [
+    tableMetrics: metrics,
+    dependencySql:
+      'include perfetto module android.memory.heap_profile.callstacks',
+    unaggregatableProperties: [{name: 'mapping_name', displayName: 'Mapping'}],
+    aggregatableProperties: [
       {
         name: 'source_location',
         displayName: 'Source Location',
         mergeAggregation: 'ONE_OR_SUMMARY',
       },
     ],
-  );
-}
-
-function getFlamegraphTitle(type: ProfileType) {
-  switch (type) {
-    case ProfileType.HEAP_PROFILE:
-      return 'Heap profile';
-    case ProfileType.JAVA_HEAP_GRAPH:
-      return 'Java heap graph';
-    case ProfileType.JAVA_HEAP_SAMPLES:
-      return 'Java heap samples';
-    case ProfileType.MIXED_HEAP_PROFILE:
-      return 'Mixed heap profile';
-    case ProfileType.NATIVE_HEAP_PROFILE:
-      return 'Native heap profile';
-    case ProfileType.PERF_SAMPLE:
-      assertFalse(false, 'Perf sample not supported');
-      return 'Impossible';
-    case ProfileType.INSTRUMENTS_SAMPLE:
-      assertFalse(false, 'Instruments sample not supported');
-      return 'Impossible';
-  }
+    nameColumnLabel: 'Symbol',
+  });
 }
 
 async function downloadPprof(trace: Trace, upid: number, ts: time) {
@@ -494,105 +504,117 @@ async function downloadPprof(trace: Trace, upid: number, ts: time) {
 
 function getHeapGraphObjectReferencesView(
   isDominator: boolean,
-): SqlTableDescription {
+): SqlTableDefinition {
   return {
     name: `_heap_graph${tableModifier(isDominator)}object_references`,
     columns: [
-      new StandardColumn('path_hash', PerfettoSqlTypes.STRING),
-      new StandardColumn('outgoing_reference_count', PerfettoSqlTypes.INT),
-      new StandardColumn('class_name', PerfettoSqlTypes.STRING),
-      new StandardColumn('self_size', PerfettoSqlTypes.INT),
-      new StandardColumn('native_size', PerfettoSqlTypes.INT),
-      new StandardColumn('heap_type', PerfettoSqlTypes.STRING),
-      new StandardColumn('root_type', PerfettoSqlTypes.STRING),
-      new StandardColumn('reachable', PerfettoSqlTypes.BOOLEAN),
+      {column: 'path_hash', type: PerfettoSqlTypes.STRING},
+      {column: 'outgoing_reference_count', type: PerfettoSqlTypes.INT},
+      {column: 'class_name', type: PerfettoSqlTypes.STRING},
+      {column: 'self_size', type: PerfettoSqlTypes.INT},
+      {column: 'native_size', type: PerfettoSqlTypes.INT},
+      {column: 'total_cumulative_size', type: PerfettoSqlTypes.INT},
+      {column: 'cumulative_size', type: PerfettoSqlTypes.INT},
+      {column: 'cumulative_native_size', type: PerfettoSqlTypes.INT},
+      {column: 'cumulative_count', type: PerfettoSqlTypes.INT},
+      {column: 'heap_type', type: PerfettoSqlTypes.STRING},
+      {column: 'root_type', type: PerfettoSqlTypes.STRING},
+      {column: 'reachable', type: PerfettoSqlTypes.BOOLEAN},
     ],
   };
 }
 
 function getHeapGraphIncomingReferencesView(
   isDominator: boolean,
-): SqlTableDescription {
+): SqlTableDefinition {
   return {
     name: `_heap_graph${tableModifier(isDominator)}incoming_references`,
     columns: [
-      new StandardColumn('path_hash', PerfettoSqlTypes.STRING),
-      new StandardColumn('class_name', PerfettoSqlTypes.STRING),
-      new StandardColumn('field_name', PerfettoSqlTypes.STRING),
-      new StandardColumn('field_type_name', PerfettoSqlTypes.STRING),
-      new StandardColumn('self_size', PerfettoSqlTypes.INT),
-      new StandardColumn('native_size', PerfettoSqlTypes.INT),
-      new StandardColumn('heap_type', PerfettoSqlTypes.STRING),
-      new StandardColumn('root_type', PerfettoSqlTypes.STRING),
-      new StandardColumn('reachable', PerfettoSqlTypes.BOOLEAN),
+      {column: 'path_hash', type: PerfettoSqlTypes.STRING},
+      {column: 'class_name', type: PerfettoSqlTypes.STRING},
+      {column: 'field_name', type: PerfettoSqlTypes.STRING},
+      {column: 'field_type_name', type: PerfettoSqlTypes.STRING},
+      {column: 'total_cumulative_size', type: PerfettoSqlTypes.INT},
+      {column: 'cumulative_size', type: PerfettoSqlTypes.INT},
+      {column: 'cumulative_native_size', type: PerfettoSqlTypes.INT},
+      {column: 'cumulative_count', type: PerfettoSqlTypes.INT},
+      {column: 'self_size', type: PerfettoSqlTypes.INT},
+      {column: 'native_size', type: PerfettoSqlTypes.INT},
+      {column: 'heap_type', type: PerfettoSqlTypes.STRING},
+      {column: 'root_type', type: PerfettoSqlTypes.STRING},
+      {column: 'reachable', type: PerfettoSqlTypes.BOOLEAN},
     ],
   };
 }
 
 function getHeapGraphOutgoingReferencesView(
   isDominator: boolean,
-): SqlTableDescription {
+): SqlTableDefinition {
   return {
     name: `_heap_graph${tableModifier(isDominator)}outgoing_references`,
     columns: [
-      new StandardColumn('path_hash', PerfettoSqlTypes.STRING),
-      new StandardColumn('class_name', PerfettoSqlTypes.STRING),
-      new StandardColumn('field_name', PerfettoSqlTypes.STRING),
-      new StandardColumn('field_type_name', PerfettoSqlTypes.STRING),
-      new StandardColumn('self_size', PerfettoSqlTypes.INT),
-      new StandardColumn('native_size', PerfettoSqlTypes.INT),
-      new StandardColumn('heap_type', PerfettoSqlTypes.STRING),
-      new StandardColumn('root_type', PerfettoSqlTypes.STRING),
-      new StandardColumn('reachable', PerfettoSqlTypes.BOOLEAN),
+      {column: 'path_hash', type: PerfettoSqlTypes.STRING},
+      {column: 'class_name', type: PerfettoSqlTypes.STRING},
+      {column: 'field_name', type: PerfettoSqlTypes.STRING},
+      {column: 'field_type_name', type: PerfettoSqlTypes.STRING},
+      {column: 'total_cumulative_size', type: PerfettoSqlTypes.INT},
+      {column: 'cumulative_size', type: PerfettoSqlTypes.INT},
+      {column: 'cumulative_native_size', type: PerfettoSqlTypes.INT},
+      {column: 'cumulative_count', type: PerfettoSqlTypes.INT},
+      {column: 'self_size', type: PerfettoSqlTypes.INT},
+      {column: 'native_size', type: PerfettoSqlTypes.INT},
+      {column: 'heap_type', type: PerfettoSqlTypes.STRING},
+      {column: 'root_type', type: PerfettoSqlTypes.STRING},
+      {column: 'reachable', type: PerfettoSqlTypes.BOOLEAN},
     ],
   };
 }
 
 function getHeapGraphRetainingObjectCountsView(
   isDominator: boolean,
-): SqlTableDescription {
+): SqlTableDefinition {
   return {
     name: `_heap_graph${tableModifier(isDominator)}retaining_object_counts`,
     columns: [
-      new StandardColumn('class_name', PerfettoSqlTypes.STRING),
-      new StandardColumn('count', PerfettoSqlTypes.INT),
-      new StandardColumn('total_size', PerfettoSqlTypes.INT),
-      new StandardColumn('total_native_size', PerfettoSqlTypes.INT),
-      new StandardColumn('heap_type', PerfettoSqlTypes.STRING),
-      new StandardColumn('root_type', PerfettoSqlTypes.STRING),
-      new StandardColumn('reachable', PerfettoSqlTypes.BOOLEAN),
+      {column: 'class_name', type: PerfettoSqlTypes.STRING},
+      {column: 'count', type: PerfettoSqlTypes.INT},
+      {column: 'total_size', type: PerfettoSqlTypes.INT},
+      {column: 'total_native_size', type: PerfettoSqlTypes.INT},
+      {column: 'heap_type', type: PerfettoSqlTypes.STRING},
+      {column: 'root_type', type: PerfettoSqlTypes.STRING},
+      {column: 'reachable', type: PerfettoSqlTypes.BOOLEAN},
     ],
   };
 }
 
 function getHeapGraphRetainedObjectCountsView(
   isDominator: boolean,
-): SqlTableDescription {
+): SqlTableDefinition {
   return {
     name: `_heap_graph${tableModifier(isDominator)}retained_object_counts`,
     columns: [
-      new StandardColumn('class_name', PerfettoSqlTypes.STRING),
-      new StandardColumn('count', PerfettoSqlTypes.INT),
-      new StandardColumn('total_size', PerfettoSqlTypes.INT),
-      new StandardColumn('total_native_size', PerfettoSqlTypes.INT),
-      new StandardColumn('heap_type', PerfettoSqlTypes.STRING),
-      new StandardColumn('root_type', PerfettoSqlTypes.STRING),
-      new StandardColumn('reachable', PerfettoSqlTypes.BOOLEAN),
+      {column: 'class_name', type: PerfettoSqlTypes.STRING},
+      {column: 'count', type: PerfettoSqlTypes.INT},
+      {column: 'total_size', type: PerfettoSqlTypes.INT},
+      {column: 'total_native_size', type: PerfettoSqlTypes.INT},
+      {column: 'heap_type', type: PerfettoSqlTypes.STRING},
+      {column: 'root_type', type: PerfettoSqlTypes.STRING},
+      {column: 'reachable', type: PerfettoSqlTypes.BOOLEAN},
     ],
   };
 }
 
 function getHeapGraphDuplicateObjectsView(
   isDominator: boolean,
-): SqlTableDescription {
+): SqlTableDefinition {
   return {
     name: `_heap_graph${tableModifier(isDominator)}duplicate_objects`,
     columns: [
-      new StandardColumn('class_name', PerfettoSqlTypes.STRING),
-      new StandardColumn('path_count', PerfettoSqlTypes.INT),
-      new StandardColumn('object_count', PerfettoSqlTypes.INT),
-      new StandardColumn('total_size', PerfettoSqlTypes.INT),
-      new StandardColumn('total_native_size', PerfettoSqlTypes.INT),
+      {column: 'class_name', type: PerfettoSqlTypes.STRING},
+      {column: 'path_count', type: PerfettoSqlTypes.INT},
+      {column: 'object_count', type: PerfettoSqlTypes.INT},
+      {column: 'total_size', type: PerfettoSqlTypes.INT},
+      {column: 'total_native_size', type: PerfettoSqlTypes.INT},
     ],
   };
 }
@@ -615,6 +637,10 @@ function getHeapGraphNodeOptionalActions(
             as: pathHashesToTableStatement(value),
           });
 
+          await trace.engine.query(
+            'include perfetto module android.memory.heap_graph.object_tree;',
+          );
+
           const tableName = `_heap_graph${tableModifier(isDominator)}object_references`;
           const macroArgs = `_heap_graph${tableModifier(isDominator)}path_hashes, ${pathHashTableName}`;
           const macroExpr = `_heap_graph_object_references_agg!(${macroArgs})`;
@@ -647,6 +673,10 @@ function getHeapGraphNodeOptionalActions(
                 as: pathHashesToTableStatement(value),
               });
 
+              await trace.engine.query(
+                'include perfetto module android.memory.heap_graph.object_tree;',
+              );
+
               const tableName = `_heap_graph${tableModifier(isDominator)}incoming_references`;
               const macroArgs = `_heap_graph${tableModifier(isDominator)}path_hashes, ${pathHashTableName}`;
               const macroExpr = `_heap_graph_incoming_references_agg!(${macroArgs})`;
@@ -673,6 +703,10 @@ function getHeapGraphNodeOptionalActions(
                 as: pathHashesToTableStatement(value),
               });
 
+              await trace.engine.query(
+                'include perfetto module android.memory.heap_graph.object_tree;',
+              );
+
               const tableName = `_heap_graph${tableModifier(isDominator)}outgoing_references`;
               const macroArgs = `_heap_graph${tableModifier(isDominator)}path_hashes, ${pathHashTableName}`;
               const macroExpr = `_heap_graph_outgoing_references_agg!(${macroArgs})`;
