@@ -12,10 +12,11 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-import {assertExists} from '../../base/logging';
+import {assertExists} from '../../base/assert';
 import {
-  metricsFromTableOrSubquery,
   QueryFlamegraph,
+  QueryFlamegraphMetric,
+  QueryFlamegraphWithMetrics,
 } from '../../components/query_flamegraph';
 import {PerfettoPlugin} from '../../public/plugin';
 import {AreaSelection, areaSelectionsEqual} from '../../public/selection';
@@ -30,28 +31,58 @@ import {
   STR,
   STR_NULL,
 } from '../../trace_processor/query_result';
-import {Flamegraph} from '../../widgets/flamegraph';
+import {
+  Flamegraph,
+  FLAMEGRAPH_STATE_SCHEMA,
+  FlamegraphState,
+} from '../../widgets/flamegraph';
 import ProcessThreadGroupsPlugin from '../dev.perfetto.ProcessThreadGroups';
 import TraceProcessorTrackPlugin from '../dev.perfetto.TraceProcessorTrack';
 import {TraceProcessorCounterTrack} from '../dev.perfetto.TraceProcessorTrack/trace_processor_counter_track';
-import {createPerfCallsitesTrack} from './perf_samples_profile_track';
+import {Store} from '../../base/store';
+import {z} from 'zod';
+import CpuProfilePlugin from '../dev.perfetto.CpuProfile';
+import {SourceDataset} from '../../trace_processor/dataset';
+import {createProfilingTrack} from '../dev.perfetto.CpuProfile/profiling_track';
 
 const PERF_SAMPLES_PROFILE_TRACK_KIND = 'PerfSamplesProfileTrack';
 
+const LINUX_PERF_PLUGIN_STATE_SCHEMA = z.object({
+  areaSelectionFlamegraphState: FLAMEGRAPH_STATE_SCHEMA.optional(),
+  detailsPanelFlamegraphState: FLAMEGRAPH_STATE_SCHEMA.optional(),
+});
+
+type LinuxPerfPluginState = z.infer<typeof LINUX_PERF_PLUGIN_STATE_SCHEMA>;
+
 function makeUriForProc(upid: number, sessionId: number) {
   return `/process_${upid}/perf_samples_profile_${sessionId}`;
 }
 
-export default class implements PerfettoPlugin {
+export default class LinuxPerfPlugin implements PerfettoPlugin {
   static readonly id = 'dev.perfetto.LinuxPerf';
   static readonly dependencies = [
     ProcessThreadGroupsPlugin,
     TraceProcessorTrackPlugin,
+    CpuProfilePlugin,
   ];
 
+  private store?: Store<LinuxPerfPluginState>;
+  // Cache of counter types per perf session, populated during onTraceLoad.
+  private counterTypesBySession = new Map<number, {name: string}[]>();
+
+  private migrateLinuxPerfPluginState(init: unknown): LinuxPerfPluginState {
+    const result = LINUX_PERF_PLUGIN_STATE_SCHEMA.safeParse(init);
+    return result.data ?? {};
+  }
+
   async onTraceLoad(trace: Trace): Promise<void> {
-    await this.addProcessPerfSamplesTracks(trace);
-    await this.addThreadPerfSamplesTracks(trace);
+    this.store = trace.mountStore(LinuxPerfPlugin.id, (init) =>
+      this.migrateLinuxPerfPluginState(init),
+    );
+    const store = assertExists(this.store);
+    await this.cacheCounterTypesPerSession(trace);
+    await this.addProcessPerfSamplesTracks(trace, store);
+    await this.addThreadPerfSamplesTracks(trace, store);
     await this.addPerfCounterTracks(trace);
 
     trace.onTraceReady.addListener(async () => {
@@ -59,7 +90,40 @@ export default class implements PerfettoPlugin {
     });
   }
 
-  private async addProcessPerfSamplesTracks(trace: Trace) {
+  // Pre-fetch and cache all counter types per perf session for faster
+  // flamegraph rendering.
+  private async cacheCounterTypesPerSession(trace: Trace): Promise<void> {
+    const result = await trace.engine.query(`
+      SELECT pct.perf_session_id, pct.name, MAX(pct.is_timebase) as is_timebase
+      FROM perf_counter_track pct
+      GROUP BY pct.perf_session_id, pct.name
+      ORDER BY pct.perf_session_id, is_timebase DESC, pct.name
+    `);
+
+    for (
+      const it = result.iter({
+        perf_session_id: NUM,
+        name: STR_NULL,
+        is_timebase: NUM_NULL,
+      });
+      it.valid();
+      it.next()
+    ) {
+      const sessionId = it.perf_session_id;
+      const name = it.name;
+      if (name === null) continue;
+
+      if (!this.counterTypesBySession.has(sessionId)) {
+        this.counterTypesBySession.set(sessionId, []);
+      }
+      this.counterTypesBySession.get(sessionId)!.push({name});
+    }
+  }
+
+  private async addProcessPerfSamplesTracks(
+    trace: Trace,
+    store: Store<LinuxPerfPluginState>,
+  ) {
     const pResult = await trace.engine.query(`
       SELECT DISTINCT upid, pct.name AS cntrName, perf_session_id AS sessionId
       FROM perf_sample
@@ -101,7 +165,19 @@ export default class implements PerfettoPlugin {
           kinds: [PERF_SAMPLES_PROFILE_TRACK_KIND],
           upid,
         },
-        renderer: createPerfCallsitesTrack(trace, uri, upid),
+        renderer: createPerfCallsitesTrack(
+          trace,
+          uri,
+          upid,
+          undefined,
+          undefined,
+          store.state.detailsPanelFlamegraphState,
+          (state) => {
+            store.edit((draft) => {
+              draft.detailsPanelFlamegraphState = state;
+            });
+          },
+        ),
       });
       const group = trace.plugins
         .getPlugin(ProcessThreadGroupsPlugin)
@@ -132,6 +208,12 @@ export default class implements PerfettoPlugin {
             upid,
             undefined,
             sessionId,
+            store.state.detailsPanelFlamegraphState,
+            (state) => {
+              store.edit((draft) => {
+                draft.detailsPanelFlamegraphState = state;
+              });
+            },
           ),
         });
         const track = new TrackNode({
@@ -158,7 +240,10 @@ export default class implements PerfettoPlugin {
     });
   }
 
-  private async addThreadPerfSamplesTracks(trace: Trace) {
+  private async addThreadPerfSamplesTracks(
+    trace: Trace,
+    store: Store<LinuxPerfPluginState>,
+  ) {
     const tResult = await trace.engine.query(`
       SELECT DISTINCT
         upid, utid, tid, thread.name AS threadName,
@@ -217,7 +302,19 @@ export default class implements PerfettoPlugin {
           utid,
           upid: upid ?? undefined,
         },
-        renderer: createPerfCallsitesTrack(trace, uri, upid ?? undefined, utid),
+        renderer: createPerfCallsitesTrack(
+          trace,
+          uri,
+          upid ?? undefined,
+          utid,
+          undefined,
+          store.state.detailsPanelFlamegraphState,
+          (state) => {
+            store.edit((draft) => {
+              draft.detailsPanelFlamegraphState = state;
+            });
+          },
+        ),
       });
       const group = trace.plugins
         .getPlugin(ProcessThreadGroupsPlugin)
@@ -248,6 +345,12 @@ export default class implements PerfettoPlugin {
             upid ?? undefined,
             utid,
             sessionId,
+            store.state.detailsPanelFlamegraphState,
+            (state) => {
+              store.edit((draft) => {
+                draft.detailsPanelFlamegraphState = state;
+              });
+            },
           ),
         });
         const track = new TrackNode({
@@ -317,7 +420,188 @@ export default class implements PerfettoPlugin {
       trace.defaultWorkspace.addChildInOrder(perfCountersGroup);
     }
 
-    trace.selection.registerAreaSelectionTab(createAreaSelectionTab(trace));
+    trace.selection.registerAreaSelectionTab(
+      this.createAreaSelectionTab(trace),
+    );
+  }
+
+  private createAreaSelectionTab(trace: Trace) {
+    let previousSelection: AreaSelection | undefined;
+    let flamegraphWithMetrics: QueryFlamegraphWithMetrics | undefined;
+
+    return {
+      id: 'perf_sample_flamegraph',
+      name: 'Perf sample flamegraph',
+      render: (selection: AreaSelection) => {
+        const changed =
+          previousSelection === undefined ||
+          !areaSelectionsEqual(previousSelection, selection);
+        if (changed) {
+          previousSelection = selection;
+          flamegraphWithMetrics = this.computePerfSampleFlamegraph(
+            trace,
+            selection,
+          );
+        }
+        if (flamegraphWithMetrics === undefined) {
+          return undefined;
+        }
+        const {flamegraph, metrics} = flamegraphWithMetrics;
+        const store = assertExists(this.store);
+        return {
+          isLoading: false,
+          content: flamegraph.render({
+            metrics,
+            state: store.state.areaSelectionFlamegraphState,
+            onStateChange: (state) => {
+              store.edit((draft) => {
+                draft.areaSelectionFlamegraphState = state;
+              });
+            },
+          }),
+        };
+      },
+    };
+  }
+
+  private computePerfSampleFlamegraph(
+    trace: Trace,
+    currentSelection: AreaSelection,
+  ): QueryFlamegraphWithMetrics | undefined {
+    const processTrackTags = getSelectedProcessTrackTags(currentSelection);
+    const threadTrackTags = getSelectedThreadTrackTags(currentSelection);
+    if (processTrackTags.length === 0 && threadTrackTags.length === 0) {
+      return undefined;
+    }
+
+    // Get unique session IDs from selected tracks
+    const sessionIds = [
+      ...new Set([
+        ...processTrackTags.map(([_, sessionId]) => sessionId),
+        ...threadTrackTags.map(([_, sessionId]) => sessionId),
+      ]),
+    ];
+
+    // Get available counter types for selected sessions from cache
+    const counterTypes = this.getCounterTypesForSessions(sessionIds);
+
+    const trackConstraints = [
+      ...processTrackTags.map(
+        ([upid, sessionId]) =>
+          `(t.upid = ${upid} AND p.perf_session_id = ${sessionId})`,
+      ),
+      ...threadTrackTags.map(
+        ([utid, sessionId]) =>
+          `(t.utid = ${utid} AND p.perf_session_id = ${sessionId})`,
+      ),
+    ].join(' OR ');
+
+    const metrics: QueryFlamegraphMetric[] = [];
+
+    // Common properties for all metrics
+    const unaggregatableProperties = [
+      {name: 'mapping_name', displayName: 'Mapping'},
+    ];
+    const aggregatableProperties = [
+      {
+        name: 'source_location',
+        displayName: 'Source location',
+        mergeAggregation: 'ONE_OR_SUMMARY' as const,
+      },
+    ];
+
+    // Add a metric for each counter type (uses weighted macro with counter values)
+    for (const {name: counterName} of counterTypes) {
+      metrics.push({
+        name: `Perf Samples (${counterName})`,
+        unit: '',
+        nameColumnLabel: 'Symbol',
+        dependencySql: `
+          include perfetto module callstacks.stack_profile;
+          include perfetto module linux.perf.counters;
+        `,
+        statement: `
+          select
+            id,
+            parent_id as parentId,
+            name,
+            mapping_name,
+            source_file || ':' || line_number as source_location,
+            self_value as value
+          from _callstacks_for_callsites_weighted!((
+            select p.callsite_id, p.counter_value as value
+            from linux_perf_sample_with_counters p
+            join thread t ON p.utid = t.id
+            join perf_counter_track pct ON p.track_id = pct.id
+            where p.ts >= ${currentSelection.start}
+              and p.ts <= ${currentSelection.end}
+              and pct.name = '${counterName}'
+              and (${trackConstraints})
+          ))
+        `,
+        unaggregatableProperties,
+        aggregatableProperties,
+      });
+    }
+
+    // Add sample count metric (uses existing query)
+    metrics.push({
+      name: 'Perf Samples (Sample Count)',
+      unit: '',
+      nameColumnLabel: 'Symbol',
+      dependencySql: 'include perfetto module linux.perf.samples',
+      statement: `
+        select
+          id,
+          parent_id as parentId,
+          name,
+          mapping_name,
+          source_file || ':' || line_number as source_location,
+          self_count as value
+        from _callstacks_for_callsites!((
+          select p.callsite_id
+          from perf_sample p
+          join thread t using (utid)
+          where p.ts >= ${currentSelection.start}
+            and p.ts <= ${currentSelection.end}
+            and (${trackConstraints})
+        ))
+      `,
+      unaggregatableProperties,
+      aggregatableProperties,
+    });
+
+    const store = assertExists(this.store);
+    store.edit((draft) => {
+      draft.areaSelectionFlamegraphState = Flamegraph.updateState(
+        draft.areaSelectionFlamegraphState,
+        metrics,
+      );
+    });
+    return {flamegraph: new QueryFlamegraph(trace), metrics};
+  }
+
+  // Get counter types for the given session IDs from the pre-populated cache.
+  private getCounterTypesForSessions(sessionIds: number[]): {name: string}[] {
+    if (sessionIds.length === 0) {
+      return [];
+    }
+
+    // Collect unique counter names across all sessions, preserving order
+    // (timebase counters first, then alphabetical).
+    const seen = new Set<string>();
+    const counterTypes: {name: string}[] = [];
+
+    for (const sessionId of sessionIds) {
+      const counters = this.counterTypesBySession.get(sessionId) ?? [];
+      for (const counter of counters) {
+        if (!seen.has(counter.name)) {
+          seen.add(counter.name);
+          counterTypes.push(counter);
+        }
+      }
+    }
+    return counterTypes;
   }
 }
 
@@ -335,32 +619,6 @@ async function selectPerfTracksIfSingleProcess(trace: Trace) {
   }
 }
 
-function createAreaSelectionTab(trace: Trace) {
-  let previousSelection: undefined | AreaSelection;
-  let flamegraph: undefined | QueryFlamegraph;
-
-  return {
-    id: 'perf_sample_flamegraph',
-    name: 'Perf sample flamegraph',
-    render(selection: AreaSelection) {
-      const changed =
-        previousSelection === undefined ||
-        !areaSelectionsEqual(previousSelection, selection);
-
-      if (changed) {
-        flamegraph = computePerfSampleFlamegraph(trace, selection);
-        previousSelection = selection;
-      }
-
-      if (flamegraph === undefined) {
-        return undefined;
-      }
-
-      return {isLoading: false, content: flamegraph.render()};
-    },
-  };
-}
-
 function getSelectedProcessTrackTags(currentSelection: AreaSelection) {
   const ret: number[][] = [];
   for (const trackInfo of currentSelection.tracks) {
@@ -393,65 +651,62 @@ function getSelectedThreadTrackTags(currentSelection: AreaSelection) {
   return ret;
 }
 
-function computePerfSampleFlamegraph(
+export function createPerfCallsitesTrack(
   trace: Trace,
-  currentSelection: AreaSelection,
+  uri: string,
+  upid: number | undefined,
+  utid: number | undefined,
+  sessionId: number | undefined,
+  detailsPanelState: FlamegraphState | undefined,
+  onDetailsPanelStateChange: (state: FlamegraphState) => void,
 ) {
-  const processTrackTags = getSelectedProcessTrackTags(currentSelection);
-  const threadTrackTags = getSelectedThreadTrackTags(currentSelection);
-  if (processTrackTags.length === 0 && threadTrackTags.length === 0) {
-    return undefined;
+  const constraints = [];
+  if (upid !== undefined) {
+    constraints.push(`(upid = ${upid})`);
   }
-
-  const trackConstraints = [
-    ...processTrackTags.map(
-      ([upid, sessionId]) =>
-        `(t.upid = ${upid} AND p.perf_session_id = ${sessionId})`,
-    ),
-    ...threadTrackTags.map(
-      ([utid, sessionId]) =>
-        `(p.utid = ${utid} AND p.perf_session_id = ${sessionId})`,
-    ),
-  ].join(' OR ');
-
-  const metrics = metricsFromTableOrSubquery(
-    `
-      (
-        select
-          id,
-          parent_id as parentId,
-          name,
-          mapping_name,
-          source_file || ':' || line_number as source_location,
-          self_count
-        from _callstacks_for_callsites!((
-          select p.callsite_id
-          from perf_sample p
-          join thread t using (utid)
-          where p.ts >= ${currentSelection.start}
-            and p.ts <= ${currentSelection.end}
-            and (${trackConstraints})
-        ))
-      )
-    `,
-    [
-      {
-        name: 'count',
-        unit: '',
-        columnName: 'self_count',
-      },
-    ],
-    'include perfetto module linux.perf.samples',
-    [{name: 'mapping_name', displayName: 'Mapping'}],
-    [
-      {
-        name: 'source_location',
-        displayName: 'Source location',
-        mergeAggregation: 'ONE_OR_SUMMARY',
-      },
-    ],
+  if (utid !== undefined) {
+    constraints.push(`(utid = ${utid})`);
+  }
+  if (sessionId !== undefined) {
+    constraints.push(`(perf_session_id = ${sessionId})`);
+  }
+  const trackConstraints = constraints.join(' AND ');
+  return createProfilingTrack(
+    trace,
+    uri,
+    {
+      dataset: new SourceDataset({
+        schema: {
+          id: NUM,
+          ts: LONG,
+          callsiteId: NUM,
+        },
+        src: `
+          SELECT
+            p.id,
+            ts,
+            callsite_id AS callsiteId,
+            upid
+          FROM perf_sample AS p
+          JOIN thread USING (utid)
+          WHERE callsite_id IS NOT NULL
+            AND ${trackConstraints}
+          ORDER BY ts
+        `,
+      }),
+      callsiteQuery: (ts) => `
+        SELECT ps.callsite_id
+        FROM perf_sample ps
+        JOIN thread t USING (utid)
+        WHERE ps.ts = ${ts}
+          AND ${trackConstraints}
+      `,
+      sqlModule: 'linux.perf.samples',
+      metricName: 'Perf Samples',
+      panelTitle: 'Perf sample',
+      sliceName: 'Perf sample',
+    },
+    detailsPanelState,
+    onDetailsPanelStateChange,
   );
-  return new QueryFlamegraph(trace, metrics, {
-    state: Flamegraph.createDefaultState(metrics),
-  });
 }
