@@ -27,9 +27,16 @@ import {DisposableStack} from '../../../base/disposable_stack';
 import {CurrentTracingSession, RecordingManager} from '../recording_manager';
 import {download} from '../../../base/download_utils';
 import {RecordSubpage} from '../config/config_interfaces';
-import {RecordPluginSchema} from '../serialization_schema';
+import {RecordPluginSchema, SavedSessionSchema} from '../serialization_schema';
 import {Checkbox} from '../../../widgets/checkbox';
 import {linkify} from '../../../widgets/anchor';
+import {getPresetsForPlatform} from '../presets';
+import {Icons} from '../../../base/semantic_icons';
+import {shareRecordConfig} from '../config/config_sharing';
+import {Card} from '../../../widgets/card';
+import {showModal} from '../../../widgets/modal';
+import {traceConfigToPb} from '../../../base/proto_utils_wasm';
+import protos from '../../../protos';
 
 type RecMgrAttrs = {recMgr: RecordingManager};
 
@@ -37,11 +44,11 @@ export function targetSelectionPage(recMgr: RecordingManager): RecordSubpage {
   return {
     kind: 'GLOBAL_PAGE',
     id: 'target',
-    icon: 'cable',
-    title: 'Target device',
-    subtitle: 'Live recording via USB/WebSocket',
+    icon: 'dashboard',
+    title: 'Overview',
+    subtitle: 'Start a new trace',
     render() {
-      return m(TargetSelectionPage, {recMgr});
+      return m(OverviewPage, {recMgr});
     },
     serialize(state: RecordPluginSchema) {
       state.target = {
@@ -50,43 +57,94 @@ export function targetSelectionPage(recMgr: RecordingManager): RecordSubpage {
         targetId: recMgr.currentTarget?.id,
       };
       state.autoOpenTrace = recMgr.autoOpenTraceWhenTracingEnds;
+      state.selectedConfigId = recMgr.selectedConfigId;
+      state.configModified = recMgr.isConfigModified;
     },
     async deserialize(state: RecordPluginSchema) {
       recMgr.autoOpenTraceWhenTracingEnds = state.autoOpenTrace;
-      if (state.target.platformId === undefined) return;
-      recMgr.setPlatform(state.target.platformId);
+
+      // Restore platform selection
+      if (state.target.platformId !== undefined) {
+        recMgr.setPlatform(state.target.platformId);
+      }
+
+      // Restore config
+      const hasSavedProbes =
+        state.lastSession !== undefined &&
+        state.lastSession.kind === 'probes' &&
+        Object.keys(state.lastSession.probes).length > 0;
+
+      if (state.selectedConfigId || hasSavedProbes) {
+        if (state.selectedConfigId) {
+          recMgr.loadConfig({
+            config: state.lastSession,
+            configId: state.selectedConfigId,
+            configName: recMgr.resolveConfigName(state.selectedConfigId),
+            configModified: state.configModified,
+          });
+        } else {
+          recMgr.loadSession(state.lastSession);
+        }
+      } else {
+        recMgr.loadDefaultConfig();
+      }
+
+      // Restore provider selection
       const prov = recMgr.getProvider(state.target.transportId ?? '');
-      if (prov === undefined) return;
-      await recMgr.setProvider(prov);
-      if (state.target.targetId === undefined) return;
-      for (const target of await recMgr.listTargets()) {
-        if (target.id === state.target.targetId) {
-          await recMgr.setTarget(target);
+      if (prov !== undefined) {
+        await recMgr.setProvider(prov);
+      }
+
+      // Restore target selection
+      if (state.target.targetId !== undefined) {
+        const targets = await recMgr.listTargets();
+        const target = targets.find((t) => t.id === state.target.targetId);
+        if (target) {
+          recMgr.setTarget(target);
         }
       }
     },
   };
 }
 
-class TargetSelectionPage implements m.ClassComponent<RecMgrAttrs> {
+class OverviewPage implements m.ClassComponent<RecMgrAttrs> {
+  private objToId = new ObjToId();
+
   view({attrs}: m.CVnode<RecMgrAttrs>) {
+    const recMgr = attrs.recMgr;
+
     return [
-      m('header', 'Select platform'),
+      m('header', 'Target platform'),
       m(SegmentedButtons, {
         className: 'platform-selector',
         options: TARGET_PLATFORMS.map((p) => ({label: p.name, icon: p.icon})),
         selectedOption: TARGET_PLATFORMS.findIndex(
-          (p) => p.id === attrs.recMgr.currentPlatform,
+          (p) => p.id === recMgr.currentPlatform,
         ),
         onOptionSelected: (num) => {
-          attrs.recMgr.setPlatform(TARGET_PLATFORMS[num].id);
-          // m.redraw();
+          const platformId = TARGET_PLATFORMS[num].id;
+          recMgr.setPlatform(platformId);
+          recMgr.loadDefaultConfig();
         },
       }),
-      [
-        m(TransportSelector, {
-          recMgr: attrs.recMgr,
-          key: attrs.recMgr.currentPlatform,
+
+      m(RecordConfigSelector, {recMgr}),
+
+      m(TransportSelector, {recMgr}),
+
+      recMgr.currentProvider && [
+        m(TargetSelector, {
+          recMgr,
+          provider: recMgr.currentProvider,
+          key: `${this.objToId.getKey(recMgr.currentProvider)}-${recMgr.currentPlatform}`,
+        }),
+      ],
+
+      recMgr.currentTarget && [
+        m(TargetDetails, {
+          recMgr,
+          target: recMgr.currentTarget,
+          key: `${this.objToId.getKey(recMgr.currentTarget)}-${recMgr.currentPlatform}`,
         }),
       ],
     ];
@@ -119,17 +177,235 @@ class TransportSelector implements m.ClassComponent<RecMgrAttrs> {
     return [
       m('header', 'Select transport'),
       m('fieldset.record-transports', ...options),
-      attrs.recMgr.currentProvider && [
-        m(TargetSelector, {
-          recMgr: attrs.recMgr,
-          provider: attrs.recMgr.currentProvider,
-          key: this.transportKeys.getKey(attrs.recMgr.currentProvider),
-        }),
-      ],
     ];
   }
 }
 
+class RecordConfigSelector implements m.ClassComponent<RecMgrAttrs> {
+  view({attrs}: m.CVnode<RecMgrAttrs>) {
+    const recMgr = attrs.recMgr;
+    const presets = getPresetsForPlatform(recMgr.currentPlatform);
+    const isEmptySelected =
+      recMgr.selectedConfigId === undefined &&
+      recMgr.isConfigModified === false &&
+      !recMgr.hasCustomTraceConfig &&
+      !recMgr.recordConfig.hasActiveProbes();
+
+    return [
+      m('header', 'Trace config'),
+      m('.pf-config-selector', [
+        m('h3', 'Quick starts'),
+        m('.pf-config-selector__grid', [
+          ...presets.map((p) =>
+            this.renderCard(
+              p.icon,
+              p.title,
+              p.subtitle,
+              recMgr.selectedConfigId === `preset:${p.id}` &&
+                recMgr.isConfigModified === false,
+              () =>
+                recMgr.loadConfig({
+                  config: p.session,
+                  configId: `preset:${p.id}`,
+                  configName: p.title,
+                }),
+            ),
+          ),
+          this.renderCard(
+            'clear_all',
+            'Empty',
+            'Start fresh',
+            isEmptySelected,
+            () => {
+              recMgr.clearSession();
+              recMgr.clearSelectedConfig();
+            },
+          ),
+        ]),
+        this.renderSavedConfigsSection(recMgr),
+      ]),
+    ];
+  }
+
+  private renderSavedConfigsSection(recMgr: RecordingManager) {
+    const hasActiveProbes = recMgr.recordConfig.hasActiveProbes();
+    const hasUnsavedCustomConfig =
+      recMgr.hasCustomTraceConfig && recMgr.selectedConfigId === undefined;
+    const shouldHighlightSave =
+      hasUnsavedCustomConfig ||
+      (hasActiveProbes && recMgr.selectedConfigId === undefined) ||
+      recMgr.isConfigModified === true;
+    return [
+      m('h3', 'User configs'),
+      m('.pf-config-selector__grid', [
+        // Saved configs
+        ...recMgr.savedConfigs.map((saved) => {
+          const isSelected =
+            recMgr.selectedConfigId === `saved:${saved.name}` &&
+            recMgr.isConfigModified === false;
+          const config = saved.config;
+          const isCustom = config.kind === 'custom';
+          return m(
+            Card,
+            {
+              className:
+                'pf-preset-card' +
+                (isSelected ? ' pf-preset-card--selected' : ''),
+              onclick: () => this.loadSavedConfig(recMgr, saved),
+              tabindex: 0,
+            },
+            m(Icon, {icon: isCustom ? 'description' : 'bookmark'}),
+            m('.pf-preset-card__title', saved.name),
+            isCustom &&
+              m(
+                '.pf-preset-card__subtitle',
+                `Imported from ${config.customConfigFileName ?? 'textproto'}`,
+              ),
+            m('.pf-preset-card__actions', [
+              !isCustom &&
+                m(Button, {
+                  icon: 'save',
+                  compact: true,
+                  title: 'Overwrite with current settings',
+                  onclick: (e: Event) => {
+                    e.stopPropagation();
+                    if (
+                      confirm(
+                        `Overwrite config "${saved.name}" with current settings?`,
+                      )
+                    ) {
+                      recMgr.saveConfig(saved.name);
+                      recMgr.app.raf.scheduleFullRedraw();
+                    }
+                  },
+                }),
+              m(Button, {
+                icon: 'share',
+                compact: true,
+                title: 'Share configuration',
+                onclick: (e: Event) => {
+                  e.stopPropagation();
+                  shareRecordConfig(saved.config);
+                },
+              }),
+              m(Button, {
+                icon: Icons.Delete,
+                compact: true,
+                title: 'Delete configuration',
+                onclick: (e: Event) => {
+                  e.stopPropagation();
+                  if (confirm(`Delete "${saved.name}"?`)) {
+                    recMgr.deleteConfig(saved.name);
+                    recMgr.app.raf.scheduleFullRedraw();
+                  }
+                },
+              }),
+            ]),
+          );
+        }),
+        // Save card - only show when highlighted (custom config)
+        shouldHighlightSave &&
+          m(
+            Card,
+            {
+              className:
+                'pf-preset-card pf-preset-card--dashed pf-preset-card--highlight',
+              onclick: () => {
+                const name = prompt('Enter a name for this configuration:');
+                if (name?.trim()) {
+                  const trimmedName = name.trim();
+                  if (recMgr.savedConfigs.some((s) => s.name === trimmedName)) {
+                    alert(
+                      `A configuration named "${trimmedName}" already exists.`,
+                    );
+                    return;
+                  }
+                  const saved = recMgr.saveConfig(trimmedName);
+                  this.loadSavedConfig(recMgr, saved);
+                  recMgr.app.raf.scheduleFullRedraw();
+                }
+              },
+              tabindex: 0,
+            },
+            m(Icon, {icon: hasUnsavedCustomConfig ? 'description' : 'tune'}),
+            m(
+              '.pf-preset-card__title',
+              hasUnsavedCustomConfig
+                ? recMgr.customConfigFileName ?? 'Imported config'
+                : 'Custom',
+            ),
+            m('.pf-preset-card__subtitle', 'Click to save'),
+          ),
+        this.renderImportCard(recMgr),
+      ]),
+    ];
+  }
+
+  private loadSavedConfig(recMgr: RecordingManager, saved: SavedSessionSchema) {
+    recMgr.loadConfig({
+      config: saved.config,
+      configId: `saved:${saved.name}`,
+      configName: saved.name,
+    });
+  }
+
+  private renderImportCard(recMgr: RecordingManager) {
+    return m(
+      Card,
+      {
+        className: 'pf-preset-card pf-preset-card--dashed',
+        onclick: () => this.openImportDialog(recMgr),
+        tabindex: 0,
+      },
+      m(Icon, {icon: 'upload_file'}),
+      m('.pf-preset-card__title', 'Import'),
+      m('.pf-preset-card__subtitle', 'Load textproto'),
+    );
+  }
+
+  private openImportDialog(recMgr: RecordingManager) {
+    const input = document.createElement('input');
+    input.type = 'file';
+    input.onchange = async () => {
+      const file = input.files?.[0];
+      if (!file) return;
+      const text = await file.text();
+      const res = await traceConfigToPb(text);
+      if (!res.ok) {
+        showModal({
+          title: 'Import error',
+          content: `Failed to parse config: ${res.error}`,
+        });
+        return;
+      }
+      const config = protos.TraceConfig.decode(res.value);
+      recMgr.setCustomTraceConfig(config, file.name);
+    };
+    input.click();
+  }
+
+  private renderCard(
+    icon: string,
+    title: string,
+    subtitle: string,
+    isSelected: boolean,
+    onclick: () => void,
+    extraClass = '',
+  ) {
+    return m(
+      Card,
+      {
+        className: `pf-preset-card${extraClass}${isSelected ? ' pf-preset-card--selected' : ''}`,
+        onclick,
+        tabindex: 0,
+      },
+      m(Icon, {icon}),
+      m('.pf-preset-card__title', title),
+      m('.pf-preset-card__subtitle', subtitle),
+    );
+  }
+}
+
 type TargetSelectorAttrs = {
   recMgr: RecordingManager;
   provider: RecordingTargetProvider;
@@ -172,7 +448,7 @@ class TargetSelector implements m.ClassComponent<TargetSelectorAttrs> {
             onchange: (e: Event) => {
               const idx = (e.target as HTMLSelectElement).selectedIndex;
               recMgr.setTarget(this.targets[idx]);
-              // m.redraw();
+              m.redraw();
             },
           },
           ...this.targets.map((target) =>
@@ -218,13 +494,6 @@ class TargetSelector implements m.ClassComponent<TargetSelectorAttrs> {
             },
           }),
       ),
-      recMgr.currentTarget && [
-        m(TargetDetails, {
-          recMgr: attrs.recMgr,
-          target: recMgr.currentTarget,
-          key: this.targetIdMap.getKey(recMgr.currentTarget),
-        }),
-      ],
     ];
   }
 
