@@ -43,7 +43,7 @@ import {ActiveCPUCountTrack, CPUType} from './active_cpu_count';
 import {uriForSchedTrack} from './common';
 import {CpuSliceByProcessSelectionAggregator} from './cpu_slice_by_process_selection_aggregator';
 import {CpuSliceSelectionAggregator} from './cpu_slice_selection_aggregator';
-import {CpuSliceTrack} from './cpu_slice_track';
+import {createCpuSliceTrack} from './cpu_slice_track';
 import {
   RunnableThreadCountTrack,
   UninterruptibleSleepThreadCountTrack,
@@ -146,6 +146,7 @@ export default class SchedPlugin implements PerfettoPlugin {
         }
         return {
           where: `utid IN (${utids.join()})`,
+          columns: {utid: NUM_NULL},
         };
       },
     });
@@ -153,14 +154,33 @@ export default class SchedPlugin implements PerfettoPlugin {
 
   async addCpuSliceTracks(ctx: Trace, cpus: ReadonlyArray<Cpu>): Promise<void> {
     ctx.selection.registerAreaSelectionTab(
-      createAggregationTab(ctx, new CpuSliceSelectionAggregator()),
+      createAggregationTab(ctx, new CpuSliceSelectionAggregator(ctx)),
     );
     ctx.selection.registerAreaSelectionTab(
-      createAggregationTab(ctx, new CpuSliceByProcessSelectionAggregator()),
+      createAggregationTab(ctx, new CpuSliceByProcessSelectionAggregator(ctx)),
     );
 
     const cpuToClusterType = await this.getAndroidCpuClusterTypes(ctx.engine);
 
+    const table = await createPerfettoTable({
+      engine: ctx.engine,
+      name: 'non_idle_sched_slices',
+      as: `
+        SELECT
+          s.id,
+          s.ts,
+          s.dur,
+          s.utid,
+          IFNULL(t.upid, 0) AS pid,
+          IFNULL(s.priority, 120) AS priority,
+          ucpu,
+          0 as depth
+        FROM sched s
+        LEFT JOIN thread t USING (utid)
+        WHERE NOT s.utid IN (SELECT utid FROM thread WHERE is_idle)
+      `,
+    });
+
     const group = new TrackNode({
       name: 'CPU Scheduling',
       sortOrder: -50,
@@ -195,7 +215,7 @@ export default class SchedPlugin implements PerfettoPlugin {
           kinds: [CPU_SLICE_TRACK_KIND],
           cpu: cpu.ucpu,
         },
-        renderer: new CpuSliceTrack(ctx, uri, cpu.ucpu, threads),
+        renderer: createCpuSliceTrack(ctx, uri, table.name, cpu.ucpu, threads),
       });
       group.addChildInOrder(new TrackNode({name, uri}));
     }
@@ -251,7 +271,7 @@ export default class SchedPlugin implements PerfettoPlugin {
     const {engine} = ctx;
 
     ctx.selection.registerAreaSelectionTab(
-      createAggregationTab(ctx, new ThreadStateSelectionAggregator()),
+      createAggregationTab(ctx, new ThreadStateSelectionAggregator(ctx)),
     );
 
     if (SchedPlugin.threadStateByCpuFlag.get()) {
@@ -468,7 +488,7 @@ export default class SchedPlugin implements PerfettoPlugin {
     const activeCpuCountTitle = 'Active CPU count';
     ctx.tracks.registerTrack({
       uri: activeCpuCountUri,
-      renderer: new ActiveCPUCountTrack({trackUri: activeCpuCountUri}, ctx),
+      renderer: new ActiveCPUCountTrack(activeCpuCountUri, ctx),
     });
     const activeCpuCountTrackNode = new TrackNode({
       name: activeCpuCountTitle,
@@ -486,11 +506,7 @@ export default class SchedPlugin implements PerfettoPlugin {
       const activeCpuTypeCountTitle = `Active CPU count: ${cpuType}`;
       ctx.tracks.registerTrack({
         uri: activeCpuTypeCountUri,
-        renderer: new ActiveCPUCountTrack(
-          {trackUri: activeCpuTypeCountUri},
-          ctx,
-          cpuType,
-        ),
+        renderer: new ActiveCPUCountTrack(activeCpuTypeCountUri, ctx, cpuType),
       });
       const activeCpuTypeCountTrackNode = new TrackNode({
         name: activeCpuTypeCountTitle,
@@ -514,7 +530,7 @@ async function getSchedCpus(ctx: Trace): Promise<Cpu[]> {
   const queryRes = await ctx.engine.query(`
     SELECT DISTINCT
       ucpu,
-      IFNULL(cpu.machine_id, 0) AS machine_id,
+      cpu.machine_id AS machine_id,
       cpu.cpu AS cpu
     FROM sched
     JOIN cpu USING (ucpu)
