@@ -21,17 +21,6 @@ export default class implements PerfettoPlugin {
   static readonly id = 'dev.perfetto.TraceInfoPage';
 
   async onTraceLoad(trace: Trace): Promise<void> {
-    // Create helper functions for accessing metadata
-    await trace.engine.query(`
-      CREATE PERFETTO FUNCTION _metadata_str(key STRING)
-      RETURNS STRING AS
-      SELECT str_value FROM metadata WHERE name = $key;
-
-      CREATE PERFETTO FUNCTION _metadata_int(key STRING)
-      RETURNS LONG AS
-      SELECT int_value FROM metadata WHERE name = $key;
-    `);
-
     trace.pages.registerPage({
       route: '/info',
       render: (subpage) => m(TraceInfoPage, {trace, subpage: subpage}),
@@ -43,12 +32,5 @@ export default class implements PerfettoPlugin {
       icon: 'info',
       sortOrder: 15,
     });
-    trace.sidebar.addMenuItem({
-      section: 'current_trace',
-      text: 'Info and Stats',
-      href: '#!/info/stats',
-      icon: 'query_stats',
-      sortOrder: 23,
-    });
   }
 }
