@@ -20,6 +20,7 @@ import {Checkbox} from '../../../widgets/checkbox';
 import {MenuItem, PopupMenu} from '../../../widgets/menu';
 import {
   Connection,
+  Label,
   Node,
   NodeGraph,
   NodeGraphApi,
@@ -29,6 +30,7 @@ import {
 import {Select} from '../../../widgets/select';
 import {TextInput} from '../../../widgets/text_input';
 import {renderDocSection, renderWidgetShowcase} from '../widgets_page_utils';
+import {Icons} from '../../../base/semantic_icons';
 
 // Base node data interface
 interface BaseNodeData {
@@ -89,6 +91,8 @@ type NodeData =
 interface NodeGraphStore {
   readonly nodes: Map<string, NodeData>;
   readonly connections: Connection[];
+  readonly labels: Label[];
+  readonly invalidNodes: Set<string>; // Track which nodes are marked as invalid
 }
 
 // Node metadata configuration
@@ -422,6 +426,7 @@ interface NodeGraphDemoAttrs {
   readonly accentBars?: boolean;
   readonly colors?: boolean;
   readonly contextMenus?: boolean;
+  readonly contextMenuOnHover?: boolean;
 }
 
 export function NodeGraphDemo(): m.Component<NodeGraphDemoAttrs> {
@@ -432,6 +437,31 @@ export function NodeGraphDemo(): m.Component<NodeGraphDemoAttrs> {
   let store: NodeGraphStore = {
     nodes: new Map([[initialId, createTableNode(initialId, 150, 100)]]),
     connections: [],
+    labels: [
+      {
+        id: uuidv4(),
+        x: 400,
+        y: 100,
+        width: 180,
+        content: m('.pf-simple-label-text', 'Simple text label'),
+      },
+      {
+        id: uuidv4(),
+        x: 400,
+        y: 200,
+        width: 180,
+        content: m(
+          '.pf-simple-label-button',
+          m(Button, {
+            label: 'Click me!',
+            onclick: () => {
+              console.log('Label button clicked!');
+            },
+          }),
+        ),
+      },
+    ],
+    invalidNodes: new Set<string>(),
   };
 
   // History management
@@ -560,12 +590,27 @@ export function NodeGraphDemo(): m.Component<NodeGraphDemoAttrs> {
     console.log(`removeNode: ${nodeId}`);
   };
 
+  const removeLabel = (labelId: string) => {
+    updateStore((draft) => {
+      const labelIndex = draft.labels.findIndex((l) => l.id === labelId);
+      if (labelIndex !== -1) {
+        draft.labels.splice(labelIndex, 1);
+      }
+    });
+
+    // Clear from selection (outside of store update)
+    selectedNodeIds.delete(labelId);
+
+    console.log(`removeLabel: ${labelId}`);
+  };
+
   // Stress test function
   const runStressTest = () => {
     updateStore((draft) => {
       // Clear existing state
       draft.nodes.clear();
       draft.connections.length = 0;
+      draft.invalidNodes.clear();
 
       // Node factory options
       const nodeFactories = [
@@ -797,7 +842,24 @@ export function NodeGraphDemo(): m.Component<NodeGraphDemoAttrs> {
   }
 
   function renderNodeContextMenu(node: NodeData) {
+    const isInvalid = store.invalidNodes.has(node.id);
     return [
+      m(MenuItem, {
+        label: isInvalid ? 'Mark as Valid' : 'Mark as Invalid',
+        icon: isInvalid ? 'check_circle' : 'error',
+        onclick: () => {
+          updateStore((draft) => {
+            if (draft.invalidNodes.has(node.id)) {
+              draft.invalidNodes.delete(node.id);
+            } else {
+              draft.invalidNodes.add(node.id);
+            }
+          });
+          console.log(
+            `Context Menu: Toggle invalid state for ${node.id}: ${!isInvalid}`,
+          );
+        },
+      }),
       m(MenuItem, {
         label: 'Delete',
         icon: 'delete',
@@ -836,7 +898,7 @@ export function NodeGraphDemo(): m.Component<NodeGraphDemoAttrs> {
         return [
           m(MenuItem, {
             label: 'Select',
-            icon: 'filter_alt',
+            icon: Icons.Filter,
             onclick: () => addNode(createSelectNode, toNode),
             style: {
               borderLeft: `4px solid hsl(${NODE_CONFIGS.select.hue}, 60%, 50%)`,
@@ -844,7 +906,7 @@ export function NodeGraphDemo(): m.Component<NodeGraphDemoAttrs> {
           }),
           m(MenuItem, {
             label: 'Filter',
-            icon: 'filter_list',
+            icon: Icons.Filter,
             onclick: () => addNode(createFilterNode, toNode),
             style: {
               borderLeft: `4px solid hsl(${NODE_CONFIGS.filter.hue}, 60%, 50%)`,
@@ -982,6 +1044,7 @@ export function NodeGraphDemo(): m.Component<NodeGraphDemoAttrs> {
           contextMenuItems: attrs.contextMenus
             ? renderNodeContextMenu(nodeData)
             : undefined,
+          invalid: store.invalidNodes.has(nodeData.id),
         };
       }
 
@@ -1014,6 +1077,7 @@ export function NodeGraphDemo(): m.Component<NodeGraphDemoAttrs> {
           contextMenuItems: attrs.contextMenus
             ? renderNodeContextMenu(nodeData)
             : undefined,
+          invalid: store.invalidNodes.has(nodeData.id),
         };
       }
 
@@ -1063,7 +1127,7 @@ export function NodeGraphDemo(): m.Component<NodeGraphDemoAttrs> {
               }),
               m(MenuItem, {
                 label: 'Select',
-                icon: 'filter_alt',
+                icon: Icons.Filter,
                 onclick: () => addNode(createSelectNode),
                 style: {
                   borderLeft: `4px solid hsl(${NODE_CONFIGS.select.hue}, 60%, 50%)`,
@@ -1071,7 +1135,7 @@ export function NodeGraphDemo(): m.Component<NodeGraphDemoAttrs> {
               }),
               m(MenuItem, {
                 label: 'Filter',
-                icon: 'filter_list',
+                icon: Icons.Filter,
                 onclick: () => addNode(createFilterNode),
                 style: {
                   borderLeft: `4px solid hsl(${NODE_CONFIGS.filter.hue}, 60%, 50%)`,
@@ -1123,6 +1187,7 @@ export function NodeGraphDemo(): m.Component<NodeGraphDemoAttrs> {
         connections: store.connections,
         selectedNodeIds: selectedNodeIds,
         multiselect: attrs.multiselect,
+        contextMenuOnHover: attrs.contextMenuOnHover,
         onReady: (api: NodeGraphApi) => {
           graphApi = api;
         },
@@ -1210,6 +1275,30 @@ export function NodeGraphDemo(): m.Component<NodeGraphDemoAttrs> {
             }
           });
         },
+        labels: store.labels,
+        onLabelMove: (labelId: string, x: number, y: number) => {
+          updateStore((draft) => {
+            const label = draft.labels.find((l) => l.id === labelId);
+            if (label) {
+              label.x = x;
+              label.y = y;
+              console.log(`onLabelMove: ${labelId} to (${x}, ${y})`);
+            }
+          });
+        },
+        onLabelResize: (labelId: string, width: number) => {
+          updateStore((draft) => {
+            const label = draft.labels.find((l) => l.id === labelId);
+            if (label) {
+              label.width = width;
+              console.log(`onLabelResize: ${labelId} to width ${width}`);
+            }
+          });
+        },
+        onLabelRemove: (labelId: string) => {
+          removeLabel(labelId);
+          console.log(`onLabelRemove: ${labelId}`);
+        },
       };
 
       return m(NodeGraph, nodeGraphAttrs);
@@ -1236,6 +1325,7 @@ export function renderNodeGraph() {
         titleBars: false,
         colors: true,
         contextMenus: true,
+        contextMenuOnHover: false,
       },
     }),
 
