@@ -19,8 +19,7 @@ export interface CommandManager {
 
   hasCommand(commandId: string): boolean;
 
-  // eslint-disable-next-line @typescript-eslint/no-explicit-any
-  runCommand(id: string, ...args: any[]): any;
+  runCommand(id: string, ...args: unknown[]): unknown;
 }
 
 export interface Command {
@@ -29,8 +28,7 @@ export interface Command {
   // A human-friendly name for this command.
   name: string;
   // Callback is called when the command is invoked.
-  // eslint-disable-next-line @typescript-eslint/no-explicit-any
-  callback: (...args: any[]) => any;
+  callback: (...args: unknown[]) => unknown;
   // Default hotkey for this command.
   // Note: this is just the default and may be changed by the user.
   // Examples:
@@ -39,4 +37,7 @@ export interface Command {
   // - '!Mod+Shift+P'
   // See hotkeys.ts for guidance on hotkey syntax.
   defaultHotkey?: Hotkey;
+  // A human-readable label shown as a left-side chip in the command palette,
+  // indicating where this command came from (e.g. extension module name).
+  source?: string;
 }
