@@ -51,7 +51,7 @@
 import '../base/static_initializers';
 import protobuf from 'protobufjs/minimal';
 import {defer, Deferred} from '../base/deferred';
-import {assertExists, assertFalse, assertTrue} from '../base/logging';
+import {assertExists, assertFalse, assertTrue} from '../base/assert';
 import {utf8Decode} from '../base/string_utils';
 import {Duration, duration, Time, time} from '../base/time';
 
@@ -206,18 +206,20 @@ export function decodeInt64Varint(buf: Uint8Array, pos: number): bigint {
 // plugin etc.
 export interface QueryErrorInfo {
   query: string;
+  tag?: string; // The EngineProxy tag.
 }
 
 export class QueryError extends Error {
-  readonly query: string;
+  readonly queryErrorInfo: QueryErrorInfo;
 
   constructor(message: string, info: QueryErrorInfo) {
     super(message);
-    this.query = info.query;
+    this.queryErrorInfo = info;
   }
 
   toString() {
-    return `${super.toString()}\nQuery:\n${this.query}`;
+    const info = this.queryErrorInfo;
+    return `${super.toString()}\nEngineTag: ${info.tag}\nQuery:\n${info.query}`;
   }
 }
 
@@ -385,6 +387,12 @@ export interface QueryResult {
 
   // Returns the last SQL statement.
   lastStatementSql(): string;
+
+  // Returns the time spend processing the query so far in milliseconds. This is
+  // timed by trace processor to avoid any time spend in the queue. It's updated
+  // every time we get a new batch to convey the time taken do far, so it should
+  // be monotonically increasing and stable when isComplete() is true.
+  elapsedTimeMs(): number;
 }
 
 // Interface exposed to engine.ts to pump in the data as new row batches arrive.
@@ -420,6 +428,7 @@ class QueryResultImpl implements QueryResult, WritableQueryResult {
   private _statementCount = 0;
   private _statementWithOutputCount = 0;
   private _lastStatementSql = '';
+  private _elapsedTimeMs = 0;
 
   constructor(errorInfo: QueryErrorInfo) {
     this._errorInfo = errorInfo;
@@ -465,6 +474,9 @@ class QueryResultImpl implements QueryResult, WritableQueryResult {
   lastStatementSql(): string {
     return this._lastStatementSql;
   }
+  elapsedTimeMs(): number {
+    return this._elapsedTimeMs;
+  }
 
   iter<T extends Row>(spec: T): RowIterator<T> {
     const impl = new RowIteratorImplWithRowData(spec, this);
@@ -587,6 +599,10 @@ class QueryResultImpl implements QueryResult, WritableQueryResult {
           this._lastStatementSql = reader.string();
           break;
 
+        case 7:
+          this._elapsedTimeMs = reader.double();
+          break;
+
         default:
           console.warn(`Unexpected QueryResult field ${tag >>> 3}`);
           reader.skipType(tag & 7);
@@ -1046,6 +1062,9 @@ class WaitableQueryResultImpl
   lastStatementSql() {
     return this.impl.lastStatementSql();
   }
+  elapsedTimeMs() {
+    return this.impl.elapsedTimeMs();
+  }
 
   // WritableQueryResult implementation.
   appendResultBatch(resBytes: Uint8Array) {
