@@ -18,12 +18,26 @@ import {HTMLCheckboxAttrs} from './common';
 
 export interface CheckboxAttrs extends HTMLCheckboxAttrs {
   // Optional text to show to the right of the checkbox.
-  label?: string;
+  readonly label?: m.Children;
+
+  // Optional text to show to the left of the checkbox.
+  readonly labelLeft?: m.Children;
+
+  // Whether to render as a switch instead of a checkbox.
+  readonly variant?: 'checkbox' | 'switch';
 }
 
 export class Checkbox implements m.ClassComponent<CheckboxAttrs> {
   view({attrs}: m.CVnode<CheckboxAttrs>) {
-    const {label, disabled, checked, className, ...htmlAttrs} = attrs;
+    const {
+      label,
+      labelLeft,
+      disabled,
+      checked,
+      className,
+      variant,
+      ...htmlAttrs
+    } = attrs;
     const classes = classNames(disabled && 'pf-disabled', className);
 
     // The default checkbox is removed and an entirely new one created inside
@@ -34,9 +48,15 @@ export class Checkbox implements m.ClassComponent<CheckboxAttrs> {
         ...htmlAttrs,
         className: classes,
       },
+      labelLeft !== undefined && m('span.pf-checkbox__label-left', labelLeft),
       m('input[type=checkbox]', {disabled, checked}),
-      m('span'),
-      label,
+      m('span.pf-checkbox__box', {
+        className:
+          variant === 'switch'
+            ? 'pf-checkbox__box--switch'
+            : 'pf-checkbox__box--check',
+      }),
+      label !== undefined && m('span.pf-checkbox__label', label),
     );
   }
 }
